
/*
 * @(#)Client.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData;

/**
 * Shared Data Client class.
 *
 * @version     0.90 10 February 1997
 * @author      Rich Burridge
 */

public interface
Client {

    /** To signify an object of type Client. */
    public static final String M_Client = "Client";

/**
 * <A NAME="SD_AUTHENTICATE"></A>
 * <EM>authenticate</EM> used to authenticate a client for potentially
 * joining a session. The Session, Channel or Token Manager will be doing
 * this client validation.
 *
 * The manager sends the client a challenge. The client responds.
 * This response is validated by the manager and determines if the
 * client will be allowed to join the session, channel or token.   
 *
 * @param <EM>challenge</EM> the challenge given by the manager to
 * the client.
 *
 * @return the response by the client to the managers challenge.
 */

    public String
    authenticate(String challenge);


/**
 * <A NAME="SD_GETNAME"></A>
 * <EM>getName</EM> get the name of this client.
 * The client name cannot include '\u0001' or newline characters. 
 *
 * @return the name of the client.
 */

    public String
    getName();


/**
 * <A NAME="SD_CLIENTCHANGED"></A>
 * <EM>clientChanged</EM> is called when a client change has been observed.
 *
 * There are six types of session change:
 *   - a client has been invited to join a session.
 *   - a client has been expelled from a session.
 *   - a client has been invited to join a channel.
 *   - a client has been expelled from a channel.
 *   - a client has been invited to join a token.
 *   - a client has been expelled from a token.
 *
 * @param <EM>event</EM> the client changed event containing more information
 * on exactly what has changed.
 *
 * This method could look something like:
 * <pre>
 *
 *    public void
 *    clientChanged(ClientEvent e) {
 *        String  name = e.getName();
 *        int     type = e.getType();
 *
 *        if (type == ClientEvent.SESSION_INVITE) {
 *
 *            // deal with session invite...
 *
 *        } else if (type == ClientEvent.SESSION_EXPEL) {
 *
 *            // deal with session expel...
 *
 *        } else if (type == ClientEvent.CHANNEL_INVITE) {
 *
 *            // deal with channel expel...
 *
 *        } else if (type == ClientEvent.CHANNEL_INVITE) {
 *
 *            // deal with channel leave...
 *
 *        } else if (type == ClientEvent.TOKEN_INVITE) {
 *
 *            // deal with token invite...
 *
 *        } else if (type == ClientEvent.TOKEN_EXPEL) {
 *
 *            // deal with token expel...
 *        }
 *    }
 *
 * </pre>
 */

    public void
    clientChanged(ClientEvent event);
}
