
/*
 * @(#)TokenEvent.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData;

/**
 * Shared Data Token Event class.
 *
 * @version     0.83 9 October 1996
 * @author      Rich Burridge
 */

public class
TokenEvent extends SharedDataEvent {

    /* The various types of token specific event. */
    public static final int GIVE   = 2;
    public static final int PLEASE = 3;

    protected Token token;


/**
 * <A NAME="SD_TOKENEVENT"></A>
 * <EM>TokenEvent</EM> the constructor for the TokenEvent class.
 * A new token event is generated for a client action for a specific token.
 *
 * @param <EM>session</EM> the session this channel belongs to.
 * @param <EM>clientName</EM> the name of the client. 
 * @param <EM>token</EM> the token.
 * @param <EM>type</EM> the type of event.
 */

    public
    TokenEvent(Session session, String clientName, Token token, int type) {
        if (TokenEvent_Debug) {
            System.out.println("TokenEvent: constructor " +
                               " session name: " + session.getName() +
                               " client name: "  + clientName +
                               " token name: "   + token.getName() +
                               " type: "         + type);
        }

        this.session    = session;
        this.clientName = clientName;
        this.token      = token;
        this.type       = type;
    }


/**
 * <A NAME="SD_GETTOKEN"></A>
 * <EM>getToken</EM> get the name of the token for this event.
 *
 * @return the name of the token for this event.
 */
 
    public Token
    getToken() {
        if (TokenEvent_Debug) {
            System.out.println("TokenEvent: getToken: " +
			       " token name: " + token.getName());
        }

        return(token);
    }
}
