
/*
 * @(#)TokenObserver.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData;

/**
 * Shared Data Token Observer interface.
 *
 * @version     0.82 3 October 1996
 * @author      Rich Burridge
 */

public interface
TokenObserver extends SharedDataObserver {

/**
 * <A NAME="SDO_TOKCHANGED"></A>
 * <EM>tokenChanged</EM> is called when a token change has been observed.
 *
 * There are four types of token change:
 * - a client has joined the token.
 * - a client has left the token.
 * - A client wishes to give a token to another client.
 * - a client wishes to grab a token from another client.
 *
 * @param <EM>event</EM> the token changed event containing more information
 * on exactly what has changed.
 *
 * This method could look something like:
 * <pre>
 *
 *    public void
 *    tokenChanged(TokenEvent e) {
 *        Session session    = e.getSession();
 *        String  clientName = e.getClientName();
 *        Token   token      = e.getToken();
 *
 *        if (type == TokenEvent.JOIN) {
 *
 *            // deal with token join...
 *
 *        } else if (type == TokenEvent.LEAVE) {
 *
 *            // deal with token leave...
 *
 *        } else if (type == TokenEvent.GIVE) {
 *
 *            // deal with token give...
 *
 *        } else if (type == TokenEvent.PLEASE) {
 *
 *            // deal with token please...
 *        }
 *    }
 *
 * </pre>
 */

    public void
    tokenChanged(TokenEvent event);
}
