
/*
 * @(#)ChannelConsumerImpl.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import SharedData.*;

/**
 * Shared Data Channel Consumer interface.
 *
 * @version     0.89 28 January 1997
 * @author      Rich Burridge
 */

class
ChannelConsumerImpl extends socketSharedDataObject implements ChannelConsumer {

    /** The name of this channel consumer. */
    protected String name;

    /** The client associated with this consumer. */
    private Client client;


/**
 * <A NAME="SD_CHANNELCONSUMERIMPL"></A>
 * <EM>ChannelConsumerImpl</EM> the constructor for the
 *  ChannelConsumerImpl class.
 *
 * @param <EM>name</EM> the name of the channel consumer being constructed.
 * This name cannot include '\u0001' or newline characters. 
 * @param <EM>client</EM> the client associated with this consumer.
 */

    public
    ChannelConsumerImpl(String name, Client client) {
        if (ChannelConsumerImpl_Debug) {
            System.out.println("ChannelConsumerImpl: constructor:" +
			       " name: "   + name +
			       " client: " + client);
        }

        this.name   = name;
	this.client = client;
    }


/**
 * <A NAME="SD_DATARECEIVED"></A>
 * <EM>dataReceived</EM> is called when data is received for this client on
 * the given channel. This interface is used for normal data and for
 * uniformly sequenced data.
 *
 * The data object received is to a copy of the client data which this
 * consumer can do with as they require.
 *   
 * @param <EM>data</EM> the data which can be of unlimited size.
 */
 
 
    public void
    dataReceived(Data data) {
        if (ChannelConsumerImpl_Debug) {
            System.out.println("ChannelConsumerImpl: dataReceived:" +
                               " data: " + data);
        }
    }


/**
 * <A NAME="SD_GETCLIENT"></A>
 * <EM>getClient</EM> returns a handle to the Client associated with this
 * channel consumer.
 *
 * @return the Client associated with this * channel consumer.
 */


    Client
    getClient() {
        if (ChannelConsumerImpl_Debug) {
            System.out.println("ChannelConsumerImpl: getClient.");
        }

	return(client);
    }
}
