
/*
 * @(#)ChannelProxyMessage.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import SharedData.*;
import java.util.StringTokenizer;

/**
 * Shared Data channel proxy message class.
 *
 * @version     0.94 7 April 1997
 * @author      Rich Burridge
 */

class
ChannelProxyMessage extends SharedDataMessage {

    /** The client-side session associated with this channel thread. */
    private Session session;

    /** The client-side channel associated with this channel thread. */
    private Channel channel;

    /** The data associated with this message. */
    private byte data[];


/**
 * <A NAME="SD_CHANNELPROXYMESSAGE"></A>
 * <EM>ChannelProxyMessage</EM>
 *
 * @param <EM>session</EM>
 * @param <EM>channel</EM>
 * @param <EM>data</EM>
 */

    public
    ChannelProxyMessage(Session session, Channel channel, byte data[]) {
        if (ChannelProxyMessage_Debug) {
            System.out.println("ChannelProxyMessage: constructor:" +
                               " session: " + session +
                               " channel: " + channel +
                               " data: "    + data);
        }
 
        this.session = session;
        this.channel = channel;
        this.data    = data;
    }


/**
 * <A NAME="SD_HANDLEMESSAGE"></A>
 * <EM>handleMessage</EM>
 *
 * @param <EM>tok</EM>
 */

    protected void
    handleMessage(StringTokenizer tok) {
        int    id   = Integer.parseInt(tok.nextToken());
        String type = tok.nextToken();

	if (ChannelProxyMessage_Debug) {
            System.out.println("ChannelProxyMessage: handleMessage:" +
                               " tok: " + tok);
        }

        if (type.equals(Channel.M_Channel)) {
            parseChannelMessage(tok);
        } else {
            System.out.println("ChannelProxyMessage: handleMessage:" +
                               " unknown type: " + type);
        }
    }


/**
 * <A NAME="SD_PARSECHANNELMESSAGE"></A>
 * <EM>parseChannelMessage</EM>
 *
 * @param <EM>tok</EM>
 */

    void
    parseChannelMessage(StringTokenizer tok) {
        String       action      = tok.nextToken();
        String       channelName = tok.nextToken();
        SessionProxy sp          = (SessionProxy) session.po.getProxy();
	Channel      channel     = sp.getChannelByName(channelName);

        if (ChannelProxyMessage_Debug) {
            System.out.println("ChannelProxyMessage: parseChannelMessage:" +
                               " action: "       + action +
			       " channel name: " + channelName);
        }

	if (action.equals(T_DataReceived)) {    /* DATARECEIVED. */
	    ChannelProxy cp     = (ChannelProxy) channel.po.getProxy();
	    String senderName   = tok.nextToken();
	    String receiverName = tok.nextToken();
	    int    priority     = Integer.parseInt(tok.nextToken());
	    int    length       = Integer.parseInt(tok.nextToken());

	    cp.dataReceived(session, channelName, senderName,
                            receiverName, priority, length, data);
	}
    }
}
