
/*
 * @(#)CheckToken.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import SharedData.*;

/**
 * Shared Data Check token status class.
 *
 * @version     0.89 28 January 1997
 * @author      Rich Burridge
 */

class
CheckToken extends socketSharedDataObject implements Runnable {

    /* The token whose status needs checking. */
    private Token token;

    /* The previous token status value for this token. */
    private int previousStatus;


/**
 * <A NAME="SD_CHECKTOKEN"></A>
 * <EM>CheckToken</EM> is a constructor for the CheckToken class. A token
 * has been given from on client to another. We need to make sure the other
 * client accepted it. If not, then we reset it's status from GIVING back to
 * GRABBED or INHIBITED.
 *
 * @param <EM>token</EM> the token whose status needs checking.
 * @param <EM>previousStatus</EM> the previous status for this token.
 */

    public
    CheckToken(Token token, int previousStatus) {
        if (CheckToken_Debug) {
            System.out.println("CheckToken: constructor."); 
        }

        this.token          = token;
	this.previousStatus = previousStatus;
    }


/**
 * <A NAME="SD_RUN"></A>
 * <EM>run</EM> wait for a certain period of time. If the token's status is
 * still giving, then the client didn't accept the token, so reset the status
 * back to it's previous value.
 */

    public void
    run() {
        TokenServer ts = (TokenServer) token.so.getServer();

	if (CheckToken_Debug) {
            System.out.println("CheckToken: run."); 
        }

	try {
	    Thread.sleep(Token.MAX_GIVE_TIME * 1000);
        } catch (Exception e) {
            System.out.println("CheckToken: run: exception " + e);
            if (CheckToken_Debug) {
                e.printStackTrace();
            }
	}

	if (ts.getTokenStatus() == Token.GIVING) {
	    ts.setTokenStatus(previousStatus);
	}
    }
}
