
/*
 * @(#)ManagerProxyThread.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import SharedData.*;
import java.util.StringTokenizer;

/**
 * Shared Data Manager proxy thread class.
 *
 * @version     0.89 27 January 1997
 * @author      Rich Burridge
 */

class
ManagerProxyThread extends TCPSocketThread {

    /** The client-side session associated with this manager. */
    private Session session;

    /** The client-side managable object associated with this thread. */
    private Manageable manageable;

    /** The client-side manager associated with this thread. */
    private SharedDataManager manager;


/**
 * <A NAME="SD_MANAGERPROXYTHREAD"></A>
 * <EM>ManagerProxyThread</EM>
 *
 * @param <EM>session</EM>
 * @param <EM>manageable</EM>
 * @param <EM>manager</EM>
 * @param <EM>host</EM>
 * @param <EM>port</EM>
 */

    public
    ManagerProxyThread(Session session, Manageable manageable,
		       SharedDataManager manager, String host, int port) {
        super(host, port);

	if (ManagerProxyThread_Debug) {
            System.out.println("ManagerProxyThread: constructor:" +
                               " manageable: " + manageable +
                               " manager: "    + manager +
			       " host: "       + host +
			       " port: "       + port);
        }

	this.session    = session;
	this.manageable = manageable;
	this.manager    = manager;
    }


/**
 * <A NAME="SD_HANDLEMESSAGE"></A>
 * <EM>handleMessage</EM>
 *
 * @param <EM>message</EM>
 */

    protected void
    handleMessage(String message) {
        StringTokenizer tok = new StringTokenizer(message, T_Sep, false);

        if (ManagerProxyThread_Debug) {
            System.out.println("ManagerProxyThread: handleMessage:" +
                               " tok: " + tok);
        }
 
        if (validMessageHeader(tok) != null) {
            ManagerProxyMessage mpm = new ManagerProxyMessage(session,
                                                         manager, manageable);
 
            mpm.handleMessage(tok);
        }
    }
}
