
/*
 * @(#)ProxyByteArray.java	1.2 97/05/28
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import SharedData.primitive.SharedByteArrayObserver;
import java.util.Hashtable;

/**
 * Proxy Byte Array class.
 *
 * @version     0.94 7 April 1997
 * @author      Rich Burridge
 */

class
ProxyByteArray extends ByteArray {

/**
 * <A NAME="SD_PROXYBYTEARRAY"></A>
 * <EM>ProxyByteArray</EM> is a constructor for the Proxy byte array class.
 * An array of bytes is turned into a ProxyByteArray object. The length is
 * set to the full length of the byte array.
 *
 * The Proxy byte array class also contains the list of observers interested
 * in changes to this shared byte array.
 *
 * @param <EM>name</EM> the name of the shared byte array being constructed.
 * This name cannot include '\u0001' or newline characters.
 * @param <EM>created</EM> indicates whether this shared byte array should
 * be placed in a created state.
 * @param <EM>value</EM> the value of the byte array.
 */

    ProxyByteArray(String name, boolean created, byte[] value) {
	super(name, created, value);

	if (ProxyByteArray_Debug) {
            System.out.println("ProxyByteArray: constructor:" +
				" name: "    + name +   
				" created: " + created +
				" value: "   + value);
        }

        observers = new Hashtable();
    }


/**
 * <A NAME="SD_SD_PROXYBYTEARRAY"></A>
 * <EM>ProxyByteArray</EM> is a constructor for the Proxy byte array class.
 * An array of bytes is turned into a ProxyByteArray object.
 * The length of the data object is provided. If this value is invalid, then
 * the length of the data object is the length of the array of bytes.
 * 
 * The Proxy byte array class also contains the list of observers interested
 * in changes to this shared byte array.
 *
 * @param <EM>name</EM> the name of the shared byte array being constructed.
 * This name cannot include '\u0001' or newline characters. 
 * @param <EM>created</EM> indicates whether this shared byte array should
 * be placed in a created state.
 * @param <EM>value</EM> the byte array.
 * @param <EM>length</EM> the length of the byte array.
 */
 
    ProxyByteArray(String name, boolean created, byte[] value, int length) {
	super(name, created, value, length);

	if (ProxyByteArray_Debug) {
            System.out.println("ProxyByteArray: constructor:" +
				" name: "   + name +   
				" created: " + created +
				" value: "  + value +
				" length: " + length);  
        }

        observers = new Hashtable();
    }


/**
 * <A NAME="SD_ADDOBSERVER"></A>
 * <EM>addObserver</EM> adds an observer to the list that will be informed
 * when the value of this shared byte array changes.
 *
 * @param <EM>observer</EM> the observer being added.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 */

    public void
    addObserver(SharedByteArrayObserver observer)
				throws SharedData.SharedDataException {
        if (ProxyByteArray_Debug) {
            System.out.println("ProxyByteArray: addObserver: " + observer);
        }

        synchronized (observers) {
            observers.put(observer.getName(), observer);
        }
    }


/**
 * <A NAME="SD_DELETEOBSERVER"></A>
 * <EM>deleteObserver</EM> deletes an observer from the list that will be 
 * informed when the value of this shared byte array changes.
 *
 * @param <EM>observer</EM> the observer being deleted.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 */

    public void
    deleteObserver(SharedByteArrayObserver observer) {
        if (ProxyByteArray_Debug) {
            System.out.println("ProxyByteArray: deleteObserver:" + 
				" observer: " + observer);
        }

        synchronized (observers) {
            observers.remove(observer.getName());
        }
    }
}
