
/*
 * @(#)RegistryServerMessage.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import java.util.StringTokenizer;

/**
 * Shared Data manager proxy message class.
 *
 * @version     0.94 7 April 1997
 * @author      Rich Burridge
 */

class
RegistryServerMessage extends SharedDataMessage {

    /** Handle to server-side registry object. */
    private Registry registry;

    /** Registry server thread associated with this message. */
    private RegistryServerThread thread;


/**
 * <A NAME="SD_REGISTRYSERVERMESSAGE"></A>
 * <EM>RegistryServerMessage</EM>
 *
 * @param <EM>registry</EM>
 * @param <EM>thread</EM>
 * @param <EM>tok</EM>
 */

    public
    RegistryServerMessage(Registry registry, RegistryServerThread thread) {
        if (RegistryServerMessage_Debug) {
            System.out.println("RegistryServerMessage: constructor:" +
                               " registry: " + registry +
                               " thread: "   + thread);
        }
 
        this.registry = registry;
	this.thread   = thread;
    }


/**
 * <A NAME="SD_HANDLEMESSAGE"></A>
 * <EM>handleMessage</EM>
 *
 * @param <EM>tok</EM>
 */

    protected void
    handleMessage(StringTokenizer tok) {
        String action = null;
        String reply  = null;
        String who    = null;

        if (RegistryServerMessage_Debug) {
            System.out.println("RegistryServerMessage: handleMessage:" +
                               " tok: " + tok);
        }
 
	action = tok.nextToken();
        who    = tok.nextToken();

               if (action.equals(T_Bind)) {
            reply = registry.bind(who, tok.nextToken());
        } else if (action.equals(T_Rebind)) {
            reply = registry.rebind(who, tok.nextToken());
        } else if (action.equals(T_Unbind)) {
            reply = registry.unbind(who, tok.nextToken());
        } else if (action.equals(T_Lookup)) {
            reply = registry.lookup(who, tok.nextToken());
        } else if (action.equals(T_List)) {
            reply = registry.list(who);
        }
 
        if (reply != null) {
            thread.writeLine(null, reply);
        }
    }
}
