
/*
 * @(#)SessionProxy.java	1.2 97/05/28
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import SharedData.*;
import SharedData.primitive.*;
import java.util.*;

/**
 * Shared Data Session Client-side class.
 *
 * @version     0.96 23 May 1997
 * @author      Rich Burridge
 */

class
SessionProxy extends SharedDataProxy implements AbstractSessionProxy {

    /** The shared bytes arrays currently created in this session. */
    private Hashtable byteArrays = null;

    /** The channels currently in use by the clients in this session. */
    private Hashtable channels = null;

    /** The tokens currently in use by the clients in this session. */
    private Hashtable tokens = null;

    /** The client currently (potentially) joining this session. */
    Client joiningSessionClient = null;
 
    /** The client currently (potentially) joining a channel. */
    Client joiningChannelClient = null;
 
    /** The client currently (potentially) joining a token. */
    Client joiningTokenClient = null;


/**
 * <A NAME="SD_SESSIONPROXY"></A>
 * <EM>SessionProxy</EM>
 *
 * @param <EM>session</EM>
 * @param <EM>name</EM>
 * The socket session proxy name cannot include '\u0001' or newline characters.
 * @param <EM>host</EM>
 * @param <EM>port</EM>
 */

    public
    SessionProxy(Session session, String name, String host, int port) {
        Thread thread;

        if (SessionProxy_Debug) {
            System.out.println("SessionProxy: constructor:" +
                               " session: " + session +
                               " name: "    + name +
                               " host: "    + host +
                               " port: "    + port);
        }

        this.session = session;
        this.name    = name;

        byteArrays   = new Hashtable();
        channels     = new Hashtable();
        clients      = new Hashtable();
        tokens       = new Hashtable();

	proxyThread = (SocketThread) new SessionProxyThread(session, host, port);
        thread = new Thread(proxyThread);
        thread.start();
    }


    public void
    initProxy(String name, Session session, Object object) {
        if (SessionProxy_Debug) {
            System.out.println("SessionnProxy: initProxy:" +
                               " name: "    + name      +
                               " session: " + session);
        }

        super.initProxy(name, session, object);
    }


    public Object
    getProxy() {
        if (SessionProxy_Debug) {
            System.out.println("SessionProxy: getProxy.");
	}

	return(this);
    }


/**
 * <A NAME="SD_ADDBYTEARRAY"></A>
 * <EM>addByteArray</EM> a new shared byte array has been created. Add it
 * to the known set for this session.
 *
 * @param <EM>byteArrayName</EM> the name of the shared byte array.
 * @param <EM>created</EM> indicates whether this shared byte array should
 * be placed in a created state.
 * @param <EM>value</EM> the value of the shared byte array.
 *
 * @return the newly created proxy shared byte array.
 */

    ProxyByteArray
    addByteArray(String byteArrayName, boolean created, byte[] value) {
        ProxyByteArray pba = new ProxyByteArray(byteArrayName, created, value);

	if (SessionProxy_Debug) {
            System.out.println("SessionProxy: addByteArray:" +
				" name: "    + byteArrayName +
				" created: " + created +
				" value: "   + value);
        }

        synchronized (byteArrays) {
	    byteArrays.put(byteArrayName, pba);
        }
	return(pba);
    }    


/**
 * <A NAME="SD_ADDCHANNEL"></A>
 * <EM>addChannel</EM>
 *
 * @param <EM>channel</EM>
 */

    void
    addChannel(Channel channel) {
        if (SessionProxy_Debug) {
            System.out.println("SessionProxy: addChannel: " + channel);
        }
 
	synchronized (channels) {
            channels.put(channel.getName(), channel);
	}
    }


/**
 * <A NAME="SD_ADDTOKEN"></A>
 * <EM>addToken</EM>
 *
 * @param <EM>token</EM>
 */

    void
    addToken(Token token) {
        if (SessionProxy_Debug) {
            System.out.println("SessionProxy: addToken:" +
			       " token name: " + token.getName());
        }

        synchronized (tokens) {
            tokens.put(token.getName(), token);
	}
    }


    public void
    attachSessionManager(SessionManager sessionManager,
                         Session session) throws SharedDataException {
        if (SessionProxy_Debug) {
            System.out.println("SessionProxy: attachSessionManager:" +
                               " session manager: " + sessionManager +
			       " session: "         + session);
        }

        attachManager(sessionManager, Session.M_Session, session);
    }


/**
 * <A NAME="SD_GETJOININGCLIENT"></A>
 * <EM>getJoiningClient</EM>
 *
 * @param <EM>objectType</EM>
 *
 * @return
 */

    Client
    getJoiningClient(String objectType) {
        if (SessionProxy_Debug) {
            System.out.println("SessionProxy: getJoiningClient:" +
                               " object type: " + objectType);
        }

	       if (objectType.equals(Channel.M_Channel)) {
	    return(joiningChannelClient);
	} else if (objectType.equals(Session.M_Session)) {
	    return(joiningSessionClient);
        } else if (objectType.equals(Token.M_Token)) {
	    return(joiningTokenClient);
	}
	return(null);
    }


/**
 * <A NAME="SD_GETURL"></A>
 * <EM>getUrl</EM>
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 *
 * @return
 */
 
    String
    getUrl() throws SharedDataException {
        int    id           = proxyThread.getId();
        String waitMessage  = addMessageHeader(session.getName(), id,
				       Session.M_Session + T_Sep + T_GetUrl);
        String message      = waitMessage + T_Sep + name;
        StringTokenizer tok = null;
 
        if (SessionProxy_Debug) {
            System.out.println("SessionProxy: getUrl:" +
                               " message: " + message);
        }
 
        tok = sendMessage(proxyThread, waitMessage, message, null, 0);
        return(tok.nextToken());
    }


/**
 * <A NAME="SD_INVITE"></A>
 * <EM>invite</EM>
 *
 * @param <EM>clients</EM>
 * @param <EM>objectType</EM>
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 */

    public void
    invite(Client[] clients, String objectType) throws SharedDataException {
        int    id      = proxyThread.getId();
	String message = addMessageHeader(session.getName(), id,
		     Session.M_Session + T_Sep + T_Invite + T_Sep + getUrl());

        if (SessionProxy_Debug) {
            System.out.println("SessionProxy: invite:" +
                               " clients: " + clients);
        }

        if (manager == null) {
	    throw new SharedDataException(
			SharedDataException.PERMISSION_DENIED);
	}

	if (clients != null) {
	    for (int i = 0; i < clients.length; i++) {
                if (clients[i] instanceof socketClient) {
		    socketClient client = (socketClient) clients[i];

                    client.cc.proxyThread.writeLine(null, message);
 
	        } else {
		    System.out.println("SessionProxy: invite:" +
				       " cannot invite client: " + clients[i]);
	        }
	    }
        }
    }


/**
 * <A NAME="SD_JOINSESSION"></A>
 * <EM>joinSession</EM>
 *
 * @param <EM>client</EM>
 * @param <EM>authenticate</EM>
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 */

    public synchronized void
    joinSession(Client client, boolean authenticate)
					 throws SharedDataException {
        int    id          = proxyThread.getId();
        String clientName  = client.getName();
	String waitMessage = addMessageHeader(session.getName(), id,
			     		Session.M_Session + T_Sep + T_Join);
        String message     = waitMessage + T_Sep + name + T_Sep + clientName;

        if (SessionProxy_Debug) {
            System.out.println("SessionProxy: joinSession: " + 
                                " client: "       + client +
                                " authenticate: " + authenticate);
        }

        joiningSessionClient = client;

	if (authenticate) {
	    if (manager != null) {   /* Client and manager have same proxy? */
		if (((SessionManager) manager).sessionRequest(session,
						client) == false) {
		    throw new 
			SharedDataException(SharedDataException.PERMISSION_DENIED);
	        }
	    } else if (authenticateClient(id, clientName,
					  Session.M_Session, name) == false) {
                throw new 
		    SharedDataException(SharedDataException.PERMISSION_DENIED);
            }
	}

        sendMessage(proxyThread, waitMessage, message, null, 0);
        clients.put(client.getName(), client);
    }


/**
 * <A NAME="SD_JOINCHANNEL"></A>
 * <EM>joinChannel</EM>
 *
 * @param <EM>session</EM> the session this channel belongs to.
 * @param <EM>channelName</EM> the name of the channel to join.
 * @param <EM>client</EM> identifies the client wishing to join this channel.
 * @param <EM>mode</EM> the mode which can be one of, READONLY, WRITEONLY or
 * READWRITE.
 * @param <EM>authenticate</EM> if true, authenticate the client.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 *
 * @return
 */

    public Channel
    joinChannel(Session session, String channelName, Client client,
	        int mode, boolean authenticate) throws SharedDataException {
        int     id          = proxyThread.getId();
        Channel channel     = null;
        boolean reliable    = true;
        String  clientName  = client.getName();
        String  waitMessage = addMessageHeader(session.getName(), id,
                      		       Channel.M_Channel + T_Sep + T_Join);
        String  message     = waitMessage + T_Sep + channelName +
                              T_Sep + clientName + T_Sep + mode;
	StringTokenizer tok = null;
	ChannelProxy    cp  = null;

        if (SessionProxy_Debug) {
            System.out.print("SessionProxy: joinChannel: " + 
                                " session: "      + session +
                                " channel name: " + channelName +
                                " client: "       + client +
                                " mode: "         + mode +
                                " authenticate: " + authenticate);
        }

        joiningChannelClient = client;

        if (authenticate) {
	    if (authenticateClient(id, clientName, Channel.M_Channel,
					       channelName) == false) {
                throw new 
		    SharedDataException(SharedDataException.PERMISSION_DENIED);
	    }
        }

        tok = sendMessage(proxyThread, waitMessage, message, null, 0);

	if ((channel = getChannelByName(channelName)) == null) {
            reliable = (new Boolean(tok.nextToken())).booleanValue();

/* Note that at the moment with just TCP/IP (reliable), and UDP (unreliable)
 * sockets, reliable == ordered.
 */
	    channel = new Channel(channelName, session, reliable, reliable);
            addChannel(channel);
        }

	cp = (ChannelProxy) channel.po.getProxy();
	cp.setChannelThreads();
	cp.join(channel, client, mode);
        cp.setJoiningClient(client);
        return(channel);
    }


/**
 * <A NAME="SD_JOINTOKEN"></A>
 * <EM>joinToken</EM>
 *
 * @param <EM>session</EM> the session this channel belongs to.
 * @param <EM>tokenName</EM> the name of the token to join.
 * @param <EM>client</EM> identifies the client wishing to join this token.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 *
 * @return
 */

    public Token
    joinToken(Session session, String tokenName, Client client)
					   throws SharedDataException {
        int    id          = proxyThread.getId();
        Token  token       = null;
        TokenProxy tp      = null;
	String clientName  = client.getName();
        String waitMessage = addMessageHeader(session.getName(), id,
                             		      Token.M_Token + T_Sep + T_Join);
        String message     = waitMessage + T_Sep + tokenName + T_Sep +
                             clientName;

        if (SessionProxy_Debug) {
            System.out.println("SessionProxy: joinToken: " + 
                                " session: "    + session +
                                " token name: " + tokenName +
                                " client: "     + client);
        }       
                
        joiningTokenClient = client;

        if (authenticateClient(id, clientName, Token.M_Token, 
					tokenName) == false) {
            throw new SharedDataException(SharedDataException.PERMISSION_DENIED);
        }       
                
        sendMessage(proxyThread, waitMessage, message, null, 0);

        if ((token = getTokenByName(tokenName)) == null) {
	    token = new Token(tokenName, session);
        }

	tp = (TokenProxy) token.po.getProxy();
	tp.join(token, client);
        return(token);
    }


    public String[]
    listChannelNames() throws SharedDataException {
	return(listNames(T_ListChannelNames));
    }


    public String[]
    listTokenNames() throws SharedDataException {
        return(listNames(T_ListTokenNames));
    }

    public String[] 
    listByteArrayNames() throws SharedDataException {
        return(listNames(T_ListByteArrayNames)); 
    }


/**
 * <A NAME="SD_LISTNAMES"></A>
 * <EM>listNames</EM> lists the names of all the objects of the given type
 * known to this session.
 *
 * @param <EM>nameType</EM> the type of names to list (Channel, Token or
 * ByteArray).
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 *
 * @return an array of names for this type of session object.
 */

    String[]
    listNames(String nameType) throws SharedDataException {
        int      id          = proxyThread.getId();
        String   waitMessage = addMessageHeader(session.getName(), id,
                                        Session.M_Session + T_Sep + nameType);
        String   message     = waitMessage + T_Sep + name;
        String[] names       = null;
        int      n           = 0;
        StringTokenizer tok  = null;

        if (SessionProxy_Debug) {
            System.out.println("SessionProxy: listNames:" +
			       " type: " + nameType);
        }

        tok = sendMessage(proxyThread, waitMessage, message, null, 0);

        n = Integer.parseInt(tok.nextToken());
        if (n == 0) {
            return((String []) null);
        }

        names = new String[n];
        for (int i = 0; i < n; i++) {
            names[i] = tok.nextToken();
        }
        return(names);
    }


/**
 * <A NAME="SD_ADDBYTEARRAYOBSERVER"></A>
 * <EM>addByteArrayObserver</EM> send a message to the session server to add
 * this observer to the list of observers that will be notified when the value
 * of this shared byte array, in this session changes.
 *
 * Note that if the shared byte array with this name doesn't exist, the
 * observer is notified when/if a shared byte array of that name is created.
 *
 * A reply value is returned from the server. IF it is non-zero, then it
 * indicates the exception type, and a Shared Data exception is thrown.
 *
 * @param <EM>byteArrayName</EM> the name of the shared byte array to observe.
 * @param <EM>observer</EM> the Shared Byte Array observer to add.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 */
 
    public void
    addByteArrayObserver(String byteArrayName,
                SharedByteArrayObserver observer) throws SharedDataException {
	ProxyByteArray pba = null;
        int    id          = proxyThread.getId();
        String waitMessage = addMessageHeader(session.getName(), id,
                                Session.M_Session + T_Sep + T_AddByteObserver);
        String message     = waitMessage + T_Sep + name + T_Sep +
                             byteArrayName + T_Sep + observer.getName();

        if (SessionProxy_Debug) {
            System.out.println("SessionProxy: addByteArrayObserver:" +
                                " byte array name: " + byteArrayName +
                                " observer: "        + observer);
        }
        
        sendMessage(proxyThread, waitMessage, message, null, 0);

        if ((pba = (ProxyByteArray) byteArrays.get(byteArrayName)) == null) {
            pba = addByteArray(byteArrayName, false, null);
	}

	pba.addObserver(observer);
    }


/**
 * <A NAME="SD_CREATEBYTEARRAY"></A>
 * <EM>createByteArray</EM> send a message to the session server to create a
 * new shared byte array with the given name for this session. If a shared
 * byte array of this name doesn't already exist, then it's created in the
 * server.
 *
 * A reply value is returned from the server. IF it is non-zero, then it
 * indicates the exception type, and a Shared Data exception is thrown.
 *
 * The shared byte array is created locally before a message is sent to the
 * server. This is so an observer can be added can be added which can be
 * automatically informed, if this shared byte array already exists, and
 * the value for it, is different than the value being passed across in the
 * message.
 *
 * Note that if the shared byte array already existed in the server and
 * contained a different value than given here, than a change message will
 * be sent back to this session proxy indicating the real value. This in
 * turn is based on to the called of the createByteArray method.
 *
 * @param <EM>client</EM> a client successfully joined to this session.
 * @param <EM>observer</EM> the observer to associate locally with this
 * shared byte array.
 * @param <EM>byteArrayName</EM> the name of the shared byte array to create.
 * @param <EM>value</EM> the value of the byte array.
 * @param <EM>length</EM> the length of the byte array.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 */

    public void
    createByteArray(Client client, SharedByteArrayObserver observer,
		    String byteArrayName, byte[] value,
		    int length) throws SharedData.SharedDataException {
        ProxyByteArray pba = null;
	String waitMessage = addMessageHeader(session.getName(),
                                proxyThread.getId(),
				Session.M_Session + T_Sep + T_CreateByteArray);
        String message     = waitMessage + T_Sep + name + T_Sep +
                             client.getName() + T_Sep + observer.getName() +
			     T_Sep + byteArrayName + T_Sep + length + T_Sep;

        if (SessionProxy_Debug) {
            System.out.println("SessionProxy: createByteArray:" +
                               " message: " + message);
        }
     
        pba = addByteArray(byteArrayName, true, value);
	pba.addObserver(observer);

        sendMessage(proxyThread, waitMessage, message, value, length);
    }


/**
 * <A NAME="SD_CREATECHANNEL"></A>
 * <EM>createChannel</EM> send a message to the session server to create a
 * new channel for this session. A value in returned from the server. If
 * this is non-zero, then it indicates the exception type, and a Shared
 * Data exception is thrown.
 *
 * @param <EM>channelName</EM> the name to give this channel.
 * @param <EM>channelManager</EM> the manager for this channel.
 * @param <EM>reliable</EM> whether the channel is reliable. In other words
 * whether data delivery is guarenteed.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 *
 * @return the channel name.
 */

    public String
    createChannel(String channelName, ChannelManager channelManager,
		  boolean reliable) throws SharedDataException {
	String  waitMessage = addMessageHeader(session.getName(),
				proxyThread.getId(),
		      		Session.M_Session + T_Sep + T_CreateChannel);
	String  message     = waitMessage + T_Sep + name + T_Sep +
			      channelName + T_Sep + reliable;
        Channel channel     = null;

        if (SessionProxy_Debug) {
            System.out.println("SessionProxy: createChannel:" +
			       " message: " + message);
        }

        sendMessage(proxyThread, waitMessage, message, null, 0);

/* Note that at the moment with just TCP/IP (reliable), and UDP (unreliable)
 * sockets, reliable == ordered. 
 */
	channel = new Channel(channelName, session, reliable, reliable);
	addChannel(channel);
        if (channelManager != null) {
	    ChannelProxy cp = (ChannelProxy) channel.po.getProxy();

	    cp.attachManager(channelManager, Channel.M_Channel, channel);
        }

	return(channelName);
    }


/**
 * <A NAME="SD_CREATETOKEN"></A>
 * <EM>createToken</EM>
 *
 * @param <EM>tokenName</EM>
 * @param <EM>tokenManager</EM>
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 *
 * @return
 */

    public String
    createToken(String tokenName, TokenManager tokenManager)
						throws SharedDataException {
	int    id           = proxyThread.getId();
	String waitMessage = addMessageHeader(session.getName(), id,
				Session.M_Session + T_Sep + T_CreateToken);
        String message     = waitMessage + T_Sep + name + T_Sep +
			     tokenName;
        Token  token       = null;

        if (SessionProxy_Debug) {
            System.out.println("SessionProxy: createToken:" +
			       " message: " + message);
        }

        sendMessage(proxyThread, waitMessage, message, null, 0);

        token = new Token(tokenName, session);
	addToken(token);
        if (tokenManager != null) {
	    TokenProxy tp = (TokenProxy) token.po.getProxy();

	    tp.attachManager(tokenManager, Token.M_Token, token);
        }

	return(tokenName);
    }


/**
 * <A NAME="SD_DELETEBYTEARRAYOBSERVER"></A>
 * <EM>deleteByteArrayObserver</EM> send a message to the session server to
 * delete this observer from the list of observers that will be notified when
 * the value of this shared byte array, in this session changes.
 *
 * A reply value is returned from the server. IF it is non-zero, then it
 * indicates the exception type, and a Shared Data exception is thrown.
 *
 * @param <EM>byteArrayName</EM> the name of the shared byte array to observe.
 * @param <EM>observer</EM> the Shared Byte Array observer to delete.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      NO_SUCH_SESSION - session given does not exist in the server.
 *      NO_SUCH_OBSERVER - observer not observing this shared byte array.
 * </PRE>
 */

    public void
    deleteByteArrayObserver(String byteArrayName,
                SharedByteArrayObserver observer) throws SharedDataException {
        ProxyByteArray pba = null;
        int    id          = proxyThread.getId();
        String waitMessage = addMessageHeader(session.getName(), id,
                                Session.M_Session + T_Sep + T_DeleteByteObserver);
        String message     = waitMessage + T_Sep + name + T_Sep +
                             byteArrayName + T_Sep + observer.getName();

        if (SessionProxy_Debug) {
            System.out.println("SessionProxy: deleteByteArrayObserver:" +
                                " byte array name: " + byteArrayName +
                                " observer: "        + observer);
        }
        
        sendMessage(proxyThread, waitMessage, message, null, 0);

        if ((pba = (ProxyByteArray) byteArrays.get(byteArrayName)) != null) {
            pba.deleteObserver(observer);
	}
    }


/**
 * <A NAME="SD_GETCHANNELBYNAME"></A>
 * <EM>getChannelByName</EM>
 *
 * @param <EM>channelName</EM>
 *
 * @return
 */

    Channel
    getChannelByName(String channelName) {
        if (SessionProxy_Debug) {
            System.out.println("SessionProxy: getChannelByName" +
			       " channel name: " + channelName);
        }
 
        if (channels != null) {
	    synchronized (channels) {
                return((Channel) channels.get(channelName));
            }
	}
        return(null);
    }


/**
 * <A NAME="SD_GETCLIENTBYNAME"></A>
 * <EM>getClientByName</EM>
 *
 * @param <EM>clientName</EM>
 *
 * @return
 */

    Client
    getClientByName(String clientName) {
        if (SessionProxy_Debug) {
            System.out.println("SessionProxy: getClientByName:" +
			       " client name: " + clientName);
        }

        if (clients != null) {
	    synchronized (clients) {
                return((Client) clients.get(clientName));
	    }
        }
        return(null);
    }


/**
 * <A NAME="SD_GETTOKENBYNAME"></A>
 * <EM>getTokenByName</EM>
 *
 * @param <EM>tokenName</EM>
 *
 * @return
 */

    Token
    getTokenByName(String tokenName) {
        if (SessionProxy_Debug) {
            System.out.println("SessionProxy: getTokenByName" +
			       " tokenname: " + tokenName);
        }

        if (tokens != null) {
	    synchronized (tokens) {
                return((Token) tokens.get(tokenName));
            }
	}
        return(null);
    }


/**
 * <A NAME="SD_INFORMBYTEARRAYOBSERVER"></A>
 * <EM>informByteArrayObserver</EM> the value of a shared byte array in the
 * server for this session has changed. Inform all the proxy observers.
 *
 * @param <EM>observerName</EM> the name of the observer to be informed that
 * this shared byte array has changed.
 * @param <EM>clientName</EM> the name of the client that made the change.
 * @param <EM>byteArrayName</EM> the name of the shared byte array.
 * @param <EM>type</EM> the types of the change (create, update, destroy).
 * @param <EM>length</EM> the length of the shared byte array.
 * @param <EM>value</EM> the new value for the shared byte array.
 */

    void
    informByteArrayObserver(SharedByteArrayObserver observer,
	                    String clientName, String byteArrayName,
			    int type, int length, byte[] value) {
        if (SessionProxy_Debug) {
            System.out.println("SessionProxy: informByteArrayObserver:" +
				" observer name: "   + observer.getName() +
				" byte array name: " + byteArrayName +
				" type: "            + type +
				" length: "          + length +
				" value: "           + value);
        }

	observer.valueChanged(new SharedByteArrayEvent(session, clientName,
			      byteArrayName, type, value, length));
    }


/**
 * <A NAME="SD_DESTROYBYTEARRAY"></A>
 * <EM>destroyByteArray</EM> send a message to the session server to destroy
 * the shared byte array with the given name for this session.
 *
 * A reply value is returned from the server. IF it is non-zero, then it
 * indicates the exception type, and a Shared Data exception is thrown.
 *
 * All observers of this shared byte array are informed that it has been
 * destroyed.
 *
 * @param <EM>client</EM> a client successfully joined to this session.
 * @param <EM>byteArrayName</EM> the name of the shared byte array to destroy.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      NO_SUCH_SESSION - session given does not exist in the server.
 *      NO_SUCH_CLIENT  - client given does not exist in the server.
 *      NO_SUCH_VALUE   - shared byte array of this name does not exist in
 *                        the server.
 * </PRE>
 */

    public void
    destroyByteArray(Client client,
                     String byteArrayName) throws SharedDataException {
        String waitMessage = addMessageHeader(session.getName(),
                                proxyThread.getId(),
				Session.M_Session + T_Sep + T_DestroyByteArray);
        String message     = waitMessage + T_Sep + name + T_Sep +
                             client.getName() + T_Sep + byteArrayName;

        if (SessionProxy_Debug) {
            System.out.println("Session: destroyByteArray:" +
                                " client name: "     + client.getName() +
                                " byte array name: " + byteArrayName);
        }

        sendMessage(proxyThread, waitMessage, message, null, 0);
    }


/**
 * <A NAME="SD_UPDATEBYTEARRAY"></A>
 * <EM>updateByteArray</EM> send a message to the session server to update a
 * shared byte array with the given name for this session. The session server
 * sends a change notification to all the observers of this shared byte array.
 *
 * A reply value is returned from the server. IF it is non-zero, then it
 * indicates the exception type, and a Shared Data exception is thrown.
 *
 * @param <EM>client</EM> a client successfully joined to this session.
 * @param <EM>byteArrayName</EM> the name of the shared byte array to update.
 * @param <EM>value</EM> the new value of the byte array.
 * @param <EM>length</EM> the length of the byte array.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 */

    public void
    updateByteArray(Client client, String byteArrayName,
		    byte[] value, int length) throws SharedDataException {
        String waitMessage = addMessageHeader(session.getName(),
                                proxyThread.getId(),
				Session.M_Session + T_Sep + T_UpdateByteArray);
        String message     = waitMessage + T_Sep + name + T_Sep +
			     client.getName() + T_Sep + byteArrayName +
			     T_Sep + length + T_Sep;

        if (SessionProxy_Debug) {
            System.out.println("SessionProxy: updateByteArray:" +
                               " message: " + message);
        }
        
	sendMessage(proxyThread, waitMessage, message, value, length);
    }


/**
 * <A NAME="SD_VALIDCHANNEL"></A>
 * <EM>validChannel</EM>
 *
 * @param <EM>channel</EM>
 *
 * @return
 */

    boolean
    validChannel(Channel channel) {
        boolean reply = false;

	if (SessionProxy_Debug) {
            System.out.println("SessionProxy: validChannel:" +
			       " channel: " + channel);
        }

        if (channels != null) {
	    synchronized (channels) {
                reply = channels.contains(channel);
	    }
	}
	return(reply);
    }


/**
 * <A NAME="SD_VALIDTOKEN"></A>
 * <EM>validToken</EM>
 *
 * @param <EM>token</EM>
 *
 * @return
 */

    boolean
    validToken(Token token) {
        boolean reply = false;

	if (SessionProxy_Debug) {
            System.out.println("SessionProxy: validToken:" +
			       " token name: " + token.getName());
        }
 
        if (tokens != null) {
	    synchronized (tokens) {
                reply = tokens.contains(token);
	    }
	}
	return(reply);
    }


/**
 * <A NAME="SD_VALUECHANGED"></A>
 * <EM>valueChanged</EM> the value of a shared byte array in the server for
 * this session has changed. Inform all the proxy observers.
 *
 * @param <EM>observerName</EM> the name of the observer to be informed that
 * this shared byte array has changed.
 * @param <EM>clientName</EM> the name of the client that made the change.
 * @param <EM>byteArrayName</EM> the name of the shared byte array.
 * @param <EM>type</EM> the types of the change (create, update, destroy).
 * @param <EM>length</EM> the length of the shared byte array.
 * @param <EM>value</EM> the new value for the shared byte array.
 *
 * @return
 */

    void
    valueChanged(String observerName, String clientName,
		 String byteArrayName, int type, int length, byte[] value) {
        if (SessionProxy_Debug) {
            System.out.println("SessionProxy: valueChanged:" +
				" observer name: "   + observerName +
				" client name: "     + clientName +
				" byte array name: " + byteArrayName +
				" type: "            + type +
				" length: "          + length +
				" value: "           + value);
        }

        synchronized (byteArrays) {
            ProxyByteArray pba = (ProxyByteArray) byteArrays.get(byteArrayName);
        
            if (pba != null) {
                Hashtable observers = pba.getObservers();

                synchronized (observers) {
                    SharedByteArrayObserver observer =
		      (SharedByteArrayObserver) observers.get(observerName);

		    informByteArrayObserver(observer, clientName,
				        byteArrayName, type, length, value);
                }

/* If the type of change is a destroy, remove the byte array with this name 
 * from the hash table of byte arrays for this session.
 */

		if (type == PrimitiveEvent.DESTROYED) {
		    byteArrays.remove(byteArrayName);
		}
	    }
        }
    }
}
