
/*
 * @(#)SessionServer.java	1.2 97/05/28
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import SharedData.*;
import SharedData.primitive.SharedByteArrayObserver;
import SharedData.primitive.PrimitiveEvent;
import java.util.*;

/**
 * Shared Data Session Server-side class.
 *
 * @version     0.96 23 May 1997
 * @author      Rich Burridge
 */

class
SessionServer extends SharedDataServer implements AbstractSessionServer {

    /** The url associated with this session. */
    String url = null;

    /** The server-side session associated with this session server. */
    protected Session session;

    /** Server socket instance for this session. */
    SocketServer socketServer;

    /** Datagram server socket for unreliable channels. */
    UDPSocketServer datagramServer = null;

    /** The shared bytes arrays currently in use in this session. */
    private Hashtable byteArrays = null;

    /** The channels currently in use by the clients in this session. */
    private Hashtable channels = null;

    /** The tokens currently in use by the clients in this session. */
    private Hashtable tokens = null;


/**
 * <A NAME="SD_SESSIONSERVER"></A>
 * <EM>SessionServer</EM>
 *
 * @param <EM>session</EM>
 * @param <EM>name</EM>
 * The socket session server name cannot include '\u0001' or newline
 * characters.
 * @param <EM>port</EM>
 */

    public
    SessionServer(Session session, String name, int port) {
        Thread thread = null;

        if (SessionServer_Debug) {
            System.out.println("SessionServer: constructor:" +
                               " session: " + session +
                               " name: "    + name +
                               " port: "    + port);
        }

        this.session = session;
        this.name    = name;

	byteArrays   = new Hashtable();
        channels     = new Hashtable();
        clients      = new Hashtable();
        tokens       = new Hashtable();

        socketServer = new TCPSocketServer(port, session);
        thread       = new Thread(socketServer);
        thread.start();
    }


    public void
    initServer(String name, Session session, Object object) {
        if (SessionServer_Debug) {
            System.out.println("SessionnServer: initServer:" +
                               " name: "    + name      +
                               " session: " + session);
        }

        super.initServer(name, session, object);
    }


    public Object
    getServer() {
        if (SessionServer_Debug) {
            System.out.println("SessionServer: getServer.");
        }
 
        return(this);
    }


/**
 * <A NAME="SD_ADDBYTEARRAY"></A>
 * <EM>addByteArray</EM> a new shared byte array has been created. Add it
 * to the known set for this session.
 *
 * @param <EM>byteArrayName</EM> the name of the shared byte array.
 * @param <EM>created</EM> indicates whether this shared byte array should
 * be placed in a created state.
 * @param <EM>value</EM> the value of the shared byte array.
 *
 * @return the newly created server shared byte array.
 */

    ServerByteArray
    addByteArray(String byteArrayName, boolean created, byte[] value) {
        ServerByteArray sba = new ServerByteArray(byteArrayName, created, value);

        if (SessionServer_Debug) {
            System.out.println("SessionServer: addByteArray:" +
                                " name: "    + byteArrayName +
                                " created: " + created +
                                " value: "   + value);
        }

        synchronized (byteArrays) {
            byteArrays.put(byteArrayName, sba);
        }
        return(sba);
    }


/**
 * <A NAME="SD_GETCHANNELBYNAME"></A>
 * <EM>getChannelByName</EM>
 *
 * @param <EM>channelName</EM>
 *
 * @return
 */

    Channel
    getChannelByName(String channelName) {
        if (SessionServer_Debug) {
            System.out.println("SessionServer: getChannelByName " +
				channelName);
        }
 
        if (channels != null) {
            synchronized (channels) {
		return((Channel) channels.get(channelName));
	    }
        }
        return(null);
    }


/**
 * <A NAME="SD_GETTOKENBYNAME"></A>
 * <EM>getTokenByName</EM>
 *
 * @param <EM>tokenName</EM>
 *
 * @return
 */

    Token
    getTokenByName(String tokenName) {
        if (SessionServer_Debug) {
            System.out.println("SessionServer: getTokenByName " + tokenName);
        }
 
        if (tokens != null) {
            synchronized (tokens) {
		return((Token) tokens.get(tokenName));
	    }
        }
        return(null);
    }


/**
 * <A NAME="SD_GETJOININGTHREAD"></A>
 * <EM>getJoiningThread</EM>
 *
 * @param <EM>clientName</EM>
 * @param <EM>objectType</EM>
 * @param <EM>objectName</EM>
 *
 * @return
 */
 
    synchronized SocketThread
    getJoiningThread(String clientName, String objectType, String objectName) {
        if (SessionServer_Debug) {
            System.out.println("SessionServer: getJoiningThread " +
			       " object type: " + objectType +
			       " object name: " + objectName);
        }

	if (objectType.equals(Channel.M_Channel)) {
	    Channel channel = getChannelByName(objectName);

	    if (channel != null) {
		ChannelServer cs = (ChannelServer) channel.so.getServer();

		return((SocketThread) cs.joiningThreads.get(clientName));
	    }

	} else if (objectType.equals(Session.M_Session)) {

	    return((SocketThread) joiningThreads.get(clientName));

        } else if (objectType.equals(Token.M_Token)) { 
	    Token token = getTokenByName(objectName);

	    if (token != null) {
		TokenServer ts = (TokenServer) token.so.getServer();

		return((SocketThread) ts.joiningThreads.get(clientName));
	    }
	}
	return(null);
    }


/**
 * <A NAME="SD_ADDBYTEARRAYOBSERVER"></A>
 * <EM>addByteArrayObserver</EM> adds an observer to the list of observers
 * that will be notified when the value of this shared byte array, in this
 * session changes. A message is returned to the calling proxy, to indicate
 * the success or failure of this operation.
 *
 * @param <EM>id</EM> the id to use for client/server messages.
 * @param <EM>thread</EM> the thread connection back to the proxy.
 * @param <EM>byteArrayName</EM> the name of the shared byte array to observe.
 * @param <EM>observer</EM> the Shared Byte Array observer to add.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 */
 
    String
    addByteArrayObserver(int id, SocketThread thread, String byteArrayName,
                         String observerName) {
        ServerByteArray sba = null;
	SharedByteArrayObserver observer =
			 new SharedByteArrayObserverImpl(observerName);

        if (SessionServer_Debug) {
            System.out.println("SessionServer: addByteArrayObserver:" +
                                " id: "              + id +
                                " thread: "          + thread +
                                " byte array name: " + byteArrayName +
                                " observer name: "   + observerName);
        }

	if ((sba = (ServerByteArray) byteArrays.get(byteArrayName)) == null) {
            sba = addByteArray(byteArrayName, false, null);
        }

	sba.addObserver(thread, observer);

        return(addMessageHeader(session.getName(), id,
		Session.M_Session + T_Sep + T_AddByteObserver + T_Sep + "0"));
    }


/**
 * <A NAME="SD_CREATEBYTEARRAY"></A>
 * <EM>createByteArray</EM> if the client is joined to this session, a new
 * shared byte array is created in the server for this session. A message
 * is returned to the calling proxy, to indicate the success or failure of
 * this operation.  If a shared byte array with this name already exists,
 * then that one is used and a new one is not created.
 *
 * If a shared byte array with this name already exists and it's value is
 * different than the new one, a change message is sent back to the proxy
 * observer to inform it of the correct value.
 *
 * @param <EM>id</EM> the id to use for client/server messages.
 * @param <EM>thread</EM> the thread connection back to the proxy.
 * @param <EM>clientName</EM> name of the client associated with this session.
 * @param <EM>observerName</EM> name of the observer for this creation.
 * @param <EM>byteArrayName</EM> the name of the shared byte array to create.
 * @param <EM>value</EM> the value of the byte array.
 *
 * @return the message to send back to the proxy.
 */

    String
    createByteArray(int id, SocketThread thread, String clientName,
		    String observerName, String byteArrayName, byte[] value) {
	ServerByteArray sba = null;
	SharedByteArrayObserver observer =
                         new SharedByteArrayObserverImpl(observerName);
        int retval = 0;

        if (SessionServer_Debug) {
            System.out.println("SessionServer: createByteArray:" +
                               " id: "              + id +
                               " client name: "     + clientName +
			       " observer name: "   + observerName +
			       " byte array name: " + byteArrayName +
			       " value: "           + value);
        }

	if (clients == null || clients.containsKey(clientName) == false) {
            retval = SharedDataException.NO_SUCH_CLIENT;
	}

	if (retval == 0) {
	    synchronized (byteArrays) {
	    	sba = (ServerByteArray) byteArrays.get(byteArrayName);

            	if (sba != null) {
  		    int    length       = sba.getLength();
		    byte[] currentValue = sba.getValue();

		    sba.addObserver(thread, observer);

		    for (int i = 0; i < length; i++) {
			if (currentValue[i] != value[i]) {
			    String message = addMessageHeader(
				session.getName(), id, 
				Session.M_Session + T_Sep +
				T_ValueChanged + T_Sep + session.getName() +
				T_Sep + observerName + T_Sep +
                                clientName + T_Sep +
				byteArrayName + T_Sep + PrimitiveEvent.CREATED +
			        T_Sep + length + T_Sep);

			    thread.writeData(null, message,
					     currentValue, length);
			    break;
			}
		    }
            	} else {
		    sba = addByteArray(byteArrayName, true, value);
		    sba.addObserver(thread, observer);
                    informByteArrayObservers(id, sba, clientName, byteArrayName,
                                             value, PrimitiveEvent.CREATED); 
	    	}
	    }
	}

        return(addMessageHeader(session.getName(), id,
	    Session.M_Session + T_Sep + T_CreateByteArray + T_Sep + retval));
    }


/**
 * <A NAME="SD_CREATECHANNEL"></A>
 * <EM>createChannel</EM> creates a new channel in the server for this
 * session. A message is returned to the calling proxy, to indicate the
 * success or failure of this operation.  If a channel with this name
 * already exists, then the return value is non-zero, indicating an
 * exception will be thrown on the other side.
 *
 * @param <EM>id</EM> the id to use for client/server messages.
 * @param <EM>channelName</EM> the name to give this channel.
 * @param <EM>reliable</EM>whether the channel is reliable. In other words
 * whether data delivery is guarenteed.
 *
 * @return the message to send back to the proxy.
 */

    String
    createChannel(int id, String channelName, boolean reliable) {
        int     retval     = 0;
        Channel newChannel = null;

        if (SessionServer_Debug) {
            System.out.println("SessionServer: createChannel:" +
                                " id: "           + id +
                                " channel name: " + channelName +
                                " reliable: "     + reliable);
        }

        synchronized (channels) {
	    if (channels.get(channelName) != null) {
                retval = SharedDataException.CHANNEL_EXISTS;
	    }
        }
     
        if (retval == 0) {

/* Note that at the moment with just TCP/IP (reliable), and UDP (unreliable)
 * sockets, reliable == ordered.
 */
	    newChannel = new Channel(channelName, session, reliable, reliable);
            synchronized (channels) {
		channels.put(channelName, newChannel);
	    }

	    if (datagramServer == null && reliable == false) {
		Thread thread = null;

		datagramServer = new UDPSocketServer(session, newChannel,
						     socketServer.getPort());
		thread         = new Thread(datagramServer);
		thread.start();
	    }
        }
        return(addMessageHeader(session.getName(), id, Session.M_Session +
				T_Sep + T_CreateChannel + T_Sep + retval));
    }


/**
 * <A NAME="SD_CREATETOKEN"></A>
 * <EM>createToken</EM>
 *
 * @param <EM>id</EM> the id to use for client/server messages.
 * @param <EM>tokenName</EM>
 *
 * @return
 */

    String
    createToken(int id, String tokenName) {
        int   retval   = 0;
        Token newToken = null;

        if (SessionServer_Debug) {
            System.out.println("SessionServer: createToken:" +
                                " id: "         + id +
			        " token name: " + tokenName);
        }

        synchronized (tokens) {
	    if (tokens.get(tokenName) != null) {
                retval = SharedDataException.TOKEN_EXISTS;
	    }
        }

        if (retval == 0) {
            newToken = new Token(tokenName, session);
            synchronized (tokens) {
		tokens.put(tokenName, newToken);
	    }
        }
        return(addMessageHeader(session.getName(), id,
       		Session.M_Session + T_Sep + T_CreateToken + T_Sep + retval));
    }


/**
 * <A NAME="SD_DELETEBYTEARRAYOBSERVER"></A>
 * <EM>deleteByteArrayObserver</EM> deletes an observer from the list of observers
 * that will be notified when the value of this shared byte array, in this
 * session changes. A message is returned to the calling proxy, to indicate
 * the success or failure of this operation.
 *
 * @param <EM>id</EM> the id to use for client/server messages.
 * @param <EM>thread</EM> the thread connection back to the proxy.
 * @param <EM>byteArrayName</EM> the name of the shared byte array to observe.
 * @param <EM>observer</EM> the Shared Byte Array observer to delete.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 */

    String
    deleteByteArrayObserver(int id, SocketThread thread, String byteArrayName,
                            String observerName) {
        ServerByteArray sba = null;
        SharedByteArrayObserver observer =
                         new SharedByteArrayObserverImpl(observerName);

        if (SessionServer_Debug) {
            System.out.println("SessionServer: deleteByteArrayObserver:" +
                                " id: "              + id +
                                " thread: "          + thread +
                                " byte array name: " + byteArrayName +
                                " observer name: "   + observerName);
        }

        if ((sba = (ServerByteArray) byteArrays.get(byteArrayName)) != null) {
            sba.deleteObserver(observer);
	}

        return(addMessageHeader(session.getName(), id,
                Session.M_Session + T_Sep + T_DeleteByteObserver + T_Sep + "0"));
    }


/**
 * <A NAME="SD_DESTROYBYTEARRAY"></A>
 * <EM>destroyByteArray</EM> if the client is joined to this session, the shared 
 * byte array with this name is destroyed in the server for this session. A 
 * message is returned to the calling proxy, to indicate the success or failure 
 * of this operation.
 *
 * All observers of this shared byte array are informed that it has been
 * destroyed.
 *
 * @param <EM>id</EM> the id to use for client/server messages.
 * @param <EM>thread</EM> the thread connection back to the proxy.
 * @param <EM>clientName</EM> name of the client associated with this session.
 * @param <EM>byteArrayName</EM> the name of the shared byte array to destroy.
 *
 * @return the message to send back to the proxy.
 */

    String
    destroyByteArray(int id, SocketThread thread, String clientName,
                     String byteArrayName) {
        int retval = 0;

        if (SessionServer_Debug) {
            System.out.println("SessionServer: createByteArray:" +
                                " id: "              + id +
				" thread: "          + thread +
				" client name: "     + clientName +
				" byte array name: " + byteArrayName);
        }
        
        if (clients == null || clients.containsKey(clientName) == false) {
            retval = SharedDataException.NO_SUCH_CLIENT;
        }
        
        if (retval == 0) {
            synchronized (byteArrays) {
                ServerByteArray sba = (ServerByteArray) 
					byteArrays.get(byteArrayName);
        
                if (sba != null) {
                    byte[] value = sba.getValue();
        
                    informByteArrayObservers(id, sba, clientName, byteArrayName,
                                             value, PrimitiveEvent.DESTROYED); 

                    byteArrays.remove(byteArrayName);
                }
            }
        }
  
        return(addMessageHeader(session.getName(), id,
	    Session.M_Session + T_Sep + T_DestroyByteArray + T_Sep + retval));
    }


/**
 * <A NAME="SD_INFORMBYTEARRAYOBSERVERS"></A>
 * <EM>informByteArrayObserver</EM> the value of a shared byte array in the
 * server for this session has changed, or the shared byte array has just been
 * created or destroyed. Inform all the proxy observers.
 *
 * @param <EM>id</EM> the id to use for client/server messages.
 * @param <EM>sba</EM> the shared byte array whose value has just changed.
 * @param <EM>clientName</EM> name of the client associated with this session.
 * @param <EM>byteArrayName</EM> the name of the shared byte array to update.
 * @param <EM>value</EM> the new value of the byte array.
 * @param <EM>type</EM> the types of the change (create, update, destroy).
 */

    void
    informByteArrayObservers(int id, ServerByteArray sba, String clientName, 
			     String byteArrayName, byte[] value, int type) {
        int length = value.length;
	Hashtable observers = sba.getObservers();
        String message = addMessageHeader(session.getName(), id,
                           Session.M_Session + T_Sep + T_ValueChanged + T_Sep +
                           session.getName() + T_Sep);

        if (SessionServer_Debug) {
            System.out.println("SessionServer: informByteArrayObservers:" +
                                " id: "                + id +
                                " server byte array: " + sba +
                                " client name: "       + clientName +
                                " byte array name: "   + byteArrayName +
                                " value: "             + value +
                                " type: "              + type);
        }

	synchronized (observers) {
	    Enumeration e, k;
        
	    for (e = observers.elements(), k = observers.keys(); 
		 e.hasMoreElements();) {
		SocketThread thread = (SocketThread) e.nextElement();
		SharedByteArrayObserver observer =
                                 (SharedByteArrayObserver) k.nextElement();
		String dataMessage = message + observer.getName() +
                                  T_Sep + clientName + T_Sep + byteArrayName +
                                  T_Sep + type + T_Sep + length + T_Sep;
        
		thread.writeData(null, dataMessage, value, length);
	    }
	}
    }


/**
 * <A NAME="SD_GETURL"></A>
 * <EM>getUrl</EM>
 *
 * @param <EM>id</EM> the id to use for client/server messages.
 *
 * @return
 */

    synchronized String
    getUrl(int id) {
        String reply = addMessageHeader(session.getName(), id,
                   Session.M_Session + T_Sep + T_GetUrl + T_Sep + "0" + T_Sep);

        if (SessionServer_Debug) {
            System.out.println("SessionServer: getUrl.");
        }

	reply += url;
	return(reply);
    }


/**
 * <A NAME="SD_JOIN"></A>
 * <EM>join</EM>
 *
 * @param <EM>id</EM> the id to use for client/server messages.
 * @param <EM>thread</EM>
 * @param <EM>clientName</EM>
 *
 * @return
 */

    String
    join(int id, SocketThread thread, String clientName) {
        int        retval = 0;
        ClientImpl client = new ClientImpl(clientName);

        if (SessionServer_Debug) {
            System.out.println("SessionServer: join: " + 
                                " id: "          + id +
                                " thread: "      + thread +
                                " client name: " + clientName);
        }

	synchronized (clients) {
	    if (clients.containsKey(clientName)) {
		retval = SharedDataException.NAME_IN_USE;
	    } else {
		clients.put(clientName, client);
	    }
	}

        if (retval == 0) {
	    informObservers(session.getName(), clientName,
		            SharedDataEvent.JOIN, Session.M_Session);
	    addClientThreadConnection(clientName, thread);
	}

	return(addMessageHeader(session.getName(), id,
			Session.M_Session + T_Sep + T_Join + T_Sep + retval));
    }


/**
 * <A NAME="SD_LISTNAMES"></A>
 * <EM>listNames</EM> lists the names of all the objects of the given type
 * known to this session.
 *
 * @param <EM>id</EM> the id to use for client/server messages.
 * @param <EM>nameType</EM> the type of names to list (Channel, Token or
 * ByteArray).
 *
 * @return a message to the proxy containing a list of names for this type
 * of session object.
 */

    String
    listNames(int id, String nameType) {
        Hashtable table = null;
	String reply = addMessageHeader(session.getName(), id,
			        Session.M_Session + T_Sep + nameType + 
				T_Sep + "0" + T_Sep);
        int size = 0;

        if (SessionServer_Debug) {
            System.out.println("SessionServer: listNames: " +
                                " id: "        + id +
			        " name type: " + nameType);
        }

        if (nameType.equals(T_ListChannelNames)) {
	    table = channels;
	} else if (nameType.equals(T_ListTokenNames)) {
	    table = tokens;
	} else if (nameType.equals(T_ListByteArrayNames)) {
	    table = byteArrays;
	}

	if (table != null) {
            synchronized (table) {
                size = table.size();
            }
        }

        reply += size;
        if (table != null) {
            synchronized (table) {
                for (Enumeration e = table.keys(); e.hasMoreElements();) {
                    reply += T_Sep + (String) e.nextElement();
                }
            }    
        }
        return(reply);
    }


/**
 * <A NAME="SD_SETURL"></A>
 * <EM>setUrl</EM>
 *
 * @param <EM>url</EM>
 */

    synchronized void
    setUrl(String url) {
        if (SessionServer_Debug) {
            System.out.println("SessionServer: setUrl:" +
                               " url: " + url);
        }

	this.url = url;
    }


/**
 * <A NAME="SD_UPDATEBYTEARRAY"></A>
 * <EM>updateByteArray</EM> updates a shared byte array in the server
 * for this session. A message is returned to the calling proxy, to indicate
 * the success or failure of this operation. The server sends a change
 * notification to all the observers of this shared byte array.
 *
 * @param <EM>id</EM> the id to use for client/server messages.
 * @param <EM>clientName</EM> name of the client associated with this session.
 * @param <EM>byteArrayName</EM> the name of the shared byte array to update.
 * @param <EM>value</EM> the new value of the byte array.
 *
 * @return the message to send back to the proxy.
 */

    String
    updateByteArray(int id, String clientName, 
		    String byteArrayName, byte[] value) {
        int length = value.length;
	int retval = 0;

        if (SessionServer_Debug) {
            System.out.println("SessionServer: updateByteArray:" +
                                " id: "              + id +
				" client name: "     + clientName +
				" byte array name: " + byteArrayName +
				" value: "           + value);
        }

        if (clients == null || clients.containsKey(clientName) == false) {
            retval = SharedDataException.NO_SUCH_CLIENT;
        }

        if (retval == 0) {
            synchronized (byteArrays) {
		ServerByteArray sba = (ServerByteArray)
			    byteArrays.get(byteArrayName);

                if (sba != null) {
		    sba.setValue(value);
		    informByteArrayObservers(id, sba, clientName, byteArrayName, 
					     value, PrimitiveEvent.UPDATED);
		} else {
                    retval = SharedDataException.NO_SUCH_VALUE;
		}
	    }
        }

        return(addMessageHeader(session.getName(), id,
	    Session.M_Session + T_Sep + T_UpdateByteArray + T_Sep + retval));
    }   


/**
 * <A NAME="SD_VALIDCLIENT"></A>
 * <EM>validClient</EM>
 *
 * @param <EM>client</EM>
 *
 * @return
 */

    boolean
    validClient(Client client) {
        if (SessionServer_Debug) {
            System.out.println("SessionServer: validClient:" +
			       " client name: " + client.getName());
        }

        if (clients != null) {
            synchronized (clients) {
	        return(clients.containsKey(client.getName()));
	    }
	}
	return(false);
    }
}
