
/*
 * @(#)SessionServerThread.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import SharedData.*;
import java.net.Socket;
import java.util.*;

/**
 * Shared Data Session Server Thread class.
 *
 * @version     0.94 7 April 1997
 * @author      Rich Burridge
 */
 
class
SessionServerThread extends TCPSocketThread {
 
    /** The Server-side sessions (and their names) for this socket. */
    private static Hashtable sessions = null;


/**
 * <A NAME="SD_SESSIONSERVERTHREAD"></A>
 * <EM>SessionServerThread</EM>
 *
 * @param <EM>socket</EM>
 * @param <EM>session</EM>
 */

    public
    SessionServerThread(Socket socket, Session session) {
        super(socket);

	if (SessionServerThread_Debug) {
            System.out.println("SessionServerThread: constructor:" +
			       " socket: "  + socket +
			       " session: " + session);
        }

        if (sessions == null) {
            sessions = new Hashtable();
        }

	if (sessions.get(session.getName()) == null) {
	    sessions.put(session.getName(), session);
	}
    }    


/**
 * <A NAME="SD_MESSAGECONTAINSDATA"></A>
 * <EM>messageContainsData</EM> checks if this is one of the message types
 * that contains data.
 *
 * @param <EM>type</EM>   - the type of this message
 * @param <EM>action</EM> - the action for this message.
 */

    private boolean
    messageContainsData(String type, String action) {
        if (SessionServerThread_Debug) {
            System.out.println("SessionServerThread: messageContainsData: " +
                               " type: "   + type +
                               " action: " + action);
        }

	return((type.equals(Channel.M_Channel) &&
		action.equals(T_Send)) ||
	       (type.equals(Session.M_Session) &&
		action.equals(T_CreateByteArray)) ||
               (type.equals(Session.M_Session) &&
		action.equals(T_UpdateByteArray)));
    }


/**
 * <A NAME="SD_HANDLEMESSAGE"></A>
 * <EM>handleMessage</EM>
 *
 * @param <EM>message</EM>
 */

    protected void
    handleMessage(String message) {
	String sessionName;
	StringTokenizer tok = new StringTokenizer(message, T_Sep, false);
 
        if (SessionServerThread_Debug) {
            System.out.println("SessionServerThread: handleMessage:" +
                               " message: " + message);
        }

	if ((sessionName = validMessageHeader(tok)) != null) {
	    Session session     = (Session) sessions.get(sessionName);
	    SessionServerMessage ssm = new SessionServerMessage(session, this,
                                                                null);

	    ssm.handleMessage(tok);
        }
    }


/**
 * <A NAME="SD_HANDLEMESSAGEANDDATA"></A>
 * <EM>handleMessageAndData</EM>
 *
 * @param <EM>message</EM>
 * @param <EM>data</EM>
 */

    protected void
    handleMessageAndData(String message, byte data[]) {
        String sessionName;
	StringTokenizer tok = new StringTokenizer(message, T_Sep, false);

        if (SessionServerThread_Debug) {
            System.out.println("SessionServerThread: handleMessageAndData:" +
                               " message: " + message +
                               " data: "    + data);
        }

        if ((sessionName = validMessageHeader(tok)) != null) {
            Session session     = (Session) sessions.get(sessionName);
            SessionServerMessage ssm = new SessionServerMessage(session, this,
                                                                data);

            ssm.handleMessage(tok);
        }
    }


/**
 * <A NAME="SD__WRITE"></A>
 * <EM>_write</EM> write a message (possibly with data) over the socket.
 *
 * @param <EM>waitCommand</EM> the reply to wait for (if any).
 * @param <EM>message</EM> the message to write to the socket.
 * @param <EM>dataBytes</EM> the data (or null if there isn't any).
 * @param <EM>length</EM> the length (in bytes) of the data.
 *
 * @return the reply (if we were waiting for it), otherwise null.
 */

    synchronized String
    _write(String waitCommand, String message, byte dataBytes[], int length) {
        byte   messageBytes[] = new byte[message.length()];
	String s              = null;

        if (SessionServerThread_Debug) {
            System.out.println("SessionServerThread: _write:" +
                                " waitCommand: " + waitCommand +
                                " message: "     + message   +
                                " data: "        + dataBytes +
                                " length: "      + length);
        }

        try {
	    if (waitCommand != null) {
                while (isWaiting == true) {
                    wait();
                }
                this.waitCommand = waitCommand;
                isWaiting        = true;
            }

            message.getBytes(0, message.length(), messageBytes, 0);
            out.write(messageBytes);
            out.write('\n');
            if (dataBytes != null) {
                out.write(dataBytes, 0, length);
            }
            out.flush();

            while (waitCommand != null && isWaiting == true) {
                s = getMessage();

                if (SessionServerThread_Debug) {
                    System.out.println("SessionServerThread: _write:" +
                                       " about to handle:" + s);
                }

                if (s.startsWith(this.waitCommand)) {
                    System.out.println("SessionServerThread: _write: found: " +
                                       this.waitCommand);
                    isWaiting = false;
                } else {
        	    StringTokenizer tok = new StringTokenizer(s, T_Sep, false);

                    if (validMessageHeader(tok) != null) {
                        int    id     = Integer.parseInt(tok.nextToken());
			String type   = tok.nextToken();
                        String action = tok.nextToken();

		        if (messageContainsData(type, action)) {
                            handleMessageAndData(s, getData());
                        } else {
                            handleMessage(s);
                        }
		    }
		}
            }    
        } catch (Exception e) {
            System.out.println("SessionServerThread: _write: exception " + e);
            if (SessionServerThread_Debug) {
		e.printStackTrace();
	    }
        }
        return(s);
    }


/**
 * <A NAME="SD_PUTMESSAGEANDDATA"></A>
 * <EM>putMessageAndData</EM>
 *
 * @param <EM>s</EM>
 *
 * @exception InterruptedException
 */

    protected void
    putMessageAndData(String s) throws InterruptedException {
        StringTokenizer tok    = new StringTokenizer(s, T_Sep, false);
        byte            data[] = null;

        if (SessionServerThread_Debug) {
            System.out.println("SessionServerThread: putMessageAndData:" +
                               " message: " + s);
        }

        if (validMessageHeader(tok) != null) {

/* Handles the following messages:
 *
 * CHANNEL SEND <channel> <sender> <receiver> <priority> <length> <uniform>
 * SESSION CREATEBYTEARRAY <session> <clientName> <observerName>
 *                         <byteArrayName> <length>
 * SESSION UPDATEBYTEARRAY <session> <clientName> <byteArrayName> <length>
 *
 * Extracts the data length from the message, then read that amount of bytes
 * and generates a second message to put in the message queue.
 */

            String id     = tok.nextToken();     /* strip off message id. */
	    String type   = tok.nextToken();     /* Strip off message type. */
            String action = tok.nextToken();     /* Get message action. */

            if (action.equals(T_Send)) {
                for (int i = 0; i < 4; i++) {
                    tok.nextToken();
                }
            } else if (action.equals(T_CreateByteArray)) {
                for (int i = 0; i < 4; i++) {
                    tok.nextToken();
                }
            } else if (action.equals(T_UpdateByteArray)) {
                for (int i = 0; i < 3; i++) {
                    tok.nextToken();
                }
	    } else {
                System.out.println("SessionServerThread: putMessageAndData:" +
                                   " unexpected message action: " + action);
            }

            data = readBytes(Integer.parseInt(tok.nextToken()));
            putMessage(s);
	    putMessage(new String(data, 0));
	}
    }


/**
 * <A NAME="SD_RUN"></A>
 * <EM>run</EM>
 */

    public void
    run() {
//        System.out.println("Inside run method of SessionServerThread");
	StringTokenizer tok = null;

        if (SessionServerThread_Debug) {
            System.out.println("SessionServerThread: run.");
        }

        try {
            while (true) {
                String s = null;

                if ((s = getSocketMessage()) == null) {
                    break;
                }

                if (SessionServerThread_Debug) {
                    System.out.println("SessionServerThread: run:" +
                        " - received: " + s + " (" + s.length() + " bytes)");
                }

                tok = new StringTokenizer(s, T_Sep, false);
                if (validMessageHeader(tok) != null) {
                    int    id     = Integer.parseInt(tok.nextToken());
		    String type   = tok.nextToken();
                    String action = tok.nextToken();

                    if (messageContainsData(type, action)) {
                        putMessageAndData(s);
                    } else {
                        putMessage(s);
                    }
		}

                if (isWaiting == false) {
                    while (messages.size() > 0) {
                        s = getMessage();

        		if (SessionServerThread_Debug) {
                            System.out.println("SessionServerThread: run:" +
				               " about to handle:" + s);
			}

                        tok = new StringTokenizer(s, T_Sep, false);
                        if (validMessageHeader(tok) != null) {
        		    int    id     = Integer.parseInt(tok.nextToken());
			    String type   = tok.nextToken();
        		    String action = tok.nextToken();

			    if (messageContainsData(type, action)) {
                                handleMessageAndData(s, getData());
 			    } else {
                                handleMessage(s);
                            }
			}
                    }    
                }
            }    
        } catch (Exception e) {
            System.out.println("SessionServerThread: run - exception " + e);
            if (SessionServerThread_Debug) {
		e.printStackTrace();
	    }
        }
    }
}
