
/*
 * @(#)SharedDataMessage.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import SharedData.*;
import java.util.*;

/**
 * Shared Data message class.
 *
 * @version     0.94 6 April 1997
 * @author      Rich Burridge
 */

abstract class
SharedDataMessage extends socketSharedDataObject {

    /** The session associated with this message. */
    protected Session session;

    /** The name of the session associated with this message. */
    protected String sessionName;


/**
 * <A NAME="SD_CLIENTCHALLENGE"></A>
 * <EM>clientChallenge</EM>
 *
 * @param <EM>id</EM> the id to use to client/server messages.
 * @param <EM>sessionName</EM>
 * @param <EM>client</EM>
 * @param <EM>challenge</EM>
 * @param <EM>objectType</EM>
 * @param <EM>objectName</EM>
 *
 * @return
 */

    protected String
    clientChallenge(int id, Client client, String challenge,
		    String objectType, String objectName) {
        SessionProxy sp = (SessionProxy) session.po.getProxy();
	String message  = addMessageHeader(sessionName, id,
				Client.M_Client + T_Sep + T_Challenge + T_Sep +
				objectType + T_Sep + objectName + T_Sep +
				client.getName());
        String response = " ";
        int    retval   = 0;

        if (SharedDataMessage_Debug) {
            System.out.println("SharedDataMessage: clientChallenge:" +
				" id: "          + id +
				" client: "      + client +
				" challenge: "   + challenge +
				" object type: " + objectType +
				" object name: " + objectName);
        }

        response = client.authenticate(challenge);

        message += T_Sep + response;
        sp.proxyThread.writeLine(null, message);
        return(null);
    }


/**
 * <A NAME="SD_HANDLEMESSAGE"></A>
 * <EM>handleMessage</EM>
 *
 * @param <EM>tok</EM>
 */

    protected abstract void
    handleMessage(StringTokenizer tok);


/**
 * <A NAME="SD_INFORMOBSERVERS"></A>
 * <EM>informObservers</EM>
 *
 * @param <EM>clientName</EM>
 * @param <EM>m</EM>
 * @param <EM>type</EM>
 */
 
    void
    informObservers(String clientName, Manageable m, int type) {
	SharedDataProxy sdp = (SharedDataProxy) m.mpo.getProxy();

	if (SharedDataMessage_Debug) {
            System.out.println("SharedDataMessage: informObservers:" +
			       " client name: " + clientName +
			       " object name: " + m.getName() +
			       " type: "        + type);
        }

        if (sdp.observers != null) {
            synchronized (sdp.observers) {
	        for (int i = 0; i < sdp.observers.size(); i++) {
                    SharedDataObserver sdo = (SharedDataObserver)
                                                 sdp.observers.elementAt(i);
 
		    ObserverMessage om = new ObserverMessage(sdo, session,
                                                	clientName, m, type);
		    Thread thread = new Thread(om);

		    thread.start();
		}
	    }
        }
    }
}
