
/*
 * @(#)SharedDataProxy.java	1.2 97/05/28
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import SharedData.*;
import java.util.*;

/**
 * Shared Data client-side proxy parent class.
 *
 * @version     0.96 23 May 1997
 * @author      Rich Burridge
 */

class
SharedDataProxy extends socketSharedDataObject
		implements AbstractSharedDataProxy {

    /** The name of this proxy object. */
    protected String name = null;

    /** Proxy thread for this object. */
    SocketThread proxyThread;

    /** The client-side session associated with this object. */
    protected Session session;

    /** The client-side proxy associated with the Session for this object. */
    protected SessionProxy sp;

    /** Handle to SharedDataProxy for this manageable object. */
    protected SharedDataProxy sdp;

    /** The clients currently joined to this object. */
    protected Hashtable clients = null;

    /** The observers observing changes for this object. */
    protected Vector observers = null;

    /** The client-side manager (if any). */
    protected SharedDataManager manager = null;

    /** Manager thread for this object (if any). */
    private SocketThread managerThread;


    public void
    initProxy(String name, Session session, Object object) {
        if (SharedDataProxy_Debug) {
            System.out.println("SharedDataProxy: initProxy.");
        }

	if (session.po != null) {
	    sp = (SessionProxy) session.po.getProxy();
	}
        sdp = this;
        observers = new Vector();
    }


    public Object
    getProxy() {
        if (SharedDataProxy_Debug) {
            System.out.println("SharedDataProxy: getProxy.");
        }

	return(sdp);
    }


/**
 * <A NAME="SD_ADDOBSERVER"></A>
 * <EM>addObserver</EM>
 *
 * @param <EM>observer</EM>
 * @param <EM>observerType</EM>
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 */

    public void
    addObserver(SharedDataObserver observer, String observerType)
					throws SharedDataException {
        int    id          = sp.proxyThread.getId();
	String waitMessage = addMessageHeader(session.getName(), id,
					observerType + T_Sep + T_AddObserver);
        String message     = waitMessage + T_Sep + name + T_Sep +
			     observer.getName();

        if (SharedDataProxy_Debug) {
            System.out.println("SharedDataProxy: addObserver:" +
                                " observer: " + observer +
                                " type: "     + observerType);
        }

        sendMessage(sp.proxyThread, waitMessage, message, null, 0);

        synchronized (observers) {
	    observers.addElement(observer);
	}
    }


/**
 * <A NAME="SD_ATTACHMANAGER"></A>
 * <EM>attachManager</EM>
 *
 * @param <EM>manager</EM>
 * @param <EM>managerType</EM>
 * @param <EM>manageable</EM>
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 */
 
    public void
    attachManager(SharedDataManager manager, String managerType,
		  Manageable manageable) throws SharedDataException {
        SocketThread managerThread = null;
        int    id          = sp.proxyThread.getId();
        String waitMessage = addMessageHeader(session.getName(), id,
					managerType + T_Sep + T__Manager);
        String message     = waitMessage + T_Sep + name + T_Sep +
			     manager.getName();
	Thread thread      = null;
 
        if (SharedDataProxy_Debug) {
            System.out.println("SharedDataProxy: attachManager:" +
                                " manager: "    + manager +
                                " type: "       + managerType +
                                " manageable: " + manageable);
        }
 
        managerThread = new ManagerProxyThread(session, manageable, manager,
                                        sp.proxyThread.getHost(),
                                        sp.proxyThread.getPort());
	setManagerThread(managerThread);
        thread = new Thread(managerThread);
	thread.start();

        sendMessage(managerThread, waitMessage, message, null, 0);
        setManager(manager);
    }


/**
 * <A NAME="SD_AUTHENTICATECLIENT"></A>
 * <EM>authenticateClient</EM>
 *
 * @param <EM>id</EM> the id to use for client/server messages.
 * @param <EM>clientName</EM>
 * @param <EM>objectType</EM>
 * @param <EM>objectName</EM>
 *
 * @return
 */
 
    boolean
    authenticateClient(int id, String clientName, String objectType,
					  String objectName) {
        String waitMessage  = addMessageHeader(session.getName(), id,
					objectType + T_Sep + T_Authenticate);
        String message      = waitMessage + T_Sep + objectName + T_Sep +
			      clientName;
        String reply        = null;
        StringTokenizer tok = null;

        if (SharedDataProxy_Debug) {
            System.out.println("SharedDataProxy: authenticateClient:" +
                                " id: "          + id +
                                " client name: " + clientName +
                                " object type: " + objectType +
                                " object name: " + objectName);
        }

	reply = sp.proxyThread.writeLine(waitMessage, message);
	tok = new StringTokenizer(reply.substring(waitMessage.length()+1),
				  T_Sep, false);

        if (Integer.parseInt(tok.nextToken()) != 0) {
            return(false);
        }
 
        return(tok.nextToken().equals("true") ? true : false);
    }


/**
 * <A NAME="SD_DELETEOBSERVER"></A>
 * <EM>deleteObserver</EM>
 *
 * @param <EM>observer</EM>
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 */

    public void
    deleteObserver(SharedDataObserver observer, String observerType)
					throws SharedDataException {
        int    id          = sp.proxyThread.getId();
	String waitMessage = addMessageHeader(session.getName(), id,
				observerType + T_Sep + T_DeleteObserver);
        String message     = waitMessage + T_Sep + name + T_Sep +
			     observer.getName();

        if (SharedDataProxy_Debug) {
            System.out.println("SharedDataProxy: deleteObserver:" +
                               " message: " + message);
        }

	sendMessage(sp.proxyThread, waitMessage, message, null, 0);

        if (observers != null) {
	    synchronized (observers) {
	        observers.removeElement(observer);
	    }
	}
    }


    public void
    expel(Client[] clients, String objectType) throws SharedDataException {
        if (SharedDataProxy_Debug) {
            System.out.println("SharedDataProxy expel:"  +
                               " clients: "    + clients +
			       " objectType: " + objectType);
        }

	expelInvite(T_Expel, clients, objectType);
    }


    public void
    invite(Client[] clients, String objectType) throws SharedDataException {
        if (SharedDataProxy_Debug) {
            System.out.println("SharedDataProxy invite:"  +
                               " clients: "    + clients +
                               " objectType: " + objectType); 
        }
 
        expelInvite(T_Invite, clients, objectType);
    }


/**
 * <A NAME="SD_EXPELINVITE"></A>
 * <EM>expelInvite</EM>
 *
 * @param <EM>messageType</EM>
 * @param <EM>clients</EM>
 * @param <EM>objectType</EM>
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 */

    void
    expelInvite(String messageType, Client[] clients,
	        String objectType) throws SharedDataException {
        int    id          = sp.proxyThread.getId();
        String waitMessage = addMessageHeader(session.getName(), id,
					objectType + T_Sep + messageType);
        String message     = waitMessage + T_Sep + name + T_Sep;

        if (SharedDataProxy_Debug) {
            System.out.println("SharedDataProxy expelInvite:" +
                               " message type: " + messageType +
			       " clients: "      + clients);
        }

        if (getManager() == null) {
            throw new SharedDataException(
                                SharedDataException.PERMISSION_DENIED);
        }

        if (clients == null) {
            return;
	}

	message += clients.length;

	for (int i = 0; i < clients.length; i++) {
            message += T_Sep + clients[i].getName();

            sendMessage(sp.proxyThread, waitMessage, message, null, 0);
        }
    }


/**
 * <A NAME="SD_LEAVE"></A>
 * <EM>leave</EM>
 *
 * @param <EM>client</EM>
 * @param <EM>objectType</EM>
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 */

    public void
    leave(Client client, String objectType) throws SharedDataException {
        int    id          = sp.proxyThread.getId();
        String clientName  = client.getName();
	String waitMessage = addMessageHeader(session.getName(), id,
					      objectType + T_Sep + T_Leave);
        String message     = waitMessage + T_Sep + name + T_Sep + clientName;

        if (SharedDataProxy_Debug) {
            System.out.println("SharedDataProxy: leave: " + message);
        }

	sendMessage(sp.proxyThread, waitMessage, message, null, 0);

        synchronized (clients) {
	    clients.remove(clientName);
	}
    }


/**
 * <A NAME="SD_LISTCLIENTNAMES"></A>
 * <EM>listClientNames</EM>
 *
 * @param <EM>objectType</EM>
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 *
 * @return
 */

    public String[]
    listClientNames(String objectType) throws SharedDataException {
        int      id          = sp.proxyThread.getId();
        String   waitMessage = addMessageHeader(session.getName(), id,
				    objectType + T_Sep + T_ListClientNames);
        String   message     = waitMessage + T_Sep + name;
        String[] clientNames = null;
        int      n           = 0;
        StringTokenizer tok  = null;

        if (SharedDataProxy_Debug) {
            System.out.println("SharedDataProxy: listClientNames.");
        }

	tok = sendMessage(sp.proxyThread, waitMessage, message, null, 0);

        n = Integer.parseInt(tok.nextToken());
        if (n == 0) {
            return((String []) null);
        }
        
        clientNames = new String[n];
        for (int i = 0; i < n; i++) {
            clientNames[i] = tok.nextToken();
        }
        return(clientNames);
    }


/**
 * <A NAME="SD_GETMANAGER"></A>
 * <EM>getManager</EM>
 *
 * @return
 */
 
    private synchronized SharedDataManager
    getManager() {
        if (SharedDataProxy_Debug) {
            System.out.println("SharedDataProxy: getManager.");
        }
 
        return(manager);
    }
 
 
/**
 * <A NAME="SD_SETMANAGER"></A>
 * <EM>setManager</EM>
 *
 * @param <EM>manager</EM>
 *
 * @return
 */
 
    private synchronized void
    setManager(SharedDataManager manager) {
        if (SharedDataProxy_Debug) {
            System.out.println("SharedDataProxy: setManager: " +
                               " manager: " + manager);
        }
 
        this.manager = manager;
    }


/**
 * <A NAME="SD_GETMANAGERTHREAD"></A>
 * <EM>getManagerThread</EM>
 *
 * @return
 */

    synchronized SocketThread
    getManagerThread() {
        if (SharedDataProxy_Debug) {
            System.out.println("SharedDataProxy: getManagerThread.");
        }

        return(managerThread);
    }


/**
 * <A NAME="SD_SETMANAGERTHREAD"></A>
 * <EM>setManagerThread</EM>
 *
 * @param <EM>thread</EM>
 *
 * @return
 */
 
    private synchronized void
    setManagerThread(SocketThread thread) {
        if (SharedDataProxy_Debug) {
            System.out.println("SharedDataProxy: setManagerThread: " +
                               " thread: " + thread);
        }
 
        managerThread = thread;
    }
}
