
/*
 * @(#)SharedDataServer.java	1.2 97/05/28
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import SharedData.*;
import java.util.*;

/**
 * Shared Data server-side parent class.
 *
 * @version     0.96 23 May 1997
 * @author      Rich Burridge
 */

class
SharedDataServer extends socketSharedDataObject 
		 implements AbstractSharedDataServer {

    /** The name of this server object. */
    protected String name = null;

    /** The server-side handle to the Session. */
    protected SessionServer ss;

    /** The clients currently attached to this object. */
    protected Hashtable clients = null;

    /** The thread/client-name pairs for clients joined to this object. */
    Hashtable clientConnections = null;

    /** Thread connections back to the proxy for the clients potentially
     *  joining this object.
     */
    Hashtable joiningThreads;

    /** The server-side manager (if any). */
    protected SharedDataManager manager = null;

    /** Thread connection back to the proxy running the manager. */
    private SocketThread managerThread;

    /** The observers and their associated threads. */
    protected Hashtable observerThreads = null;


/**
 * <A NAME="SD_INITSERVER"></A>
 * <EM>initServer</EM> initialise the "server-side" for this object.
 *
 * @param <EM>name</EM> the name of the server object being constructed.
 * This name cannot include '\u0001' or newline characters.
 * @param <EM>session</EM> the server-side session the object belongs to.
 * @param <EM>object</EM> the server-side object.
 */
 
    public void
    initServer(String name, Session session, Object object) {
        if (SharedDataServer_Debug) {
            System.out.println("SharedDataServer: initServer:" +
                               " name: "    + name           +   
                               " session: " + session        +   
                               " object: "  + object);
        }
 
        if (session.so != null) {
	    ss = (SessionServer) session.so.getServer();
	}

        clientConnections = new Hashtable();
        joiningThreads    = new Hashtable();
        observerThreads   = new Hashtable();
    }


    public Object
    getServer() {
        if (SharedDataServer_Debug) {
            System.out.println("SharedDataServer: getServer.");
        }

        System.out.println("SharedDataServer: initServer: " +
                           "Should be over-ridden by it's subclass.");
	return(null);
    }


/**
 * <A NAME="SD_ADDCLIENTTHREADCONNECTION"></A>
 * <EM>addClientThreadConnection</EM>
 *
 * @param <EM>clientName</EM>
 * @param <EM>thread</EM>
 */

    void
    addClientThreadConnection(String clientName, SocketThread thread) {
        if (SharedDataServer_Debug) {
            System.out.println("SharedDataServer:" +
			       " addClientThreadConnection: " +
                               " client name: " + clientName +
                               " thread: "      + thread);
        }

	synchronized (clientConnections) {
	    clientConnections.put(clientName, thread);
	}
    }


/**
 * <A NAME="SD_ADDOBSERVER"></A>
 * <EM>addObserver</EM>
 *
 * @param <EM>id</EM> the id to use for client/server messages.
 * @param <EM>thread</EM>
 * @param <EM>sessionName</EM>
 * @param <EM>observerName</EM>
 * @param <EM>observerType</EM>
 *
 * @return
 */

    protected String
    addObserver(int id, SocketThread thread, String sessionName,
		String observerName, String observerType) {
        SharedData.SharedDataObserver observer =
			new SharedDataObserverImpl(observerName);

        if (SharedDataServer_Debug) {
            System.out.println("SharedDataServer: addObserver:" +
			       " observer name: " + observerName);
        }

        synchronized (observerThreads) {
	    observerThreads.put(observer, thread);
        }

	return(addMessageHeader(sessionName, id,
			observerType + T_Sep + T_AddObserver + T_Sep + "0"));
    }


/**
 * <A NAME="SD_ATTACHMANAGER"></A>
 * <EM>attachManager</EM>
 *
 * @param <EM>id</EM> the id to use for client/server messages.
 * @param <EM>thread</EM>
 * @param <EM>sessionName</EM>
 * @param <EM>managerName</EM>
 * @param <EM>managerType</EM>
 *
 * @return
 */

    protected String
    attachManager(int id, SocketThread thread, String sessionName,
		  String managerName, String managerType) {
        int retval = 0;

        if (SharedDataServer_Debug) {
            System.out.println("SharedDataServer: attachManager:" +
                               " id: "           + id +
                               " thread: "       + thread +
                               " session name: " + sessionName +
			       " manager name: " + managerName +
			       " manager type: " + managerType);
        }

        if (getManager() != null) {
            retval = SharedDataException.MANAGER_EXISTS;
        } else {
            setManager(new SharedDataManagerImpl(managerName));
            if (SharedDataServer_Debug) {
                System.out.println("SharedDataServer: attachManager:" +
                    " created manager with name: " + getManager().getName());
            }
            setManagerThread(thread);
        }

        return(addMessageHeader(sessionName, id,
			managerType + T_Sep + T__Manager + T_Sep + retval));
    }


/**
 * <A NAME="SD_AUTHENTICATECLIENT"></A>
 * <EM>authenticateClient</EM>
 *
 * @param <EM>id</EM> the id to use for client/server messages.
 * @param <EM>thread</EM>
 * @param <EM>sessionName</EM>
 * @param <EM>clientName</EM>
 * @param <EM>objectType</EM>
 */

    protected String
    authenticateClient(int id, SocketThread thread, String sessionName,
		       String clientName, String objectType) {
	String message = addMessageHeader(sessionName, id,
				T_Manager + T_Sep + objectType + T_Sep +
				T_Authenticate + T_Sep + name + T_Sep +
				clientName);
        String reply        = null;
        StringTokenizer tok = null;
 
        if (SharedDataServer_Debug) {
            System.out.println("SharedDataServer: authenticateClient:" +
                               " message: " + message);
        }
 
	if (getManager() != null) {
	    getManagerThread().writeLine(null, message);

            synchronized (joiningThreads) {
		joiningThreads.put(clientName, thread);
	    }

	    reply = null;
	} else {
	    reply = addMessageHeader(sessionName, id,
			objectType + T_Sep + T_Authenticate + T_Sep + "0" +
			T_Sep + "true");
	}
	return(reply);
    }


/**
 * <A NAME="SD_CLIENTCHALLENGE"></A>
 * <EM>clientChallenge</EM>
 *
 * @param <EM>id</EM> the id to use for client/server messages.
 * @param <EM>session</EM>
 * @param <EM>clientName</EM>
 * @param <EM>challenge</EM>
 * @param <EM>objectType</EM>
 * @param <EM>objectName</EM>
 *
 * @return
 */

    protected String
    clientChallenge(int id, Session session, String clientName, String challenge,
		    String objectType, String objectName) {
        String message  = addMessageHeader(session.getName(), id,
			      Client.M_Client + T_Sep + T_Challenge + T_Sep +
			      objectType + T_Sep + objectName + T_Sep +
			      clientName + T_Sep + challenge);
	SocketThread thread = null;

        if (SharedDataServer_Debug) {
            System.out.println("SharedDataServer: clientChallenge:" +
                               " id: "           + id +
                               " session name: " + session.getName() +
			       " client name: "  + clientName +
                               " challenge: "    + challenge +
			       " object type: "  + objectType +
			       " object name: "  + objectName);
        }

	if ((thread = ss.getJoiningThread(clientName, objectType,
						  objectName)) != null) {
            thread.writeLine(null, message);
	}

	return(null);
    }


/**
 * <A NAME="SD_CLIENTRESPONSE"></A>
 * <EM>clientResponse</EM>
 *
 * @param <EM>id</EM> the id to use for client/server messages.
 * @param <EM>sessionName</EM>
 * @param <EM>clientName</EM>
 * @param <EM>response</EM>
 * @param <EM>objectType</EM>
 * @param <EM>objectName</EM>
 *
 * @return
 */

    protected String
    clientResponse(int id, String sessionName, String clientName, 
		   String response, String objectType, String objectName) {
        String message = addMessageHeader(sessionName, id,
			     Client.M_Client + T_Sep + T_Authenticate + T_Sep +
			     objectType + T_Sep + objectName + T_Sep +
			     clientName + T_Sep + "0" + T_Sep + response);

	if (SharedDataServer_Debug) {
            System.out.println("SharedDataServer: clientResponse:" +
                               " id: "           + id +
                               " session name: " + sessionName +
			       " client name: "  + clientName +
                               " response: "     + response +
			       " object type: "  + objectType +
			       " object name: "  + objectName);
        }
        
        getManagerThread().writeLine(null, message);
        return(null);
    }


/**
 * <A NAME="SD_DELETEOBSERVER"></A>
 * <EM>deleteObserver</EM>
 *
 * @param <EM>id</EM> the id to use for client/server messages.
 * @param <EM>sessionName</EM>
 * @param <EM>observerName</EM>
 * @param <EM>observerType</EM>
 *
 * @return
 */

    protected String
    deleteObserver(int id, String sessionName,
		   String observerName, String observerType) {
        int retval = 0;
        SharedDataObserver observer = null;

        if (SharedDataServer_Debug) {
            System.out.println("SharedDataServer: deleteObserver:" +
			       " observer name: " + observerName);
        }

        if (observerThreads == null ||
            (observer = getObserverByName(observerName)) == null) {
            retval = SharedDataException.NO_SUCH_OBSERVER;
        } else {
            synchronized (observerThreads) {
		observerThreads.remove(observer);
	    }
        }
        return(addMessageHeader(sessionName, id,
		   observerType + T_Sep + T_DeleteObserver + T_Sep + retval));
    }


/**
 * <A NAME="SD_EXPEL"></A>
 * <EM>expel</EM>
 *
 * @param <EM>id</EM> the id to use for client/server messages.
 * @param <EM>sessionName</EM>
 * @param <EM>clientNames</EM>
 * @param <EM>objectType</EM>
 * @param <EM>expelType</EM>
 *
 * @return
 */

    protected String
    expel(int id, String sessionName, String[] clientNames,
	  String objectType, String expelType) {
        SocketThread thread  = null;
        int          retval  = 0;
        Client       client  = null;
        String       message = null;

        if (SharedDataServer_Debug) {
            System.out.println("SharedDataServer: expel:" +
                               " id: "           + id +
                               " session name: " + sessionName +
			       " client names: " + clientNames +
                               " object type: "  + objectType +
			       " expel type: "   + expelType);
        }

        for (int i = 0; i < clientNames.length; i++) {
	    if ((thread = getThreadForClient(clientNames[i])) != null) {
                message = addMessageHeader(sessionName, id,
                              Client.M_Client + T_Sep + expelType + T_Sep +
                              name + T_Sep + clientNames[i]);
		thread.writeLine(null, message);

                if (clients != null) {
		    synchronized (clients) {
			if (clients.remove(clientNames[i]) != null) {
                            informObservers(sessionName, clientNames[i],
                                            SharedDataEvent.LEAVE, objectType);
                            removeClientThreadConnection(clientNames[i]);
		        } else {
                            retval = SharedDataException.NO_SUCH_CLIENT;
		        }
		    }
		}
            } else {
                retval = SharedDataException.NO_SUCH_CLIENT;
	    }
        }

        return(addMessageHeader(sessionName, id,
			objectType + T_Sep + T_Expel + T_Sep + retval));
    }


/**
 * <A NAME="SD_GETCLIENTBYNAME"></A>
 * <EM>getClientByName</EM>
 *
 * @param <EM>clientName</EM>
 *
 * @return
 */

    protected Client
    getClientByName(String clientName) {
        if (SharedDataServer_Debug) {
            System.out.println("SharedDataServer: getClientByName:" +
                               " client name: " + clientName);
        }

        if (clients != null) {
            synchronized (clients) {
		return((Client) clients.get(clientName));
	    }
	}
	return(null);
    }


/**
 * <A NAME="SD_GETOBSERVERBYNAME"></A>
 * <EM>getObserverByName</EM>
 *
 * @param <EM>observerName</EM>
 *
 * @return
 */

    private SharedDataObserver
    getObserverByName(String observerName) {
        if (SharedDataServer_Debug) {
            System.out.println("SharedDataServer: getObserverByName:" +
                               " observer name: " + observerName);
        }

        if (observerThreads != null) {
	    synchronized (observerThreads) {
                for (Enumeration e = observerThreads.keys();
			         e.hasMoreElements();) {
                    SharedDataObserver currentObserver =
                                        (SharedDataObserver) e.nextElement();
                    String currentName = currentObserver.getName();

                    if (currentName.equals(observerName)) {
                        return(currentObserver);
                    }
		}
	    }
        }
        return(null);
    }


/**
 * <A NAME="SD_GETTHREADFORCLIENT"></A>
 * <EM>getThreadForClient</EM>
 *
 * @param <EM>clientName</EM>
 *
 * @return
 */

    SocketThread
    getThreadForClient(String clientName) {
        if (SharedDataServer_Debug) {
            System.out.println("SharedDataServer: getThreadForClient:" +
                               " client name: " + clientName);
        }
 
	if (clientConnections != null) {
	    synchronized (clientConnections) {
		return((SocketThread) clientConnections.get(clientName));
	    }
	}
	return(null);
    }


/**
 * <A NAME="SD_INFORMOBSERVERS"></A>
 * <EM>informObservers</EM>
 *
 * @param <EM>sessionName</EM>
 * @param <EM>clientName</EM>
 * @param <EM>type</EM>
 * @param <EM>observerType</EM>
 */

    protected void
    informObservers(String sessionName, String clientName,
		    int type, String observerType) {
        String message = addMessageHeader(sessionName, 0,
			     observerType + T_Sep + T_InformObserver + T_Sep +
                             name + T_Sep + clientName + T_Sep + type);

        if (SharedDataServer_Debug) {
            System.out.println("SharedDataServer: informObservers:" +
			       " message: " + message);
        }

	if (observerThreads != null) {
	    synchronized (observerThreads) {
		for (Enumeration e = observerThreads.elements();
			         e.hasMoreElements();) {
                    SocketThread thread = (SocketThread) e.nextElement();

		    thread.writeLine(null, message);
		}
            }
        }
    }


/**
 * <A NAME="SD_INVITE"></A>
 * <EM>invite</EM>
 *
 * @param <EM>id</EM> the id to use for client/server messages.
 * @param <EM>session</EM>
 * @param <EM>clientNames</EM>
 * @param <EM>objectType</EM>
 * @param <EM>inviteType</EM>
 *
 * @return
 */

    protected String
    invite(int id, Session session, String[] clientNames,
	   String objectType, String inviteType) {
	SocketThread thread      = null;
        String       sessionName = session.getName();
        int          retval      = 0;
        Client       client      = null;
        String       message     = null;

        if (SharedDataServer_Debug) {
            System.out.println("SharedDataServer: invite:" +
                               " id: "           + id +
                               " session name: " + sessionName +
			       " client names: " + clientNames +
                               " object type: "  + objectType +
			       " invite type: "  + inviteType);
        }

        for (int i = 0; i < clientNames.length; i++) {
            thread = ss.getThreadForClient(clientNames[i]);
	    if (thread != null) {
                message = addMessageHeader(sessionName, id,
                              Client.M_Client + T_Sep + inviteType + T_Sep + 
			      name + T_Sep + clientNames[i]);
		thread.writeLine(null, message);
            } else {
                retval = SharedDataException.NO_SUCH_CLIENT;
	    }
        }

        return(addMessageHeader(sessionName, id,
	       		objectType + T_Sep + T_Invite + T_Sep + retval));
    }


/**
 * <A NAME="SD_LEAVE"></A>
 * <EM>leave</EM>
 *
 * @param <EM>id</EM> the id to use for client/server messages.
 * @param <EM>session</EM>
 * @param <EM>clientName</EM>
 * @param <EM>objectType</EM>
 *
 * @return
 */

    protected synchronized String
    leave(int id, Session session, String clientName, String objectType) {
        int retval    = 0;
        Client client = null;

        if (SharedDataServer_Debug) {
            System.out.println("SharedDataServer: leave:" +
                               " session: "     + session +
                               " client name: " + clientName +
			       " object type: " + objectType);
        }

	if (clients != null) {
	    synchronized (clients) {
		if (clients.remove(clientName) != null) {
                    informObservers(session.getName(), clientName,
                                    SharedDataEvent.LEAVE, objectType);
                    removeClientThreadConnection(clientName);
	        } else {
                    retval = SharedDataException.NO_SUCH_CLIENT;
	        }
	    }
	}

        return(addMessageHeader(session.getName(), id,
	       		objectType + T_Sep + T_Leave + T_Sep + retval));
    }


/**
 * <A NAME="SD_LISTCLIENTNAMES"></A>
 * <EM>listClientNames</EM> "server-side" method that provides a list
 *  of all the clients who are currently joined to this object.
 *
 * @param <EM>id</EM> the id to use for client/server messages.
 * @param <EM>sessionName</EM>
 * @param <EM>objectType</EM>
 *
 * @return a String containing the return status plus a list of the clients
 * currently joined to this object.
 */

    protected String
    listClientNames(int id, String sessionName, String objectType) {
        String reply = addMessageHeader(sessionName, id,
		           objectType + T_Sep + T_ListClientNames + T_Sep +
                           "0" + T_Sep);
        int size = 0;
 
        if (SharedDataServer_Debug) {              
            System.out.println("SharedDataServer: listClientNames:" +
				" id: "           + id +
				" session name: " + sessionName +
				" object type: "  + objectType);
        }
 
        if (clients != null) {
            synchronized (clients) {
		size = clients.size();
	    }
        }
 
        reply += size;
        if (clients != null) {
	    synchronized (clients) {
		for (Enumeration e = clients.keys(); e.hasMoreElements();) {
                    reply += T_Sep + (String) e.nextElement();
		}
	    }
        }
        return(reply);
    }


/**
 * <A NAME="SD_REMOVECLIENTTHREADCONNECTION"></A>
 * <EM>removeClientThreadConnection</EM>
 *
 * @param <EM>clientName</EM>
 *
 * @return
 */

    boolean
    removeClientThreadConnection(String clientName) {
        if (SharedDataServer_Debug) {
            System.out.println("SharedDataServer:" +
			       " removeClientThreadConnection: " +
                               " client name: " + clientName);
        }
 
        if (clientConnections != null) {
            synchronized (clientConnections) {
		return(clientConnections.remove(clientName) != null);
	    }
	}
	return(false);
    }


/**
 * <A NAME="SD_GETMANAGER"></A>
 * <EM>getManager</EM>
 *
 * @return
 */
 
    private synchronized SharedDataManager
    getManager() {
        if (SharedDataServer_Debug) {
            System.out.println("SharedDataServer: getManager.");
        }
 
        return(manager);
    }
 
 
/**
 * <A NAME="SD_SETMANAGER"></A>
 * <EM>setManager</EM>
 *
 * @param <EM>manager</EM>
 *
 * @return
 */
 
    private synchronized void
    setManager(SharedData.SharedDataManager manager) {
        if (SharedDataServer_Debug) {
            System.out.println("SharedDataServer: setManager: " +
                               " manager: " + manager);
        }
 
        this.manager = manager;
    }


/**
 * <A NAME="SD_GETMANAGERTHREAD"></A>
 * <EM>getManagerThread</EM>
 *
 * @return
 */

    private synchronized SocketThread
    getManagerThread() {
        if (SharedDataServer_Debug) {
            System.out.println("SharedDataServer: getManagerThread.");
        }

        return(managerThread);
    }


/**
 * <A NAME="SD_SETMANAGERTHREAD"></A>
 * <EM>setManagerThread</EM>
 *
 * @param <EM>thread</EM>
 *
 * @return
 */

    private synchronized void
    setManagerThread(SocketThread thread) {
        if (SharedDataServer_Debug) {
            System.out.println("SharedDataServer: setManagerThread: " +
                               " thread: " + thread);
        }

        managerThread = thread;
    }
}
