
/*
 * @(#)TCPSocketServer.java	1.2 97/05/28
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import SharedData.*;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.*;

/**
 * Shared Data TCP Socket Server class.
 *
 * @version     0.96 23 May 1997
 * @author      Rich Burridge
 */

class
TCPSocketServer extends SocketServer {

    /** The threads currently being run by the server. */
    private Vector connections = null;

    /** The server socket to await connections on. */
    private static Hashtable serverSockets = null;

    /** The server socket associated with this thread. */
    private ServerSocket ssock = null;


/**
 * <A NAME="SD_TCPSOCKETSERVER"></A>
 * <EM>TCPSocketServer</EM>
 *
 * @param <EM>port</EM>
 * @param <EM>object</EM>
 */

    public
    TCPSocketServer(int port, Object object) {
        if (TCPSocketServer_Debug) {
            System.out.println("TCPSocketServer: constructor:" +
                               " port "     + port +
                               " object: "  + object);
        }

        try {
            if (serverSockets == null) {
		serverSockets = new Hashtable();
	    }

	    synchronized (serverSockets) {
		Integer portNo = new Integer(port);

		if ((ssock = (ServerSocket) serverSockets.get(portNo)) == null) {
                    ssock = new ServerSocket(port);
		    serverSockets.put(portNo, ssock);
		}
            }
        } catch (Exception e) {
            System.out.println("TCPSocketServer: exception " + e);
            if (TCPSocketServer_Debug) {
                e.printStackTrace();
            }
        }

        this.port   = port;
	this.object = object;
        connections = new Vector();
    }


/**
 * <A NAME="SD_RUN"></A>
 * <EM>run</EM>
 */

    public void
    run() {
        if (TCPSocketServer_Debug) {
            System.out.println("TCPSocketServer: run.");
        }

        try {
            while (true) {
                Socket sock;
                SocketThread socketThread;
                Thread thread;

                if (TCPSocketServer_Debug) {
                    System.out.println("TCPSocketServer: Waiting...");
                }
                sock = ssock.accept();
                if (TCPSocketServer_Debug) {
                    System.out.println("TCPSocketServer: Got a connection.");
                }

                       if (object instanceof Session) {
                    socketThread = new SessionServerThread(sock,
                                                           (Session) object);
                } else if (object instanceof Client) {
                    socketThread = new ClientServerThread(sock,
                                                          (Client) object);
                } else {
                    System.out.println("TCPSocketServer: run:" +
                                       " unexcepted object type: " + object);
                    return;
                }
                connections.addElement(socketThread);
                thread = new Thread(socketThread);
                thread.start();
            }
        } catch (Exception e) {
            System.out.println("TCPSocketServer: run: exception " + e);
            if (TCPSocketServer_Debug) {
                e.printStackTrace();
            }
        }
    }
}
