
/*
 * @(#)TokenProxy.java	1.2 97/05/28
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import SharedData.*;
import java.util.*;

/**
 * Shared Data Token client-side proxy class.
 *
 * @version     0.96 23 May 1997
 * @author      Rich Burridge
 */

public class
TokenProxy extends SharedDataProxy implements AbstractTokenProxy {

    /** The client-side token associated with this token client. */
    private Token token;

    /** The clients who have currently grabbed/inhibited this token. */
    private Vector grabbedClients = null;


    public Object
    getProxy() {
        if (TokenProxy_Debug) {
            System.out.println("TokenProxy: getProxy.");
        }

        return(this);
    }


/**
 * <A NAME="SD_INITPROXY"></A>
 * <EM>initProxy</EM> initialise the "client-side" proxy for this Token.
 *
 * @param <EM>name</EM> the name of the Token proxy being constructed.
 * This name cannot include '\u0001' or newline characters.
 * @param <EM>session</EM> the client-side session the Token belongs to.
 * @param <EM>object</EM> the client-side Token.
 */

    public void
    initProxy(String name, Session session, Object object) {
        if (TokenProxy_Debug) {
            System.out.println("TokenProxy: initProxy:" +
            		       " name: "    + name      +
            		       " session: " + session);
        }

        this.name    = name;
        this.session = session;
        this.token   = (Token) object;
        clients      = new Hashtable();
	super.initProxy(name, session, object);
    }


/**
 * <A NAME="SD_GIVE"></A>
 * <EM>give</EM>
 *
 * @param <EM>client</EM> the client giving this token.
 * @param <EM>receivingClientName</EM> the name of the client to receive
 * the token.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>        
 * The following exception types can occur:
 *      NO_SUCH_TOKEN  - token given does not exist.
 *      NO_SUCH_CLIENT - client given does not exist.
 * </PRE>       
 *
 * @return the status of the token.
 */

    public int
    give(Client client, String receivingClientName)
				throws SharedData.SharedDataException {
        int    id           = sp.proxyThread.getId();
        String waitMessage  = addMessageHeader(session.getName(), id,
			      		       Token.M_Token + T_Sep + T_Give);
        String message      = waitMessage + T_Sep + name + T_Sep +
			      client.getName() + T_Sep + receivingClientName;
        StringTokenizer tok = null;

        if (TokenProxy_Debug) {
            System.out.println("TokenProxy: give:" +
                        " client: "                + client +
                        " receiving client name: " + receivingClientName);
        }

        tok = sendMessage(sp.proxyThread, waitMessage, message, null, 0);
        return(Integer.parseInt(tok.nextToken()));
    }


/**
 * <A NAME="SD_GRAB"></A>
 * <EM>grab</EM>
 *
 * @param <EM>client</EM> the client grabbing/inhibiting this token.
 * @param <EM>exclusive</EM> indicates whether the grab should be exclusive.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      NO_SUCH_TOKEN  - token given does not exist.
 *      NO_SUCH_CLIENT - client given does not exist.
 * </PRE>
 *
 * @return the status of the token.
 */

    public int
    grab(Client client, boolean exclusive)
				 throws SharedData.SharedDataException {
        int    id           = sp.proxyThread.getId();
        String waitMessage  = addMessageHeader(session.getName(), id,
			      		       Token.M_Token + T_Sep + T_Grab);
        String message      = waitMessage + T_Sep + name + T_Sep +
			      client.getName() + T_Sep + exclusive;
        StringTokenizer tok = null;

        if (TokenProxy_Debug) {
            System.out.println("TokenProxy: grab:" +
                                " client: "    + client  +
                                " exclusive: " + exclusive);
        }

        tok = sendMessage(sp.proxyThread, waitMessage, message, null, 0);
        return(Integer.parseInt(tok.nextToken()));
    }


/**
 * <A NAME="SD_JOIN"></A>
 * <EM>join</EM>
 *
 * @param <EM>token</EM>
 * @param <EM>client</EM>
 */

    void
    join(Token token, Client client) {
        if (TokenProxy_Debug) {
            System.out.println("TokenProxy: join:" +
                               " token: "  + token +
                               " client: " + client);
        }

        synchronized (clients) {
	    clients.put(client.getName(), client);
        }

	if (sp.validToken(token) == false) {
            sp.addToken(token);
        }
    }    


/**
 * <A NAME="SD_PLEASE"></A>
 * <EM>please</EM>
 *
 * @param <EM>client</EM> the client requesting this token.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      NO_SUCH_TOKEN - token given does not exist.
 * </PRE>
 *
 * @return the status of the token.
 */

    public int
    please(Client client) throws SharedData.SharedDataException {
        int    id           = sp.proxyThread.getId();
        String waitMessage  = addMessageHeader(session.getName(), id,
		      		       Token.M_Token + T_Sep + T_Please);
        String message      = waitMessage + T_Sep + name + T_Sep +
			      client.getName();
        StringTokenizer tok = null; 
 
        if (TokenProxy_Debug) {
            System.out.println("TokenProxy: please:" +
                                " client: "    + client);
        }
 
        tok = sendMessage(sp.proxyThread, waitMessage, message, null, 0);
        return(Integer.parseInt(tok.nextToken()));
    }


/**
 * <A NAME="SD_RELEASE"></A>
 * <EM>release</EM>
 *
 * @param <EM>client</EM> the client releasing this token.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      NO_SUCH_TOKEN       - token given does not exist.
 *      NO_SUCH_CLIENT      - client given does not exist.
 *      CLIENT_NOT_GRABBING - this client is not grabbing this token.
 *      CLIENT_NOT_RELEASED - the token was not released successfully.
 * </PRE>
 *
 * @return the status of the token.
 */

    public int
    release(Client client) throws SharedData.SharedDataException {
        int    id           = sp.proxyThread.getId();
        String waitMessage  = addMessageHeader(session.getName(), id,
				       Token.M_Token + T_Sep + T_Release);
        String message      = waitMessage + T_Sep + name + T_Sep +
			      client.getName();
        StringTokenizer tok = null; 

        if (TokenProxy_Debug) {
            System.out.println("TokenProxy: release:" +
                                " client: "    + client);
        }

        tok = sendMessage(sp.proxyThread, waitMessage, message, null, 0);
        return(Integer.parseInt(tok.nextToken()));
    }


/**
 * <A NAME="SD_TEST"></A>
 * <EM>test</EM>
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      NO_SUCH_TOKEN - token given does not exist.
 * </PRE>
 *
 * @return one of the following:
 * <PRE>
 * NOT_IN_USE : not in use
 * GRABBED    : grabbed
 * INHIBITED  : inhibited
 * GIVING     : giving
 * </PRE>
 */
 
    public int
    test() throws SharedData.SharedDataException {
        int    id           = sp.proxyThread.getId();
        String waitMessage  = addMessageHeader(session.getName(), id,
					       Token.M_Token + T_Sep + T_Test);
        String message      = waitMessage + T_Sep + name;
        StringTokenizer tok = null;

        if (TokenProxy_Debug) {
            System.out.println("TokenProxy: test:" +
                                " message: " + message);
        }

        tok = sendMessage(sp.proxyThread, waitMessage, message, null, 0);
        return(Integer.parseInt(tok.nextToken()));
    }
}
