
/*
 * @(#)UDPSocketServer.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import SharedData.*;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.util.StringTokenizer;

/**
 * Shared Data UDP Socket Server class.
 *
 * @version     0.89 27 January 1997
 * @author      Rich Burridge
 */

class
UDPSocketServer extends SocketServer {

    /** The session that this UDP channel belongs too. */
    private Session session;

    /** The UDP channel that this is a server for. */
    private Channel channel;

    /** The datagram socket to await packets on. */
    private static DatagramSocket dsock = null;


/**
 * <A NAME="SD_UDPSOCKETSERVER"></A>
 * <EM>UDPSocketServer</EM>
 *
 * @param <EM>session</EM>
 * @param <EM>channel</EM>
 * @param <EM>port</EM>
 */

    public
    UDPSocketServer(Session session, Channel channel, int port) {
        if (UDPSocketServer_Debug) {
            System.out.println("UDPSocketServer: constructor:" +
                               " session "  + session +
                               " channel: " + channel +
			       " port: "    + port);
        }

        try {
            if (dsock == null) {
                dsock = new DatagramSocket(port);
            }
        } catch (Exception e) {
            System.out.println("UDPSocketServer: exception " + e);
            if (UDPSocketServer_Debug) {
                e.printStackTrace();
            }
        }

        this.session = session;
	this.channel = channel;
	this.port    = port;
    }


/**
 * <A NAME="SD_RUN"></A>
 * <EM>run</EM>
 */

    public void
    run() {
	if (UDPSocketServer_Debug) {
            System.out.println("UDPSocketServer: run.");
        }

	while (true) {
            int            size   = socketSharedDataObject.UDP_BUF_SIZE;
            DatagramPacket packet = new DatagramPacket(new byte[size], size);
            UDPChannelServerThread cst    = null;
            Thread                 thread = null;

            try {
                if (UDPSocketServer_Debug) {
                    System.out.println("UDPSocketServer: Waiting...");
                }

                dsock.receive(packet);

                if (UDPSocketServer_Debug) {
                    System.out.println("UDPSocketServer: Got a packet.");
                }

		cst = new UDPChannelServerThread(session, channel, packet);
                thread = new Thread(cst);
                thread.start();

            } catch (Exception e) {
                System.out.println("UDPSocketServer: run: exception " + e);
                if (UDPSocketServer_Debug) {
                    e.printStackTrace();
                }
            }
	}
    }
}
