
/*
 * @(#)ChannelProxy.java        1.12 97/02/12
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import SharedData.*;
import java.util.*;

/**
 * Shared Data Session client-side proxy class.
 *
 * @version     0.90 11 February 1997
 * @author      Rich Burridge
 */

public class
ChannelProxy extends SharedDataProxy implements AbstractChannelProxy {

    /** The client-side channel associated with this channel client. */
    private Channel channel;

    /** The client that joined this channel. */
    private Client client;

    /** Clients currently joined to this channel and their data direction. */
    protected Hashtable clientDataDirections = null;

    /** A dummy client for use with the receive() method. */
    private ReceiveClient receiveClient = null;

    /** The consumers for the various clients on this channel. */
    protected Hashtable channelConsumers = null;

    /** Channel threads for sending and consuming data (one for each
     *  priority level).
     */
    private SocketThread channelThreads[];


/**
 * <A NAME="SD_INITPROXY"></A>
 * <EM>initProxy</EM> initialise the "client-side" proxy for this Channel.
 *
 * @param <EM>name</EM> the name of the Channel proxy being constructed.
 * This name cannot include '\u0001' or newline characters.
 * @param <EM>session</EM> the client-side session the Channel belongs to.
 * @param <EM>object</EM> the client-side Channel.
 */

    public void
    initProxy(String name, Session session, Object object) {
	if (ChannelProxy_Debug) {
	    System.out.println("ChannelProxy: initProxy:" +
			       " name: "    + name        +
			       " session: " + session     +
			       " channel: " + channel);
	}

	this.name    = name;
	this.session = session;
	this.channel = (Channel) object;
	super.initProxy(name, session, object);
    }


    public Object
    getProxy() {
	if (ChannelProxy_Debug) {
	    System.out.println("ChannelProxy: getProxy.");
	}

	return(this);
    }


/**
 * <A NAME="SD_DATARECEIVED"></A>
 * <EM>dataReceived</EM>
 *
 * @param <EM>session</EM>
 * @param <EM>channelName</EM>
 * @param <EM>senderName</EM>
 * @param <EM>receiverName</EM>
 * @param <EM>priority</EM>
 * @param <EM>length</EM>
 * @param <EM>theBytes</EM>
 */

    void
    dataReceived(Session session, String channelName,
		 String senderName, String receiverName,
		 int priority, int length, byte[] theBytes) {
	Data data = new Data(theBytes, length);

	if (ChannelProxy_Debug) {
	    System.out.println("ChannelProxy: dataReceived:" +
			       " session: "       + session +
			       " channel name: "  + channelName +
			       " sender name: "   + senderName +
			       " receiver name: " + receiverName +
			       " priority: "      + priority +
			       " length: "        + length +
			       " data: "          + theBytes);
	}

	if (clients == null) {
	    return; 
	} 
 
	data.setPriority(priority);
	data._setSenderName(senderName);
	data._setChannel(channel);
	synchronized (clients) {
	    Client client = (Client) clients.get(receiverName);
	    if (client != null) {
		ChannelConsumer consumer = getConsumer(client);

		if (consumer != null) {
		    ConsumerMessage cm = new ConsumerMessage(consumer, data);
		    Thread thread = new Thread(cm);

		    thread.start();
		}
	    }
	}
    }


/**
 * <A NAME="SD_GETCONSUMER"></A>
 * <EM>getConsumer</EM>
 *
 * @param <EM>client</EM>
 */

    public ChannelConsumer
    getConsumer(Client client) {
	if (ChannelProxy_Debug) {
	    System.out.println("ChannelProxy: getConsumer: " + client);
	}

	if (channelConsumers != null) {
	    synchronized (channelConsumers) {
		return((ChannelConsumer) channelConsumers.get(client));
	    }
	}
	return(null);
    }


/**
 * <A NAME="SD_JOIN"></A>
 * <EM>join</EM>
 *
 * @param <EM>channel</EM>
 * @param <EM>client</EM>
 * @param <EM>mode</EM>
 */

    void
    join(Channel channel, Client client, int mode) {
	if (ChannelProxy_Debug) {
	    System.out.println("ChannelProxy: join:" +
				" channel: " + channel     +
				" client: "  + client      +
				" mode: "    + mode);
	}

	if (clients == null) {
	    clients = new Hashtable();
	}
	synchronized (clients) {
	    clients.put(client.getName(), client);
	}

	if (clientDataDirections == null) {
	    clientDataDirections = new Hashtable();
	}   
	synchronized (clientDataDirections) {
	    clientDataDirections.put(client, new Integer(mode));
	}

	if (sp.validChannel(channel) == false) {
	    sp.addChannel(channel);
	}
    }


/**
 * <A NAME="SD_LEAVE"></A>
 * <EM>leave</EM>
 *
 * @param <EM>client</EM>
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 */

    public void
    leave(Client client) throws SharedData.SharedDataException {
	if (ChannelProxy_Debug) {
	    System.out.println("ChannelProxy: leave: " + client);
	}

	super.leave(client, Channel.M_Channel);
	synchronized (clientDataDirections) {
	    clientDataDirections.remove(client);
	}
    }


/**
 * <A NAME="SD_RECEIVE"></A>
 * <EM>receive</EM>
 *
 * @return the next Data object available on this channel.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 */

    public Data
    receive() throws SharedData.SharedDataException {
	Data data = null;

	if (ChannelProxy_Debug) {
	    System.out.println("ChannelProxy: receive.");
	}

	if (((Integer) clientDataDirections.get(client)).intValue() ==
						 Channel.WRITEONLY) {
	    throw new SharedData.SharedDataException(
			SharedData.SharedDataException.PERMISSION_DENIED);
	}

	if (receiveClient == null) {
	    receiveClient = new ReceiveClient(channel, "__receive__");

	    sp.joinSession(receiveClient, false);
	    sp.joinChannel(session, channel.getName(), receiveClient,
				   Channel.READONLY, false);
	    setConsumer(receiveClient, (ChannelConsumer) receiveClient);
	}

	try {
	    data = receiveClient.getMessage();
	} catch (Exception e) {
	    System.out.println("ChannelProxy: receive: exception " + e);
	    if (ChannelProxy_Debug) {
		e.printStackTrace();
	    }
	}
	return(data);
    }


    void
    setJoiningClient(Client client) {
	this.client = client;
    }


/**
 * <A NAME="SD_DATAAVAILABLE"></A>
 * <EM>dataAvailable</EM>
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 *
 * @return true if data available to read on this channel; false if not.
 */

    public boolean
    dataAvailable() throws SharedData.SharedDataException {
	if (ChannelProxy_Debug) {
	    System.out.println("ChannelProxy: dataAvailable.");
	}

	if (receiveClient == null) {
	    receiveClient = new ReceiveClient(channel, "__receive__");

	    sp.joinSession(receiveClient, false);
	    sp.joinChannel(session, channel.getName(), receiveClient,
				   Channel.READONLY, false);
	    setConsumer(receiveClient, (ChannelConsumer) receiveClient);
	}
	return(receiveClient.dataAvailable());
    }


/**
 * <A NAME="SD_SEND"></A>
 * <EM>send</EM> is used to send data to clients joined to the specified
 * channel. A message containing the data is sent to the session server
 * which then sends messages to the appropriate receiving channel consumers.
 *
 * @param <EM>sendingClient</EM> the client sending the data.
 * @param <EM>receivingClientName</EM> the name of the client receiving the
 * data (or the special tokens D_All or D_Others).
 * @param <EM>data</EM> the data being send over this channel.
 * @param <EM>uniform</EM> set true if this is a uniform send.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 */

    public void
    send(Client sendingClient, String receivingClientName,
	 Data data, boolean uniform) throws SharedData.SharedDataException {
	byte   theBytes[]   = data.getData();
	int    length       = data.getLength();
	int    priority     = data.getPriority();
	int    retval       = 0;
	String channelName  = channel.getName();
	String senderName   = sendingClient.getName();
	String waitMessage  = addMessageHeader(session.getName(),
				       Channel.M_Channel + T_Sep + T_Send);
	String message      = waitMessage + T_Sep + channelName + T_Sep +
			      senderName + T_Sep + receivingClientName +
			      T_Sep + priority + T_Sep + length + T_Sep +
			      uniform + T_Sep;
	String reply        = null;

	if (ChannelProxy_Debug) {
	    System.out.println("ChannelProxy: send: " + message);
	}

	reply = channelThreads[priority].writeData(waitMessage, message,
						   theBytes, length);
	retval = Integer.parseInt(reply.substring(waitMessage.length()+1));

	if (retval != 0) {
	    throw new SharedData.SharedDataException(retval);
	}
    }

 /* I'm writing this class for implementing the sendwithFlag method declared in the */
 /* AbstractChannelProxy interface       */

  public void sendwithFlag(Client sendingClient,String receivingClientName,Data data,int
			   flag,boolean uniform) throws SharedData.SharedDataException {
		      byte theBytes[]  =  data.getData();
		      int length       =  data.getLength();
		      int priority     =  data.getPriority();
			    int flag         =  data.getFlag();
		      int retval       = 0;
		      String channelName = channel.getName();
			    String senderName  = sendingClient.getName();
		      String waitMessage = addMessageHeader(session.getName(),Channel.M_Channel+
							    T_Sep +T_Send);
		      String message     = waitMessage + T_Sep+channelName+T_Sep+senderName+
							 T_Sep+receivingClientName+T_Sep+priority+T_Sep+length+
							 T_Sep+flag+T_Sep+uniform+T_Sep;
		      String reply =null;

		   if(ChannelProxy_Debug)
		     {
				System.out.println("ChannelProxy:sendwithFlag: " + message);
		     }

		   reply = channelThreads[priority].writeData(waitMessage,message,theBytes,length);

		   /* A doubt exists here - does flag needs to be added as a parameter for the */
		   /* writeData method or is it enough if it's included in the message String  */
		   
		   retval = Integer.parseInt(reply.substring(waitMessage.length()+1));
		 
			if(retval !=0)
			 {
			  throw new SharedData.SharedDataException(retval);
		   }

		   }

/**
 * <A NAME="SD_SETCONSUMER"></A>
 * <EM>setConsumer</EM>
 *
 * @param <EM>client</EM>
 * @param <EM>consumer</EM>
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 */

    public void
    setConsumer(Client client, ChannelConsumer consumer)
					throws SharedData.SharedDataException {
	String waitMessage = addMessageHeader(session.getName(),
				Channel.M_Channel + T_Sep + T_SetConsumer);
	String message     = waitMessage + T_Sep + name + T_Sep +
			     client.getName() + T_Sep;
	String reply       = null;
	int    retval      = 0;

	if (ChannelProxy_Debug) {
	    System.out.println("ChannelProxy: setConsumer:" + message);
	}

	if (consumer == null) {
	    message += T_Null;
	} else {
	    message += consumer;
	}

	for (int i = 0; i < Channel.MAX_PRIORITIES; i++) {
	    String threadMessage = message + T_Sep + i;

	    reply = channelThreads[i].writeLine(waitMessage, threadMessage);
	    retval = Integer.parseInt(reply.substring(waitMessage.length()+1));

	    if (retval != 0) {
		throw new SharedData.SharedDataException(retval);
	    }
	}

	if (channelConsumers == null) {
	    channelConsumers = new Hashtable();
	}
 
	synchronized (channelConsumers) {
	    if (consumer != null) {
		channelConsumers.put(client, consumer);
	    } else {
		if (channelConsumers.remove(client) == null) {
		    throw new SharedData.SharedDataException(
			SharedData.SharedDataException.NO_SUCH_CONSUMER);
		}
	    }
	}
    }


/**
 * <A NAME="SD_SETCHANNELTHREADS"></A>
 * <EM>setChannelThreads</EM> creates a new thread (and therefore a new
 * socket connection for each priority level for this channel.
 */

    void
    setChannelThreads() {
	boolean reliable = channel.isReliable();

	if (ChannelProxy_Debug) {
	    System.out.println("ChannelProxy: setChannelThreads.");
	}

	channelThreads = new SocketThread[Channel.MAX_PRIORITIES];
	for (int i = 0; i < Channel.MAX_PRIORITIES; i++) {
	    Thread thread;

	    if (reliable == true) {
		channelThreads[i] = new TCPChannelProxyThread(session, channel,
					    sp.proxyThread.getHost(),
					    sp.proxyThread.getPort());
	    } else {
		channelThreads[i] = new UDPChannelProxyThread(session, channel,
					    sp.proxyThread.getHost(),
					    sp.proxyThread.getPort());
	    }
	    thread = new Thread(channelThreads[i]);

	    thread.start();
	}
    }
}
