
/*
 * @(#)socketSession.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import SharedData.*;

/**
 * Shared Data Socket Session class.
 *
 * @version     0.90 11 February 1997
 * @author      Rich Burridge
 */

public class
socketSession extends Session implements socketDebugFlags {


/**
 * <A NAME="SD_SOCKETSESSION"></A>
 * <EM>socketSession</EM> the default constructor for the socketSession class.
 * Note that the session name is supplied by _createProxy or _createServer.
 */

    public
    socketSession() {
    }


/**
 * <A NAME="SD_SOCKETSESSION"></A>
 * <EM>socketSession</EM> the constructor for the socketSession class.
 *
 * @param <EM>name</EM> the name of the session being constructed.
 * This name cannot include '\u0001' or newline characters.
 */

    public
    socketSession(String name) {
        if (socketSession_Debug) {
            System.out.println("socketSession: constructor:" +
                               " name: " + name);
        }
 
        this.name = name;
    }


/**
 * <A NAME="SD__CREATEPROXY"></A>
 * <EM>_createProxy</EM> create a proxy-side connection for this socket
 * Session.
 *
 * @param <EM>name</EM> the name of the session being constructed.
 * This name cannot include '\u0001' or newline characters.
 * @param <EM>connectionType</EM> the type of this connection.
 * @param <EM>host</EM> the host of the server-side connection.
 * @param <EM>port</EM> the port number to use for the connection.
 */

    public synchronized void
    _createProxy(String name, String connectionType, String host, int port) {
	if (socketSession_Debug) {
            System.out.println("socketSession: _createProxy:"        +
			       " object name: "     + name           +
                               " connection type: " + connectionType +
			       " host: "            + host           +
			       " port: "            + port);
        }

        setConnectionType(connectionType);

	this.name  = name;
	po         = new SessionProxy(this, name, host, port);
	po.initProxy(name, this, this);
	objectType = M_Session;
	mpo        = (SharedDataProxy) po;
    }


/**
 * <A NAME="SD__CREATESERVER"></A>
 * <EM>_createServer</EM> create a server-side connection for this socket
 * Session.
 *
 * @param <EM>name</EM> the name of the session being constructed.
 * This name cannot include '\u0001' or newline characters.
 * @param <EM>connectionType</EM> the type of this connection.
 * @param <EM>url</EM> the url associated with this object.
 * @param <EM>port</EM> the port number to use for the connection.
 */

    public synchronized void
    _createServer(String name, String connectionType, String url, int port) {
	SessionServer ss = null;

	if (socketSession_Debug) {
            System.out.println("socketSession: _createServer:"       +
			       " object name: "     + name           +
                               " connection type: " + connectionType +
			       " url: "             + url            +
			       " port: "            + port);
        }

        setConnectionType(connectionType);

	this.name = name;
	so = new SessionServer(this, name, port);
	so.initServer(name, this, this);

	ss = (SessionServer) so.getServer();
	ss.setUrl(url);
    }
}
