
/*
 * @(#)socketSharedDataObject.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import SharedData.*;
import java.net.InetAddress;
import java.util.StringTokenizer;

/**
 * Shared Data root class.
 *
 * @version     0.94 7 April 1997
 * @author      Rich Burridge
 */

class
socketSharedDataObject implements socketDebugFlags {

    /** The type of this implemention (socket, rmi, lrmp ...). */
    String connectType;

    /** The maximum size for UDP datagram packets. */

    static final int UDP_BUF_SIZE = 8192;

    /** The version number string for this release. */

    static final String versionString = "1";

    /* Initial message tokens. */

    static String T_Session_Name;
    static String T_Version;

    /* The types of objects that can receive messages.  Note that the tokens
     * for Session, Channel and Token are obtained from those classes.
     */

    static String T_Manager;

    /* The various message types. */

    static String T_AddByteObserver;
    static String T_AddObserver;
    static String T_Authenticate;
    static String T_Challenge;
    static String T_CreateByteArray;
    static String T_CreateChannel;
    static String T_CreateToken;
    static String T_DataReceived;
    static String T_DeleteByteObserver;
    static String T_DeleteObserver;
    static String T_DestroyByteArray;
    static String T_Expel;
    static String T_GetUrl;
    static String T_Give;
    static String T_Grab;
    static String T_InformObserver;
    static String T_Invite;
    static String T_Join;
    static String T_Leave;
    static String T_ListByteArrayNames;
    static String T_ListChannelNames;
    static String T_ListClientNames;
    static String T_ListTokenNames;
    static String T_Please;
    static String T_Release;
    static String T_Send;
    static String T_SetConsumer;
    static String T_Test;
    static String T_UpdateByteArray;
    static String T_ValueChanged;
    static String T__Manager;

    /* The possible registry messages. */

    static String T_Bind;
    static String T_Rebind;
    static String T_Unbind;
    static String T_Lookup;
    static String T_List;
 
    /* The possible client change messages. */
 
    static String T_Session_Invite;
    static String T_Session_Expel;
    static String T_Channel_Invite;
    static String T_Channel_Expel;
    static String T_Token_Invite;
    static String T_Token_Expel;
 
    /* The token separator. */
 
    static final String T_Sep  = "\u0001";
 
    /* Token separators for parsing messages. */

    static final String M_Sep = "\n\u0001";

    /* Other useful message tokens. */
 
    static final String T_All    = "__all__";
    static final String T_Null   = "__null__";
    static final String T_Others = "__others__";

    static {
	if (socketSharedDataObject_Debug) {

            T_Session_Name     = "SESSION_NAME";
            T_Version          = "VERSION";

            T_Manager          = "MANAGER"; 

	    T_AddByteObserver  = "ADDBYTEOBSERVER";
            T_AddObserver      = "ADDOBSERVER";
            T_Authenticate     = "AUTHENTICATE";
            T_Challenge        = "CHALLENGE";
            T_CreateByteArray  = "CREATEBYTEARRAY";
	    T_CreateChannel    = "CREATECHANNEL";
            T_CreateToken      = "CREATETOKEN";
            T_DataReceived     = "DATARECEIVED";
            T_DeleteByteObserver = "DELETEBYTEOBSERVER";
	    T_DeleteObserver   = "DELETEOBSERVER";
            T_DestroyByteArray = "DESTROYBYTEARRAY";
	    T_Expel            = "EXPEL"; 
            T_GetUrl           = "GETURL";
            T_Give             = "GIVE";
            T_Grab             = "GRAB";
            T_InformObserver   = "INFORMOBSERVER";
            T_Invite           = "INVITE";
            T_Join             = "JOIN";
            T_Leave            = "LEAVE";
            T_ListByteArrayNames = "LISTBYTEARRAYNAMES";
	    T_ListChannelNames = "LISTCHANNELNAMES";
            T_ListClientNames  = "LISTCLIENTNAMES";
            T_ListTokenNames   = "LISTTOKENNAMES";
            T_Please           = "PLEASE";
            T_Release          = "RELEASE";
            T_Send             = "SEND";
            T_SetConsumer      = "SETCONSUMER";
            T_Test             = "TEST";
            T_UpdateByteArray  = "UPDATEBYTEARRAY";
	    T_ValueChanged     = "VALUECHANGED";
	    T__Manager         = "_MANAGER";

            T_Bind             = "BIND";
            T_Rebind           = "REBIND";
            T_Unbind           = "UNBIND";
            T_Lookup           = "LOOKUP";
            T_List             = "LIST";

            T_Session_Invite   = "SESSION_INVITE";
            T_Session_Expel    = "SESSION_EXPEL";
            T_Channel_Invite   = "CHANNEL_INVITE";
            T_Channel_Expel    = "CHANNEL_EXPEL";
            T_Token_Invite     = "TOKEN_INVITE";
            T_Token_Expel      = "TOKEN_EXPEL";

	} else {

            T_Session_Name    = "SN";
            T_Version         = "V";

            T_Manager         = "M";

            T_AddByteObserver  = "ABO";
            T_AddObserver      = "AD";
            T_Authenticate     = "AU";
            T_Challenge        = "CH";
            T_CreateByteArray  = "CBA";
	    T_CreateChannel    = "CRC";
            T_CreateToken      = "CRT";
            T_DataReceived     = "DA";
            T_DeleteByteObserver = "DBO";
	    T_DeleteObserver   = "DE";
            T_DestroyByteArray = "DBA";
	    T_Expel            = "E";
            T_GetUrl           = "GE";
            T_Give             = "GI";
            T_Grab             = "GR";
            T_InformObserver   = "INF";
            T_Invite           = "INV";
            T_Join             = "J";
            T_Leave            = "LE";
            T_ListByteArrayNames = "LBAN";
	    T_ListChannelNames = "LICH";
            T_ListClientNames  = "LICL";
            T_ListTokenNames   = "LIT";
            T_Please           = "P";
            T_Release          = "R";
            T_Send             = "SEN";
            T_SetConsumer      = "SET";
            T_Test             = "T";
            T_UpdateByteArray  = "UBA";
	    T_ValueChanged     = "VC";
	    T__Manager         = "_M";

            T_Bind             = "B";
            T_Rebind           = "R";
            T_Unbind           = "U";
            T_Lookup           = "LO";
            T_List             = "LI";

            T_Session_Invite   = "SI";
            T_Session_Expel    = "SE";
            T_Channel_Invite   = "CI";
            T_Channel_Expel    = "CE";
            T_Token_Invite     = "TI";
            T_Token_Expel      = "TE";
	}
    }


/**
 * <A NAME="SD_ADDMESSAGEHEADER"></A>
 * <EM>addMessageHeader</EM> prepend the session name and version number
 * to the given message.
 *
 * @param <EM>sessionName</EM> the session name to use in this message.
 * @param <EM>id</EM> the id to use for client/server messages.
 * @param <EM>message</EM> the message.
 *
 * @return a String with the standard prepended parts.
 */

    String
    addMessageHeader(String sessionName, int id, String message) {
        if (socketSharedDataObject_Debug) {
            System.out.println("socketSharedDataObject: addMessageHeader:" +
                               " session name: " + sessionName +
                               " id: "           + id +
                               " message: "      + message);
        }

	return(T_Version      + T_Sep + versionString + T_Sep +
	       T_Session_Name + T_Sep + sessionName   + T_Sep + 
	       id             + T_Sep + message);
    }


/**
 * <A NAME="SD_SENDMESSAGE"></A>
 * <EM>sendMessage</EM> send a message (possibly with data) over a socket
 * to the server. If waiting for a reply, check the return value and throw
 * a SharedDataException if it's non-zero, otherwise return a StringTokenizer
 * handle to the rest of the return message.
 *
 * @param <EM>thread</EM> the thread connection to the server.
 * @param <EM>waitMessage</EM> the reply to wait for (if any).
 * @param <EM>message</EM> the message to write to the socket.
 * @param <EM>data</EM> the data (or null if there isn't any).
 * @param <EM>length</EM> the length (in bytes) of the data.
 *
 * @return a StringTokenizer handle to the remainder of the return message.
 */

    StringTokenizer
    sendMessage(SocketThread thread, String waitMessage, String message,
                byte[] data, int length) throws SharedDataException {
        StringTokenizer tok    = null;
        String          reply  = null;
        int             retval = 0;
 
        if (socketSharedDataObject_Debug) {
            System.out.println("socketSharedDataObject: sendMessage:" +
                                " thread: "      + thread +
                                " waitMessage: " + waitMessage +
                                " message: "     + message +
                                " data: "        + data +
                                " length: "      + length);
        }
 
        if (data != null) {
            reply = thread.writeData(waitMessage, message, data, length);
        } else {
            reply = thread.writeLine(waitMessage, message);
        }
        tok = new StringTokenizer(reply.substring(waitMessage.length()+1),
                                  M_Sep, false);
        if ((retval = Integer.parseInt(tok.nextToken())) != 0) {
            throw new SharedDataException(retval);
        }
        return(tok);
    }


/**
 * <A NAME="SD_UNIQUENAME"></A>
 * <EM>uniqueName</EM> generates a unique name for object based on its name,
 * it's hashcode and the host it's running on.
 *
 * @param <EM>object</EM> the object in question.
 * @param <EM>name</EM> the name of this object.
 *
 * @return a String which is a unique name based on the name/object for this
 * host
 */

    String
    uniqueName(Object object, String name) {
        String newName = null;
        String s = null;
 
        if (socketSharedDataObject_Debug) {
            System.out.println("it is here..socketSharedDataObject: uniqueName.");
        }

        try {
    System.out.println(InetAddress.getLocalHost());

	    newName = name + '\t' + InetAddress.getLocalHost() + '\t' +
                      object.hashCode();
	} catch (Exception e) {
            System.out.println("problem in assigning uniquename:" +
				" exception " + e);
         if(socketSharedDataObject_Debug) {
                e.printStackTrace();
            } 
	}
    System.out.println("Still returning "+" "+newName);
	return(newName);

    }


/**
 * <A NAME="SD_VALIDMESSAGEHEADER"></A>
 * <EM>validMessageHeader</EM> check to see if this message contains valid
 * version string and parse over the session name.
 *
 * @param <EM>tok</EM> the string tokenizer for this message.
 *
 * @return a String containing the session name.
 */

    String
    validMessageHeader(StringTokenizer tok) {
        String sessionName = null;
	String token       = tok.nextToken();

	if (socketSharedDataObject_Debug) {
            System.out.println("socketSharedDataObject: validMessageHeader.");
        }

        if (token.equals(T_Version)) {
            tok.nextToken();                 /* Get version string. */
            tok.nextToken();                 /* Get T_Session_Name token. */
            sessionName = tok.nextToken();   /* Get session name. */
        } else {
            System.out.println("socketSharedDataObject: validMessageHeader:" +
                               " unexpected token: " + token);
        }
	return(sessionName);
    }
}
