// Copyright (c) 1995, 1996 Regents of the University of California.
// All rights reserved.
//
// This software was developed by the Arcadia project
// at the University of California, Irvine.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the University of California, Irvine.  The name of the
// University may not be used to endorse or promote products derived
// from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.

// File: ActionGroup.java
// Classes: ActionGroup
// Original Author: jrobbins@ics.uci.edu
// $Id: ActionGroup.java,v 1.9 1997/06/10 23:42:12 jrobbins Exp $

package uci.graphedit;

import java.awt.*;
import java.util.*;

/** Action to group all the DiagramElement's selected in the current
 *  editor into a single FigList. Needs-More-Work: should use
 *  composite DiagramElement, not composite Fig.  Needs-More-Work:
 *  DiagramElement's other than Figs are ignored.
 *  <A HREF="../features.html#basic_shapes_group">
 *  <TT>FEATURE: basic_shapes_group</TT></A>
 *
 * @see FigList
 * @see ActionUngroup  */

public class ActionGroup extends Action {

  public ActionGroup() { }

  public String name() {
    return "Group selected DiagramElement's into a FigList";
  }

  public void doIt(Event e) {
    Editor ce = Globals.curEditor();
    Vector selectedDEs = ce.selectedDEs();
    if (selectedDEs.size() < 2 && !e.shiftDown()) return;
    FigList newItem = new FigList();
    Enumeration eachDE = selectedDEs.elements();
    while (eachDE.hasMoreElements()) {
      Object o = eachDE.nextElement();
      if (o instanceof Fig) {
	Fig f = (Fig) o;
	newItem.addFig(f);
      }
    }
    eachDE = selectedDEs.elements();
    while (eachDE.hasMoreElements()) {
      Object o = eachDE.nextElement();
      if (o instanceof Fig) {
	Fig f = (Fig) o;
	ce.remove(f);
      }
    }
    ce.add(newItem);
    ce.deselect();
    ce.selectItem(newItem);
  }

  public void undoIt() { System.out.println("not done yet"); }

} /* end class ActionGroup */

