// Copyright (c) 1995, 1996 Regents of the University of California.
// All rights reserved.
//
// This software was developed by the Arcadia project
// at the University of California, Irvine.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the University of California, Irvine.  The name of the
// University may not be used to endorse or promote products derived
// from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.

// File: ActionRemoveVertex.java
// Classes: ActionRemoveVertex
// Original Author: jrobbins@ics.uci.edu
// $Id: ActionRemoveVertex.java,v 1.6 1997/06/10 23:42:18 jrobbins Exp $

package uci.graphedit;

import java.awt.Event;

/** Action to remove the selected (last manipulated) vertex from a FigPoly.
 *
 * @see FigPoly
 */

public class ActionRemoveVertex extends Action {

  public ActionRemoveVertex() { }

  public String name() { return "Remove selected vertex from polygon"; }

  /** Ask the current editor's selected DiagramElement to remove its vertex. */
  public void doIt(Event e) {
    DiagramElement de = null;
    SelectionSingle sel = null;
    Editor ce = Globals.curEditor();
    SelectionMultiple sels = ce.selection();
    if (sels.locked()) {
      Globals.showStatus("Cannot Modify Locked Objects");
      return;
    }

    if (sels != null && !sels.contents().isEmpty())
      sel = (SelectionSingle) sels.contents().firstElement();
    if (sel != null) de = (DiagramElement) sel.content();
    if (de != null && de instanceof FigPoly) {
        FigPoly p = (FigPoly) de;
        p.startTrans();
        p.removeSelectedVertex();
        p.endTrans();
    }
    if (de != null && de instanceof ArcPerzRectiline) {
        ArcPerzRectiline p = (ArcPerzRectiline) de;
        p.startTrans();
        p.removeSelectedVertex();
        p.endTrans();
    }
  }

  public void undoIt() {
    System.out.println("this operation currently cannot be undone");
  }

} /* end class ActionRemoveVertex */
