// Copyright (c) 1995, 1996 Regents of the University of California.
// All rights reserved.
//
// This software was developed by the Arcadia project
// at the University of California, Irvine.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the University of California, Irvine.  The name of the
// University may not be used to endorse or promote products derived
// from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.

// File: EditorMenus.java
// Classes: EditorMenus
// Original Author: ics125b spring 1996
// $Id: EditorMenus.java,v 1.22 1997/06/10 23:42:38 jrobbins Exp $

package uci.graphedit;

import java.awt.*;

/** Contains the menu items used in Editor and handles menu events. */

public class EditorMenus extends Object implements GEF {

  ////////////////////////////////////////////////////////////////
  // instance variables

  /* Menu items */
  public MenuBar menuBar; /* Needs-More-Work */
  private Menu fileMenu;
  private MenuItem newItem;
  private MenuItem openItem;
  private MenuItem saveItem;
  private MenuItem printItem;
  private MenuItem prefsItem;
  private MenuItem closeItem;
  private MenuItem quitItem;

  private Menu editMenu;
  private Menu selectMenu;
  private MenuItem selectAllItem;
  private MenuItem selectNextItem;
  private MenuItem selectPrevItem;
  private MenuItem selectInvertItem;
  private MenuItem selectSuchThatItem;
  private MenuItem deleteItem;
  private MenuItem disposeItem;
  private MenuItem copyItem;
  private MenuItem cutItem;
  private MenuItem pasteItem;
  private MenuItem editNodeItem;
  private MenuItem executeActionItem;

  private Menu viewMenu;
  private MenuItem spawnViewItem;
  private MenuItem spawnPropertySheetItem;
  private MenuItem zoomInItem;
  private MenuItem zoomOutItem;
  private MenuItem zoomNormalItem;
  private MenuItem adjustGridItem;
  private MenuItem adjustGuideItem;
  private MenuItem adjustBreaksItem;

  private Menu arrangeMenu;
  private MenuItem groupItem;
  private MenuItem ungroupItem;

  private Menu alignMenu;
  private MenuItem alignTopsItem;
  private MenuItem alignBottomsItem;
  private MenuItem alignLeftsItem;
  private MenuItem alignRightsItem;
  private MenuItem alignCentersItem;
  private MenuItem alignHCentersItem;
  private MenuItem alignVCentersItem;
  private MenuItem alignToGridItem;

  private Menu reorderMenu;
  private MenuItem sendToBackItem;
  private MenuItem bringToFrontItem;
  private MenuItem sendBackwardItem;
  private MenuItem bringForwardItem;

  Editor parent;

  ////////////////////////////////////////////////////////////////
  // constructor

  public EditorMenus(Editor ce) {
    menuBar = new MenuBar();
    parent = ce;
    /* Initialize the file menu */
    menuBar.add(fileMenu = new Menu("File"));
    fileMenu.add(newItem = new MenuItem("New"));
    newItem.disable();
    fileMenu.add(openItem = new MenuItem("Open..."));
    fileMenu.add(saveItem = new MenuItem("Save..."));
    fileMenu.add(printItem = new MenuItem("Print Diagram..."));
    fileMenu.addSeparator();
    fileMenu.add(prefsItem = new MenuItem("Preferences..."));
    fileMenu.addSeparator();
    fileMenu.add(closeItem = new MenuItem("Close Window"));
    fileMenu.add(quitItem = new MenuItem("Quit"));

    /* Initialize the edit Menu */
    menuBar.add(editMenu = new Menu("Edit"));
    editMenu.add(selectMenu = new Menu("Select"));
    selectMenu.add(selectAllItem = new MenuItem("All"));
    selectMenu.add(selectNextItem = new MenuItem("Next"));
    selectMenu.add(selectPrevItem = new MenuItem("Prev"));
    selectMenu.add(selectInvertItem = new MenuItem("Invert Selection"));
    selectMenu.add(selectSuchThatItem = new MenuItem("Such That..."));
    selectSuchThatItem.disable();
    editMenu.add(disposeItem = new MenuItem("Delete"));
    editMenu.add(copyItem = new MenuItem("Copy"));
    copyItem.disable();
    editMenu.add(cutItem = new MenuItem("Cut"));
    cutItem.disable();
    editMenu.add(pasteItem = new MenuItem("Paste"));
    pasteItem.disable();
    editMenu.addSeparator();
    editMenu.add(editNodeItem = new MenuItem("Edit Node"));
    editMenu.addSeparator();
    editMenu.add(executeActionItem = new MenuItem("Execute Action..."));

    /* Initialize the view Menu */
    menuBar.add(viewMenu = new Menu("View"));
    viewMenu.add(spawnViewItem = new MenuItem("Spawn New View"));
    viewMenu.add(spawnPropertySheetItem =
		 new MenuItem("Spawn Property Sheet"));
    viewMenu.addSeparator();
    viewMenu.add(zoomInItem = new MenuItem("Zoom In"));
    zoomInItem.disable();
    viewMenu.add(zoomOutItem = new MenuItem("Zoom Out"));
    zoomOutItem.disable();
    viewMenu.add(zoomNormalItem = new MenuItem("Zoom Normal"));
    zoomNormalItem.disable();
    viewMenu.addSeparator();
    viewMenu.add(adjustGridItem = new MenuItem("Adjust Grid Appearance"));
    viewMenu.add(adjustGuideItem = new MenuItem("Adjust Grid Snap"));
    viewMenu.add(adjustBreaksItem = new MenuItem("Toggle Page Breaks"));

    menuBar.add(arrangeMenu = new Menu("Arrange"));
    arrangeMenu.add(groupItem = new MenuItem("Group"));
    arrangeMenu.add(ungroupItem = new MenuItem("Ungroup"));
    arrangeMenu.add(alignMenu = new Menu("Align"));
    alignMenu.add(alignTopsItem = new MenuItem("Tops"));
    alignMenu.add(alignBottomsItem = new MenuItem("Bottoms"));
    alignMenu.add(alignLeftsItem = new MenuItem("Lefts"));
    alignMenu.add(alignRightsItem = new MenuItem("Rights"));
    alignMenu.add(alignCentersItem = new MenuItem("Centers"));
    alignMenu.add(alignHCentersItem = new MenuItem("Horizontal Centers"));
    alignMenu.add(alignVCentersItem = new MenuItem("Vertical Centers"));
    alignMenu.add(alignToGridItem = new MenuItem("To Grid"));
    arrangeMenu.add(reorderMenu = new Menu("Reorder"));
    reorderMenu.add(sendToBackItem = new MenuItem("To Back"));
    reorderMenu.add(bringToFrontItem = new MenuItem("To Front"));
    reorderMenu.add(sendBackwardItem = new MenuItem("Backward"));
    reorderMenu.add(bringForwardItem = new MenuItem("Forward"));

    ce.setMenuBar(menuBar);
  }

  public void add(Menu m) {
    menuBar.add(m);
  }

  ////////////////////////////////////////////////////////////////
  // event handlers

  public boolean handleMenuEvent(Event e) {
    Editor ce = parent;
    Action act = null;
    Object arg = e.arg;
    if ("Quit".equals(arg))
      act = new ActionQuit();
    else if ("Close Window".equals(arg))
      parent.close();
    else if ("Preferences...".equals(arg))
      act = new ActionOpenWindow("uci.graphedit.PrefsEditor");
    else if ("Open...".equals(arg))
      act = new ActionLoad();
    else if ("Save...".equals(arg))
      act = new ActionSave();
    else if ("Print Diagram...".equals(arg))
      act = new ActionPrint();
    else if ("Edit Node".equals(arg))
      act = new ActionEditNode();
    else if ("Execute Action...".equals(arg))
      act = new ActionOpenWindow("uci.graphedit.ExecuteActionWindow");
    else if ("Spawn New View".equals(arg))
      act = new ActionSpawn();
    else if ("Spawn Property Sheet".equals(arg)) {
      act = null;
      Globals.startPropertySheet();
      return true;
    }
    // Needs-More-Work: there is no action Zoom yet
    // Needs-More-Work: there is no action Zoom yet
    // else if (e.target == zoomInItem)
    //   (new ActionZoom(ActionZoom.IN)).doIt(e);
    // else if (e.target == zoomOutItem)
    //   (new ActionZoom(ActionZoom.OUT)).doIt(e);
    // else if (e.target == zoomNormalItem)
    //   (new ActionZoom(ActionZoom.NORMAL)).doIt(e);
    else if ("Adjust Grid Appearance".equals(arg))
      act = new ActionAdjustGrid();
    else if ("Adjust Grid Snap".equals(arg))
      act = new ActionAdjustGuide();
    else if ("Toggle Page Breaks".equals(arg))
      act = new ActionAdjustPageBreaks();
    else if ("Group".equals(arg))
      act = new ActionGroup();
    else if ("Ungroup".equals(arg))
      act = new ActionUngroup();
    else if ("Tops".equals(arg))
      act = new ActionAlign(ActionAlign.ALIGN_TOPS);
    else if ("Bottoms".equals(arg))
      act = new ActionAlign(ActionAlign.ALIGN_BOTTOMS);
    else if ("Lefts".equals(arg))
      act = new ActionAlign(ActionAlign.ALIGN_LEFTS);
    else if ("Rights".equals(arg))
      act = new ActionAlign(ActionAlign.ALIGN_RIGHTS);
    else if ("Centers".equals(arg))
      act = new ActionAlign(ActionAlign.ALIGN_CENTERS);
    else if ("Horizontal Centers".equals(arg))
      act = new ActionAlign(ActionAlign.ALIGN_H_CENTERS);
    else if ("Vertical Centers".equals(arg))
      act = new ActionAlign(ActionAlign.ALIGN_V_CENTERS);
    else if ("To Grid".equals(arg))
      act = new ActionAlign(ActionAlign.ALIGN_TO_GRID);
    //else if ("Delete".equals(arg))
    //      act = new ActionDelete();
    else if ("To Back".equals(arg))
      act = new ActionReorder(ActionReorder.SEND_TO_BACK);
    else if ("To Front".equals(arg))
      act = new ActionReorder(ActionReorder.BRING_TO_FRONT);
    else if ("Backward".equals(arg))
      act = new ActionReorder(ActionReorder.SEND_BACKWARD);
    else if ("Forward".equals(arg))
      act = new ActionReorder(ActionReorder.BRING_FORWARD);
    else if ("All".equals(arg))
      act = new ActionSelectAll();
    else if ("Next".equals(arg))
      act = new ActionSelectNext(true);
    else if ("Prev".equals(arg))
      act = new ActionSelectNext(false);
    else if ("Invert Selection".equals(arg))
      act = new ActionSelectInvert();
    //else if ("Such That...".equals(arg))
    //  act = new ActionSelectSuchThat();
    else if ("Delete".equals(arg))
      act = new ActionDispose();
    else return false;

    ce.executeAction(act, e);
    return true;
  }

} /* end class EditorMenus */
