// Copyright (c) 1995, 1996 Regents of the University of California.
// All rights reserved.
//
// This software was developed by the Arcadia project
// at the University of California, Irvine.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the University of California, Irvine.  The name of the
// University may not be used to endorse or promote products derived
// from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.

// File: FigInk.java
// Classes: FigInk
// Original Author: jrobbins@ics.uci.edu
// $Id: FigInk.java,v 1.9 1997/06/10 23:42:42 jrobbins Exp $

package uci.graphedit;

import java.applet.*;
import java.awt.*;
import java.io.*;
import java.util.*;

/** Primitive Fig to draw Ink on a LayerDiagram. Ink is like an open
 *  polygon with no fill. The main difference between FigInk and
 *  FigPoly is in the way that they are created.
 *  <A HREF="../features.html#basic_shapes_ink">
 *  <TT>FEATURE: basic_shapes_ink</TT></A>
 *
 * @see FigPoly
 * @see ModeCreateFigInk */

public class FigInk extends FigPoly {

  /** Construct a new FigInk w/ the given line color,
   *  and fill color. */
  public FigInk(Color line_color, Color fill_color) {
    super(line_color, fill_color);
    filled = false;
  }

  /** Construct a new FigInk w/ the given line color. */
  public FigInk(Color line_color) {
    super(line_color);
    filled = false;
  }

  /** Construct a new FigInk w/ the given attributes. */
  public FigInk(Hashtable gAttrs){ super(gAttrs); filled = false;}

  /** Construct a new FigInk w/ the given point and attributes. */
  public FigInk(int x, int y, Hashtable gAttrs){
    this(null, null);
    put(gAttrs);
    addPoint(x, y);
    filled = false;
  }

  ////////////////////////////////////////////////////////////////
  // accessors

  /** Line width of ink must be at least 1. Since Java does not support
   *  thicker lines, it will always be 2. */
  public void setLineWidth(int w) { lineWidth = Math.max(1, Math.min(1, w)); }

  /** FigInks can never be filled, do nothing. */
  public void setFilled(boolean f) { filled = false; }

  /** FigInks can never be rectilinear, do nothing. */
  public void rectilinear(boolean r) { _rectilinear = false; }

  public boolean canPut(String key) {
    if (key.equals(pFILLED) || key.equals(pLINE_WIDTH) ||
	key.equals(pRECTILINEAR))
      return false;
    return super.canPut(key);
  }

  ////////////////////////////////////////////////////////////////
  // drawing methods

  public void draw(Graphics g) {
    // FigInk's are never filled
    if (lineWidth > 0) {
      g.setColor(objectLineColor);
      Compat.drawPolyline(polygon(), g);
    }
  }

  /** Needs-More-Work: remove this method, its logic has been moved to
   *  SelectionBox. */
  public void drawSelected(Graphics g) { }

  ////////////////////////////////////////////////////////////////
  // selection methods

  public Selection selectionObject() {
    // no one handle is selected when the polygon is first selected
    _selectedHandle = -1;
    return new SelectionBox(this);
  }

  /** Return a handle ID for the handle under the mouse, or -1 if
   *  none.  Needs-More-Work: in the future, return a Handle instance
   *  or null. */
  public int pickHandle(int x, int y) { return -1; }

  public boolean inside(int x, int y) {
    return super.pickHandle(x, y) != -1;
  }

} /* end class FigInk */

