// Copyright (c) 1995, 1996 Regents of the University of California.
// All rights reserved.
//
// This software was developed by the Arcadia project
// at the University of California, Irvine.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the University of California, Irvine.  The name of the
// University may not be used to endorse or promote products derived
// from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.

// File: FigLine.java
// Classes: FigLine
// Original Author: ics125b spring 1996
// $Id: FigLine.java,v 1.13 1997/06/10 23:42:43 jrobbins Exp $

package uci.graphedit;

import java.applet.*;
import java.awt.*;
import java.io.*;
import java.util.*;

/** Class to implement lines in diagrams.
 *  <A HREF="../features.html#basic_shapes_line">
 *  <TT>FEATURE: basic_shapes_line</TT></A>
 */

public class FigLine extends Fig {

  ////////////////////////////////////////////////////////////////
  // instance variables

  ////////////////////////////////////////////////////////////////
  // constructors

  /** Construct a new FigLine with the given coordinates and color. */
  public FigLine(int x, int y, int r_width, int r_height, Color l_color){
    super(x, y, r_width, r_height, l_color, null, 2);
  }

  /** Construct a new FigLine with the given coordinates and attributes. */
  public FigLine(int x,int y,int r_width,int r_height, Hashtable gAttrs){
    super(x, y, r_width, r_height, Color.black, null, 2);
    put(gAttrs);
  }

  ////////////////////////////////////////////////////////////////
  // accessors

  public void setShape(Point p1, Point p2) {
    position().x = p1.x;
    position().y = p1.y;
    objectWidth = p2.x;
    objectHeight = p2.y;
  }

  public void setShape(int x1, int y1, int x2, int y2) {
    position().x = x1;
    position().y = y1;
    objectWidth = x2;
    objectHeight = y2;
  }

  ////////////////////////////////////////////////////////////////
  // DiagramElement API

  /** Translate this Fig. */
  public void translate(int dx, int dy) {
    position().x += dx;
    position().y += dy;
    objectWidth += dx;
    objectHeight += dy;
  }

  /** Set the position of this Fig to the goven location */
  public void position(int x, int y) {
    int dx = x - position().x;
    int dy = y - position().y;
    translate(dx,dy);
  }

  /** Draw this line object. */
  public void draw(Graphics g) {
    if (lineWidth > 0) {
      g.setColor(objectLineColor);
      g.drawLine(position().x, position().y, objectWidth, objectHeight);
    }
  }

  /** Draw this line when it is selected. */
  public void drawSelected(Graphics g) {
    _handleRects[0].x = position().x - 3;
    _handleRects[0].y = position().y  - 3;
    _handleRects[1].x = objectWidth - 3;
    _handleRects[1].y = objectHeight - 3;
    drawHandles(g);
  }

  /** When this Fig is selected, the Editor should use an instance of
   *  SelectionHandles to record that fact. */
  public Selection selectionObject() { return new SelectionHandles(this); }

  /** Reply true iff the given point is "near" the line. Nearness
   *  allows the user to more easily select the line with the
   *  mouse. Needs-More-Work: I should probably have two functions
   *  inside() which gives a strict geometric version, and near() which
   *  is for selection by mouse clicks. */
  public boolean inside(int x, int y) {
    Point p = position();
    return Geometry.nearSegment(p.x, p.y, objectWidth, objectHeight,
				x, y, GRIP_MARGIN);
  }

  /** Reply true iff this line passes through, or is even partly
   *  inside the given rectangle, or if any corner of the rect is on
   *  the line. What happens if the line runs along one edge of the
   *  rect, but not all the way to either corner? */
  public boolean intersects(Rectangle r) {
    Point p = position();
    return Geometry.intersects(r, p.x, p.y, objectWidth, objectHeight);
  }

  /** Reply a rect that encloses this line. I add a few pixels on
   *  either side so that the rect will have non-zero area. */
  public Rectangle getBoundingBox() {
    Point p = position();
    return new Rectangle(Math.min(p.x, objectWidth) - 1,
			 Math.min(p.y, objectHeight) - 1,
			 Math.max(p.x, objectWidth) -
			 Math.min(p.x, objectWidth) + 2,
			 Math.max(p.y, objectHeight) -
			 Math.min(p.y, objectHeight) + 2);
  }

  /** Allow he user to change either end point of the line by dragging
   *  on a handle. */
  public void dragHandle(int m_x, int m_y, int an_x, int an_y, Handle h) {
    int world_x = m_x;
    int world_y = m_y;
    switch (h.index) {
    case 0:			// The origin handle
      position().x = m_x;
      position().y = m_y;
      break;
    case 1:			// the endpoint handle
      objectWidth = m_x;
      objectHeight = m_y;
      break;
    }
  }

  /** Resize the object for drag on creation. It bypasses the things
   *  done in resize so that the position of the object can be kept as
   *  the anchor point. Needs-More-Work: do I really need this
   *  function?
   *
   * @see FigLine#drag */
  public void createDrag(int anchorX, int anchorY, int x, int y,
			 int snapX, int snapY) {
    objectWidth = snapX;
    objectHeight = snapY;
  }

} /* end class FigLine */

