// Copyright (c) 1995, 1996 Regents of the University of California.
// All rights reserved.
//
// This software was developed by the Arcadia project
// at the University of California, Irvine.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the University of California, Irvine.  The name of the
// University may not be used to endorse or promote products derived
// from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.

// File: FigRRect.java
// Classes: FigRRect
// Original Author: ics125b spring 1996
// $Id: FigRRect.java,v 1.10 1997/06/10 23:42:45 jrobbins Exp $

package uci.graphedit;

import java.applet.*;
import java.awt.*;
import java.io.*;
import java.util.*;
import uci.util.*;
import uci.ui.*;

/** Primitive Fig to draw rounded rectangles on a LayerDiagram.
 *  <A HREF="../features.html#basic_shapes_rounded_rect">
 *  <TT>FEATURE: basic_shapes_rounded_rect</TT></A>
 */

public class FigRRect extends FigRect {

  ////////////////////////////////////////////////////////////////
  // instance variables

  protected int _radius = 16;

  ////////////////////////////////////////////////////////////////
  // constructors

  public FigRRect(int x, int y, int w, int h, Color line, int r) {
    super(x, y, w, h, line);
  }

  /** Construct a new FigRRect w/ the given position, size, and attributes */
  public FigRRect(int x, int y, int w, int h, Hashtable gAttrs) {
    super(x, y, w, h, gAttrs);
  }

  /** Construct a new FigRRect w/ the given position, size, line color,
   * and fill color*/
  public FigRRect(int x, int y, int w, int h, Color line, Color fill) {
    super(x, y, w, h, line, fill);
  }

  ////////////////////////////////////////////////////////////////
  // accessors

  public void radius(int r) { _radius = r; }
  public int radius() { return _radius; }

  public void setCornerRadius(int r) { _radius = r; }
  public int getCornerRadius() { return _radius; }

  public Object get(String key) {
    if (key.equals(pCORNER_RADIUS)) return new Integer(getCornerRadius());
    else return super.get(key);
  }

  public boolean put(String key, Object value) {
    if (key.equals(pCORNER_RADIUS))
      setCornerRadius(((Integer)value).intValue());
    else return super.put(key, value);
    return true;
  }

  public static Vector _GeometryPropName = null;

  public Enumeration keysIn(String category) {
    if (_GeometryPropName == null) {
      _GeometryPropName = new Vector();
      _GeometryPropName.addElement(pCORNER_RADIUS);
    }
    Enumeration geomEnum = _GeometryPropName.elements();
    if (category.equals("Geometry"))
      return new EnumerationComposite(super.keysIn(category), geomEnum);
    else return super.keysIn(category);
  }

  public boolean canPut(String key) {
    return key.equals(pCORNER_RADIUS) || super.canPut(key);
  }

  ////////////////////////////////////////////////////////////////
  /// drawing methods

  /** Draw this FigRRect */
  public void draw(Graphics g) {
    int rx = position().x;
    int ry = position().y;
    int new_x, new_y, new_w, new_h;
    int r = radius();

    /* Compensate for negative width and height (for drag) */
    if (objectWidth > 0) { new_x = rx; new_w = objectWidth; }
    else { new_x = rx + objectWidth; new_w = -objectWidth; }

    if (objectHeight > 0) { new_y = ry; new_h = objectHeight; }
    else { new_y = ry + objectHeight; new_h = -objectHeight; }

    if (filled) {
      g.setColor(objectFillColor);
      g.fillRoundRect(new_x, new_y, new_w, new_h, r, r);
    }
    if (lineWidth > 0) {
      g.setColor(objectLineColor);
      g.drawRoundRect(new_x, new_y,
		      new_w - lineWidth, new_h - lineWidth,
		      r, r);
    }
  }

} /* end class FigRRect */

