// Copyright (c) 1995, 1996 Regents of the University of California.
// All rights reserved.
//
// This software was developed by the Arcadia project
// at the University of California, Irvine.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the University of California, Irvine.  The name of the
// University may not be used to endorse or promote products derived
// from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.

// File: ModeCreateFigInk.java
// Classes: ModeCreateFigInk
// Original Author: ics125b spring 1996
// $Id: ModeCreateFigInk.java,v 1.4 1997/06/10 23:43:06 jrobbins Exp $

package uci.graphedit;

import java.awt.Event;

/** A Mode to interpert user input while creating a FigInk.
 *  <A HREF="../features.html#basic_shapes_ink">
 *  <TT>FEATURE: basic_shapes_ink</TT></A>
 */

public class ModeCreateFigInk extends ModeCreate {

  ////////////////////////////////////////////////////////////////
  // instance variables

  public static final int MIN_DELTA = 4;

  protected int _lastX, _lastY;

  ////////////////////////////////////////////////////////////////
  // Mode API

  public String instructions() {
    return "Drag to draw a stream of ink";
  }

  ////////////////////////////////////////////////////////////////
  // ModeCreate API

  /** Create a new FigRect instance based on the given mouse down
   *  event and the state of the parent Editor. */
  public DiagramElement createNewItem(Event e, int snapX, int snapY) {
    FigInk p = new FigInk(snapX, snapY, parent.graphAttrs());
    _lastX = snapX; _lastY = snapY;
    return p;
  }

  ////////////////////////////////////////////////////////////////
  // Event handlers

  /** Mouse up means that the user is done. */
  public boolean mouseUp(Event e, int x, int y) {
    parent.damaged(newItem);
    // do not call creationDrag()
    parent.add(newItem);
    parent.selectItem(newItem);
    newItem = null;
    done();
    return true;
  }

  /** Dragging adds points to the ink. */
  public boolean mouseDrag(Event e, int x, int y) {
    FigInk ink = (FigInk)newItem;
    if (!nearLast(x, y)) {
      parent.damaged(newItem); // startTrans?
      ink.addPoint(x, y);
      parent.damaged(newItem); // endTrans?
      _lastX = x; _lastY = y;
    }
    return true;
  }

  /** Internal function to test if the current point is so close to
   *  the last point that it should not be added to the ink. */
  protected boolean nearLast(int x, int y) {
    return x > _lastX - MIN_DELTA &&
      x < _lastX + MIN_DELTA &&
      y > _lastY - MIN_DELTA &&
      y < _lastY + MIN_DELTA;
  }

} /* end class ModeCreateFigInk */

