// Copyright (c) 1995, 1996 Regents of the University of California.
// All rights reserved.
//
// This software was developed by the Arcadia project
// at the University of California, Irvine.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the University of California, Irvine.  The name of the
// University may not be used to endorse or promote products derived
// from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.

// File: SelectionBox.java
// Classes: SelectionBox
// Original Author: jrobbins@ics.uci.edu
// $Id: SelectionBox.java,v 1.10 1997/06/10 23:43:30 jrobbins Exp $

package uci.graphedit;

import java.util.*;
import java.awt.*;

/** Selection object for DiagramElement's that do not have handles. A
 *  colorded rectangular frame is drawn just outside the bounding box
 *  of the DiagramElement. The box has things that look like handles on
 *  its four corners so that it looks more like a selection and less
 *  like the highlighting used by Argo. However, these fake handles
 *  cannot be dragged to resize the DiagramElement.
 *  <A HREF="../features.html#selections_box">
 *  <TT>FEATURE: selections_box</TT></A>
 *
 * @see FigText
 * @see Perspective */

public class SelectionBox extends SelectionSingle {

  /** The margin between the contents bbox and the frame */
  public static final int BORDER_WIDTH = 4;

  /**  Height and width in pixels of (square) rectangles used for handle */
  public static final int HAND_SIZE = 6;

  /** Construct a new SelectionBox around the given DiagramElement */
  public SelectionBox(DiagramElement de) { super(de); }

  /** Draw the selection. */
  public void draw(Graphics g) {
    Rectangle bb = _content.getBoundingBox();
    g.setColor(Globals.prefs().handleColor(_content));
    g.drawRect(bb.x - BORDER_WIDTH,
	       bb.y - BORDER_WIDTH,
	       bb.width + BORDER_WIDTH * 2 - 1,
	       bb.height + BORDER_WIDTH * 2 - 1);
    g.drawRect(bb.x - BORDER_WIDTH - 1,
	       bb.y - BORDER_WIDTH - 1,
	       bb.width + BORDER_WIDTH * 2 + 2 - 1,
	       bb.height + BORDER_WIDTH * 2 + 2 - 1);
    g.fillRect(bb.x - HAND_SIZE, bb.y - HAND_SIZE,
	       HAND_SIZE, HAND_SIZE);
    g.fillRect(bb.x + bb.width, bb.y - HAND_SIZE,
	       HAND_SIZE, HAND_SIZE);
    g.fillRect(bb.x - HAND_SIZE, bb.y + bb.height,
	       HAND_SIZE, HAND_SIZE);
    g.fillRect(bb.x + bb.width, bb.y + bb.height,
	       HAND_SIZE, HAND_SIZE);
  }

  /** SelectionBox is used when there are no handles, so dragHandle
   * does nothing. Actually, pickHandle always returns -1 , so this
   * method should never even get called. */
  public void dragHandle(int mx, int my, int an_x,int an_y, Handle h) {
    /* do nothing */
  }

  /** The bounding box of the SelectionBox is the contents bbox plus a
   * border on all sides */
  public Rectangle getBoundingBox() {
    Rectangle r = _content.getBoundingBox();
    return r;
  }

} /* end class SelectionBox */

