// Copyright (c) 1995, 1996 Regents of the University of California.
// All rights reserved.
//
// This software was developed by the Arcadia project
// at the University of California, Irvine.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the University of California, Irvine.  The name of the
// University may not be used to endorse or promote products derived
// from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.

// File: ArcData.java
// Classes: ArcData
// Original Author: jrobbins@ics.uci.edu
// $Id: ArcData.java,v 1.4 1997/06/10 23:40:26 jrobbins Exp $

package uci.graphedit.demo;

import uci.graphedit.*;
import java.util.*;
import uci.util.*;

/** A sample NetArc subclass for use in the Example application.  There
 *  are no real details here yet.  If I was to expand this Example more
 *  the arc could have more attributes, e.g. bandwidth... and it
 *  could have its own subclasses of ArcPerspective to make it look a
 *  certain way. */

public class ArcData extends NetArc {
  ////////////////////////////////////////////////////////////////
  // constants

  public static final String pCABLE_TYPE = "Cable Type";
  public static final String pBITS_PER_SECOND = "Bits/Second";

  public static final String CABLE_SCSI = "SCSI";
  public static final String CABLE_ETHERNET = "Ethernet";
  public static final String CABLE_FIREWIRE = "FireWire";
  public static final String CABLE_FDDI = "FDDI";

  ////////////////////////////////////////////////////////////////
  // instance variables

  /** Speed of this data line */
  protected int _bitsPerSecond = 100000;

  /** Type of cable, choosen from the constants above: SCSI, FireWire,
   * etc. */
  protected String _cableType = CABLE_SCSI;


  ////////////////////////////////////////////////////////////////
  // constructors

  public ArcData() { } /* needs-more-work */

  ////////////////////////////////////////////////////////////////
  // accessors

  public String getCableType() { return _cableType; }
  public void setCableType(String ct) {
    if (_PossibleCableTypes.contains(ct))
      _cableType = ct;
  }

  public int getBitsPerSecond() { return _bitsPerSecond; }
  public void setBitsPerSecond(int bps) { _bitsPerSecond = bps; }

  public Object get(String key) {
    if (key.equals(pCABLE_TYPE))
      return getCableType();
    else if (key.equals(pBITS_PER_SECOND))
      return new Integer(getBitsPerSecond());
    else return super.get(key);
  }

  public boolean put(String key, Object value) {
    if (key.equals(pCABLE_TYPE))
      setCableType((String)value);
    else if (key.equals(pBITS_PER_SECOND))
      setBitsPerSecond(((Integer)value).intValue());
    else return super.put(key, value);
    return true;
  }

  public static Vector _ModelPropNames = null;

  public Enumeration keysIn(String category) {
    if (_ModelPropNames == null) {
      _ModelPropNames = new Vector();
      _ModelPropNames.addElement(pCABLE_TYPE);
      _ModelPropNames.addElement(pBITS_PER_SECOND);
    }
    Enumeration modelKeys = _ModelPropNames.elements();
    if (category.equals("Model"))
      return new EnumerationComposite(super.keysIn(category), modelKeys);
    else return super.keysIn(category);
  }

  public boolean canPut(String key) {
    return key.equals(pCABLE_TYPE);
    // note that pBITS_PER_SECONDis not editable
  }

  public static Vector _PossibleCableTypes = null;

  static {
    _PossibleCableTypes = new Vector();
    _PossibleCableTypes.addElement(CABLE_SCSI);
    _PossibleCableTypes.addElement(CABLE_ETHERNET);
    _PossibleCableTypes.addElement(CABLE_FIREWIRE);
    _PossibleCableTypes.addElement(CABLE_FDDI);
    uci.ui.PropSheetCategory.addEnumProp(pCABLE_TYPE, _PossibleCableTypes);
  }

  ////////////////////////////////////////////////////////////////
  // perspectives

  public ArcPerspective makePerspective(Layer lay) {
    return new ArcPerzRectiline();
  }

} /* end class ArcData */
