// Copyright (c) 1995, 1996 Regents of the University of California.
// All rights reserved.
//
// This software was developed by the Arcadia project
// at the University of California, Irvine.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the University of California, Irvine.  The name of the
// University may not be used to endorse or promote products derived
// from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.

// File: IProps.java
// Interfaces: IProps
// Original Author: jrobbins@ics.uci.edu
// $Id: IProps.java,v 1.4 1997/06/10 23:38:53 jrobbins Exp $

package uci.ui;

import java.util.*;
import uci.util.*;

/** Interface that must be implmented by any object that is to be
 *  edited in the property sheet window. Basically requires getting and
 *  putting named properties.  Default implementations of some of these
 *  methods is included as comments in this source file. */

public interface IProps extends uci.util.SERIALIZABLE { //extends Observable


  ////////////////////////////////////////////////////////////////
  // accessors

  /** Get an property by name. Subclasses should call super.get(Object)
   * returning if they do not hold a property. */
  public Object get(String key);
  //{
  //    // NMW: check shared properties
  //    return null;
  //}

  /** Get a property by name. If it is not defined, return def. */
  public Object get(String key, Object def);
  //{
  //    Object res = get(key);
  //    if (null != res) return res;
  //    return def;
  //}

  ////////////////////////////////////////////////////////////////
  // modifiers

  /** Set the value of a property. Subclasses must override this such that it
   *  calls notifyChange() with the name of the changed property. */
  public boolean put(String key, Object value);
  //{ return false; }

  /** Put each property listed in the given Hashtable. Keys are
   * property names, values are property values. */
  public void put(Hashtable props);

  /** Set the value of a property iff it is not already defined. */
  public boolean define(String key, Object value);
  //{
  //    if (null == get(key)) {
  //        put(key, value);
  //        return true;
  //   }
  //    return false;
  //}

  ////////////////////////////////////////////////////////////////
  // convience methods

  /** Same as get(String), but returns an int. */
  //public final int getInt(String key) { return getInt(key, 0); }

  /** Same as get(String, Object), but returns an int. */
  //public final int getInt(String key, int def) {
  //    Object res = get(key);
  //    if (res instanceof Integer) return ((Integer)res).intValue();
  //    return def;
  //}

  /** Same as put(String, Object), but takes an int. */
  //public final boolean putInt(String key, int value) {
  //    return put(key, new Integer(value));
  //}

  /** Same as get(String), but returns a boolean. */
  //public final boolean getBool(String key) { return getBool(key, false); }

  /** Same as get(String, Object), but returns a boolean. */
  //public final boolean getBool(String key, boolean def) {
  //    Object res = get(key);
  //    if (res instanceof Boolean) return ((Boolean)res).booleanValue();
  //    return def;
  //}

  /** Same as put(String, Object), but takes a boolean. */
  //public final boolean putBool(String key, boolean value) {
  //   return put(key, (value) ? Boolean.TRUE : Boolean.FALSE);
  //}

  ////////////////////////////////////////////////////////////////
  // meta-information

  /** Return an Enumeration of Strings for property names that are defined on this Props.
   *  If category is non-null then it names a subset of properties. Subclasses
   *  should override this to call super.keysIn() and then add their own keys. */
  public Enumeration keysIn(String category);
  //{
  //   return new EmptyEnumeration();
  //}

  public boolean canPut(String key);

  ////////////////////////////////////////////////////////////////
  // notifications and updates

  /** Notify observers that one of my properties has changed. */
  //public void changedProp(String key) {
  //   setChanged();
  //  notifyObservers(key);
  //}


} /* end class IProps */
