// Copyright (c) 1995, 1996 Regents of the University of California.
// All rights reserved.
//
// This software was developed by the Arcadia project
// at the University of California, Irvine.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the University of California, Irvine.  The name of the
// University may not be used to endorse or promote products derived
// from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.

// File: PropEditorColor.java
// Interfaces: PropEditorColor
// Original Author: jrobbins@ics.uci.edu
// $Id: PropEditorColor.java,v 1.2 1997/06/10 23:39:02 jrobbins Exp $

package uci.ui;

import java.awt.*;

/** <A HREF="../features.html#color_picker">
 *  <TT>FEATURE: color_picker</TT></A>
 */

public class PropEditorColor extends Panel {
  ////////////////////////////////////////////////////////////////
  // instance variables
  public Color _color = null;

  public PropEditorColor(Color c) {
    //{{INIT_CONTROLS
    setLayout(null);
    addNotify();
    resize(insets().left + insets().right + 20,
	   insets().top + insets().bottom + 20);
    //}}
    _color = c;
    setForeground(Color.lightGray);
  }

  public PropEditorColor() { this(Color.white); }

  public void addNotify() {
    if (_color != null) super.addNotify();
  }

  ////////////////////////////////////////////////////////////////
  // accessors

  public void setColor(Color c) {
    if (c == null) return;
    _color = c;
    repaint();
    ColorPickerGrid.updateIfEditing(this);
  }

  public Color getColor() { return _color; }


  ////////////////////////////////////////////////////////////////
  // drawing methods
  public void paint(Graphics g) {
    Dimension size = size();
    if (isEnabled()) {
      g.setColor(getForeground());
      g.draw3DRect(0, 0, size.width - 1, size.height - 1, true);
      g.setColor(_color);
    }
    else g.setColor(getBackground());
    g.fill3DRect(1, 1, size.width - 2, size.height - 2, true);
  }

  ////////////////////////////////////////////////////////////////
  // event handlers

  public boolean handleEvent(Event event) {
    if (event.id == Event.MOUSE_DOWN) {
      int gx = event.x;
      int gy = event.y;
      Component c = this;
      while (c instanceof Component) {
	Point loc = c.location();
	gx += loc.x;
	gy += loc.y;
	c = c.getParent();
      }
      ColorPickerGrid.edit(this, gx, gy);
      return true;
    }
    return super.handleEvent(event);
  }

  public void colorPicked(Color newColor) {
    if (newColor == null) return;
    setColor(newColor);
    postEvent(new Event(this, Event.ACTION_EVENT, newColor));
  }

  public Frame getFrame() {
    Component c = this;
    while (c != null && !(c instanceof Frame)) {
      c = c.getParent();
    }
    return (Frame) c;
  }

  public void hide() {
    super.hide();
    ColorPickerGrid.stopIfEditing(this);
  }
  
  //{{DECLARE_CONTROLS
  //}}
} /* end class PropEditorColor */
