// Copyright (c) 1995, 1996 Regents of the University of California.
// All rights reserved.
//
// This software was developed by the Arcadia project
// at the University of California, Irvine.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the University of California, Irvine.  The name of the
// University may not be used to endorse or promote products derived
// from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.

// File: PropEditorRect.java
// Interfaces: PropEditorRect
// Original Author: jrobbins@ics.uci.edu
// $Id: PropEditorRect.java,v 1.3 1997/06/10 23:39:07 jrobbins Exp $

package uci.ui;

import java.awt.*;

/** A widget that allows the user to edit rectangles in the property
 *  sheet window. Needs-More-Work: Currently only displays them without
 *  allowing editing.
 *  <A HREF="../features.html#rectangle_editor">
 *  <TT>FEATURE: rectangle_editor</TT></A>
 */

public class PropEditorRect extends Panel {
  ////////////////////////////////////////////////////////////////
  // instance variables
  public Rectangle _rect = null;

  public PropEditorRect(Rectangle r) {
    //{{INIT_CONTROLS
    setLayout(new GridLayout(1, 4));
    addNotify();
    resize(insets().left + insets().right + 20,
	   insets().top + insets().bottom + 20);
    xField = new TextField("0", 4);
    yField = new TextField("0", 4);
    hField = new TextField("0", 4);
    wField = new TextField("0", 4);
    add(xField);
    add(yField);
    add(hField);
    add(wField);
    //}}
    _rect = r;
  }

  public PropEditorRect() { this(new Rectangle(0, 0, 100, 100)); }

  public void addNotify() {
    if (_rect != null) super.addNotify();
  }

  public void enable() {
    xField.setEditable(true);
    yField.setEditable(true);
    wField.setEditable(true);
    hField.setEditable(true);
  }
    
  public void disable() {
    xField.setEditable(false);
    yField.setEditable(false);
    wField.setEditable(false);
    hField.setEditable(false);
  }
  
  ////////////////////////////////////////////////////////////////
  // accessors

  public void setRectangle(Rectangle r) {
    if (r == null) return;
    _rect = r;
    setFields();
  }

  public Rectangle getRectangle() { return _rect; }

  protected void setFields() {
    xField.setText(Integer.toString(_rect.x));
    yField.setText(Integer.toString(_rect.y));
    wField.setText(Integer.toString(_rect.width));
    hField.setText(Integer.toString(_rect.height));
  }

  protected void readFields() {
    try {
      _rect.x = Integer.parseInt(xField.getText());
      _rect.y = Integer.parseInt(yField.getText());
      _rect.width = Integer.parseInt(wField.getText());
      _rect.height = Integer.parseInt(hField.getText());
    }
    catch (java.lang.NumberFormatException ignore) {
      setFields();
    }
  }


  ////////////////////////////////////////////////////////////////
  // event handlers

  public boolean handleEvent(Event event) {
    if (event.target != this && event.id == Event.ACTION_EVENT) {
      readFields();
      postEvent(new Event(this, Event.ACTION_EVENT, _rect));
      return true;
    }
    return super.handleEvent(event);
  }


  //{{DECLARE_CONTROLS
  TextField xField;
  TextField yField;
  TextField wField;
  TextField hField;
  //}}
} /* end class PropEditorRect */
