// Copyright (c) 1995, 1996 Regents of the University of California.
// All rights reserved.
//
// This software was developed by the Arcadia project
// at the University of California, Irvine.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the University of California, Irvine.  The name of the
// University may not be used to endorse or promote products derived
// from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.

// File: PropsComposite.java
// Interfaces: PropsComposite
// Original Author: jrobbins@ics.uci.edu
// $Id: PropsComposite.java,v 1.3 1997/06/10 23:39:17 jrobbins Exp $

package uci.ui;

import java.util.*;

/** This class shows an example of how to efficiently combine
 *  IProps. Needs-More-Work: this class is not currently used in GEF,
 *  instead some classes basically duplicate this
 *  functionality. Needs-More-Work: it would be nice to implement a
 *  fly-weight pattern. */

public class PropsComposite extends Observable implements IProps {
  ////////////////////////////////////////////////////////////////
  // instance variables

  protected Vector _subs = new Vector();

  ////////////////////////////////////////////////////////////////
  // constructors

  public PropsComposite() { }

  public PropsComposite(IProps p1, IProps p2, IProps p3) {
    if (p1 != null) _subs.addElement(p1);
    if (p2 != null) _subs.addElement(p2);
    if (p3 != null) _subs.addElement(p3);
  }

  ////////////////////////////////////////////////////////////////
  // IProps API

  public Object get(String key) {
    Enumeration eachSub = _subs.elements();
    while (eachSub.hasMoreElements()) {
      IProps p = (IProps) eachSub.nextElement();
      Object res = p.get(key);
      if (res != null) return res;
    }
    return null;
  }
  public final Object get(String key, Object def) {
    Object res = get(key);
    if (null != res) return res;
    return def;
  }

  public boolean put(String key, Object value) {
    Enumeration eachSub = _subs.elements();
    while (eachSub.hasMoreElements()) {
      IProps p = (IProps) eachSub.nextElement();
      if (p.put(key, value)) return true;
    }
    return false;
  }

  public void put(Hashtable props) {
    Enumeration keys = props.keys();
    while (keys.hasMoreElements()) {
      String key = (String) keys.nextElement();
      Object value = props.get(key);
      put(key, value);
    }
  }

  public Enumeration keysIn(String category) {
    return (new Vector()).elements();
  }

  public boolean canPut(String key) {
    Enumeration eachSub = _subs.elements();
    while (eachSub.hasMoreElements()) {
      IProps p = (IProps) eachSub.nextElement();
      if (p.canPut(key)) return true;
    }
    return false;
  }

  public boolean define(String key, Object value) {
    if (null == get(key)) {
      put(key, value);
      return true;
    }
    return false;
  }

} /* end class PropsComposite */
