// Copyright (c) 1995, 1996 Regents of the University of California.
// All rights reserved.
//
// This software was developed by the Arcadia project
// at the University of California, Irvine.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the University of California, Irvine.  The name of the
// University may not be used to endorse or promote products derived
// from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.

// File: PropsGeneric.java
// Interfaces: PropsGeneric
// Original Author: jrobbins@ics.uci.edu
// $Id: PropsGeneric.java,v 1.3 1997/06/10 23:39:21 jrobbins Exp $

package uci.ui;
import java.util.*;

/** This class shows an example of how to implement interface
 *  IProps. Needs-More-Work: this class is not currently used in GEF,
 *  instead some classes basically duplicate this
 *  functionality. Needs-More-Work: it would be nice to implement a
 *  fly-weight pattern. */

public class PropsGeneric extends Observable implements IProps {
  ////////////////////////////////////////////////////////////////
  // instance variables
  private Hashtable _ht = new Hashtable();

  ////////////////////////////////////////////////////////////////
  // IProps API

  /** Get an property by name. */
  public Object get(String key) {
    return _ht.get(key);
  }
  public final Object get(String key, Object def) {
    Object res = get(key);
    if (null != res) return res;
    return def;
  }


  /** Set the value of a property. Null keys and values are ignored. */
  public boolean put(String key, Object value) {
    if (key != null && value != null) {
      _ht.put(key, value);
      changedProp(key);
      return true;
    }
    return false;
  }

  public void put(Hashtable props) {
    Enumeration keys = props.keys();
    while (keys.hasMoreElements()) {
      String key = (String) keys.nextElement();
      Object value = props.get(key);
      put(key, value);
    }
  }

  /** Return an Enumeration of Strings for property
   * names that are defined on this Props. */
  public Enumeration keysIn(String category) {
    return _ht.keys();
  }

  public boolean canPut(String key) { return true; }

  public boolean define(String key, Object value) {
    if (null == get(key)) {
      put(key, value);
      return true;
    }
    return false;
  }

  ////////////////////////////////////////////////////////////////
  // notifications and updates

  /** Notify observers that one of my properties has changed. */
  public void changedProp(String key) {
    setChanged();
    notifyObservers(key);
  }


} /* end class PropsGeneric */
