
package webflow.backend;

/* not needed in general: just for the test stub */
import java.io.*;
import java.util.StringTokenizer;

/**
* the ID class stores an unique ID that can be used by either ports or 
* modules. It only knows how to compare itself with different instances 
* of the ID class. 
*/
public class ID extends Object{
  /* ID state resides in a string and an integer. These should uniquely
     identify each ID */
  String uniqueStr;
  int uniqueInt;

  /**
  * creator of the ID class, used only when preparing to accept an ID
  * from a stream. Sets internal state to 'empty'
  */
  ID() {
    uniqueStr = null;
    uniqueInt = 0;
  }

  /** 
  * creator of the ID class. IDs are meant to be created by an IDFactory. 
  * Only that way can their uniqueness be guaranteed. 
  * @param uniqueString: String given by the IDFactory. Unique per 
  * 			 IDFactory 
  * @param uniqueInt: integer that makes IDs created by the same 
  *		      IDFactory unique
  */
  ID(String uniqueString, int uniqueInteger) {
    uniqueStr = uniqueString;
    uniqueInt = uniqueInteger;
  }

  /**
  * check whether the current instance is equal to the instance refered 
  * to by 'that'
  * @param that: reference to an object (presumably an ID) that the current 
  *              object is being compared to. 
  * @return true if the current object has the same ID as the object 
  *	    refered to by 'that', false if not. 
  */
  public boolean equals(Object comparedTo) {
    if(uniqueStr == null)
      return(false);
    boolean rett;
    try {
      ID that = (ID) comparedTo;
      rett = uniqueStr.equals(that.uniqueStr) && uniqueInt == that.uniqueInt;
    }
    catch(Exception e) {
      rett = false; 
    }
    return(rett);
  }

  /**
   * generate a hash code value for an ID object. For now it is the hashCode
   * of the string, multiplied by 1000, and then summed with the 
   * unique integer. 
   * @return integer: hash code value for the ID object. 
   */
  public int hashCode() {
    return uniqueStr.hashCode() * 1000 + uniqueInt;
  }

  /**
  */
  public String 
  getString() 
  { return new String(uniqueStr); }

  /**
  * Module name is in the following format;
  * [URL-address-of-the-WebFlowServer]_[Module-Name]
  */
  public String
  getModuleName()
  {
     String sTok = new String(uniqueStr);
     sTok.replace('_',' ');
     StringTokenizer st = new StringTokenizer(sTok);
     
     String ret = st.nextToken();

     if(st.countTokens()>0) ret = st.nextToken(); 
     
     return ret;

  }// end of getModuleName()

  /**
  */
  public int
  getInt()
  {  return uniqueInt; }

  /**
  * read the ID from the input stream 'in'.
  * @param in: an InputStream from which the representation for the
  *            ID will be read.
  * @return true if the ID was read, false if some error occured.
  */
  public boolean 
  fromStream(InputStream in) 
  {
    boolean rett;
    int  line;

    try
    {
	  // wrap a data input stream over the input stream. 
	  DataInputStream din = new DataInputStream(in);

      uniqueInt = din.readInt();

	  if (din.readBoolean())
	  {
	    int bufsize = din.readInt();

	    byte[] b = new byte[bufsize];

	    din.readFully(b,0,bufsize);

	    uniqueStr = new String(b,0,0,bufsize);

	  } else uniqueStr = null;

        rett =true;

    } catch(IOException e)
      { rett = false; }

    return rett;

  }// end of fromStream()

  /**
  * write the ID to the output stream 'out'.
  * @param out: an OutputStream to which the representation for the ID
  *             will be written.
  * @return true if the ID was written, false if some error occured.
  */
  public boolean 
  toStream(OutputStream out) 
  {
    boolean rett;

    try
    {
	   DataOutputStream dout = new DataOutputStream(out);
	   dout.writeInt(uniqueInt);

	   if(uniqueStr == null)
	      dout.writeBoolean(false);
	   else
	    {
	       dout.writeBoolean(true);
	       dout.writeInt(uniqueStr.length());
	       dout.writeBytes(uniqueStr);
	    }

	   dout.flush();

	   rett= true;

    } catch(IOException e)
      { rett = false; }

    return rett;

  }// end of toStream()
  
  /**
  * test stub ONLY. Only tries to make three IDs and compare them! 
  */
  public static void main(String [] argv)
  {
    ID one = new ID("blah1", 1);
    ID two = new ID("blah2", 2);
    ID three = one;
    if(argv[0].equals("give"))
    {
      System.out.println(" one == two: " + one.equals((Object)two));
      System.out.println(" two == three: " + two.equals((Object)three));
      System.out.println(" three == one: " + three.equals((Object)one));
      System.out.println(" hashCode(one): " + one.hashCode());
      System.out.println(" hashCode(two): " + two.hashCode());
      System.out.println(" hashCode(three): " + three.hashCode());
    }
    else if(argv[0].equals("to"))
      one.toStream(System.out);
    else if(argv[0].equals("from"))
    {
      one.fromStream(System.in);
      System.out.println(" one == three: " + one.equals((Object)three));
      System.out.println(" hashCode(one): " + one.hashCode());
    }
  }
}
