package webflow.backend;

import java.net.*;
import java.io.*;

/**
 * the ModuleRepresentation class will be used by the SessionManager to store 
 * and access the  list of the modules and the host on which the modules are 
 * present .This class is extends the MetaModule class which contains data 
 * about the module. 
*/
public class ModuleRepresentation extends MetaModule {
  // host on which the module resides
  String hostname;

  // port numbers of the module- and connection- manager on the host on 
  // which the module resides
  int MMport, CMport;
  
  /**
   * constructor used to initialize the host and ports, as well as the 
   * parent MetaModule
   * @param host - host on which the module resides
   * @param mm - MetaModule to copy in the ModuleRepresentation's internals
   * @param portMM - port for the ModuleManager
   * @param portCM - port for the ConnectionManager
   */
  ModuleRepresentation( String host, MetaModule mm, int portMM, int portCM)
  {
    super();

    hostname=host;
    MMport = portMM;
    CMport = portCM;

    // stream mm through pos, and read it into the parent MetaModule
    PipedOutputStream pos = null;
    PipedInputStream pis = null;

    // this is a go-around way of making a copy constructor work, when one
    // was not provided in the original design. Will be provided in the 
    // subsequent versions
    try{
      pos = new PipedOutputStream();
      pis = new PipedInputStream(pos);
      mm.toStream(pos);
      fromStream(pis);
    }
    catch(IOException e)
      {
      }
  }

  /**
   * return the host name on which the module resides
   * @return String giving the hostname.
   */
  public String getHostName()
  {
    return hostname;
  }

  /**
   * return the port on which the ModuleManager for this module listens
   * @return int - port number on which the ModuleManager listens
   */
  public int getModuleManagerPort()
  {
    return MMport;
  }
 
  /**
   * return the port on which the ConnectionManager for this module listens
   * @return int - port number on which the ConnectionManager listens
   */
  public int getConnectionManagerPort()
  {
    return CMport;
  }
  
  /**
   * test stub
   */
  public static void main(String[] args)
  {
    MetaModule mm1 = new MetaModule("hello");
    ModuleRepresentation mRep = new ModuleRepresentation("serval",mm1,1040,3000);
    
    if(args[0].equals("params")) {
      System.out.println("Host is: " + mRep.getHostName());
      System.out.println("MM port is: " + mRep.getModuleManagerPort());
      System.out.println("CM port is: " + mRep.getConnectionManagerPort());
    }
    else if(args[0].equals("meta")) 
      mRep.toStream(System.out);
  }
}
