package webflow.backend;

import java.io.*;
import java.net.*;
import java.util.*;

/**
 * ServerAddress is a class that hides the details of getting the socket 
 * address of a server through its URL. It is used by the SessionManager
 */
/* This class is made public only so that it can appear in JavaDoc. It
   should NOT be public. The user will never see it.
*/
public class ServerAddress {
  // host name and port number of the server
  String hostName;
  int portNumber;

  /**
   * take the URL of the server, get a response, and extract the host name and
   * port number from the response
   * @param serverURL - String that gives the URL access point of the server
   */
  public ServerAddress(String serverURL) throws IOException {
    try {
      URL response = new URL(serverURL);
      DataInputStream dis = new DataInputStream(response.openStream());
      hostName = (String)dis.readLine();
      portNumber = Integer.parseInt((String)dis.readLine());
    }
    catch(Exception e) {
      System.out.println("IOException in ServerAddress ");
      throw new IOException("Unable to get server address from URL " + 
			    serverURL);
    }
  }

  /**
   * return the host name 
   * @return String - host name part of the address
   */
  public String getHostName() {
    return hostName;
  }

  /**
   * return the port number
   * @return int - port number of the address
   */
  public int getPortNumber() {
    return portNumber;
  }

  /* test stub */
  public static void main(String [] args) {
    try {
      ServerAddress sa = new ServerAddress("http://osprey1.npac.syr.edu:9080/mm.html");
      System.out.println("Host name: " + sa.getHostName());
      System.out.println("Port number: " + sa.getPortNumber());
    }
    catch(IOException ioe) {
      System.out.println("Exception occured: " + ioe);
    }
  }
}
