package webflow.backend;

import javax.servlet.*;
import java.net.*;
import java.util.*;
import java.io.*;

// the import uci.graphedit.webflow.*; is needed for the ModuleSourceList class
import webflow.frontend.*;
import webflow.backend.WFSession.*;
import webflow.util.*;

/**
 * SessionManager is a jeeves Servlet. It maintains a representation of
 * the data flow diagram, greated with the FrontEnd. It interprets user 
 * commands from the FrontEnd into mapping of the Modules on physical servers,
 * send initialize Module requests to the ModuleManager at appropriate server,
 * send initialize connection between ports to ConnectionManager, starts the
 * execution and  receives notification for the trafic in the created network. 
 * 
 * In the prototype, the SessionManager also doubles as a viewer manager. 
 * When viewer modules are created, they return a string containing HTML code
 * for loading up an applet. The SessionManager stores that string, and issues
 * a key towards it. The key is used by the FrontEnd to retrieve the applet 
 * code

 */
/* This class is made public only so that it can appear in JavaDoc. It
   should NOT be public. The user will never see it. 
*/
public class SessionManager extends GenericServlet  implements Runnable{
  
  //Hashtable of proxymodules indexed by the module number
  Hashtable ProxyML; 

  Hashtable UserInfo;
  int NumOfClients;
  
  //Server socket on which the SessionManager will listen for incoming requests
  ServerSocket listener;
  
  //HostName and port number on which the SessionManager is l istening
  String hostName;
  int portNumber;

  //thread which will service the listener socket
  Thread socketThread;

  //buffer of viewer URLs
  ViewerList viewerList;

  //list of modules and hosts on which they exist
  ModuleSourceList moduleList;

  //url of the list of modules and hosts
  String moduleUrl;

  // WorkerPool
  WorkerPool myWorkers = null;

  // diagnostic object
  Diagnostic diag;

  // diagnostics on or off (true = on, false = off)
  static final boolean DIAGNOSTICS_ACTIVE = true;
  
  static final boolean DEBUG = true;

  /**
   * initialize the SessionManager. It initialize a thread that will respond 
   * to FrontEnd calls, initialize the data structure for the  network,
   * initialize the sockect for outgoing commands, and socket for incomming
   * notification
   * @param initialParameters : ServletStub that contains at least one 
   *                            parameter: moduleFileURL - a URL towards the
   *                            file of all modules
   */
  public void init(ServletConfig initialParameters)
       throws ServletException
  {
    // the module file contains the descriptions, names, and hosts on which
    // the modules in the system reside
    System.out.println("NEW Session");
    super.init(initialParameters);
    moduleUrl = initialParameters.getInitParameter("ModuleFileURL");
    System.out.println(" moduleUrl : "+moduleUrl);
    moduleList = new ModuleSourceList();
    System.out.println("NEW Session1");
    // for now just ignore whether the parse method worked or not
    moduleList.parse(moduleUrl);
    System.out.println("NEW Session2");
    // now initialize the server socket, thread, 
    listener = null;
    socketThread = null;  
    ProxyML = new Hashtable();
    UserInfo = new Hashtable();
    NumOfClients = 0;
    viewerList = new ViewerList();
    System.out.println("NEW Session3");
    // initialize the diagnostic object
    diag = new Diagnostic("sm.diag");
    diag.activate(DIAGNOSTICS_ACTIVE);
    diag.write("Activated diagnostics object");

    try
      {
	// get local host name
System.out.println("Session4");
	InetAddress local = java.net.InetAddress.getLocalHost();
System.out.println("Session5");
	hostName = local.getHostName();
	
	// open new server socket
	//listener = new ServerSocket(9703);
	listener = new ServerSocket(0);       // TIMUCIN
	portNumber = listener.getLocalPort();
	
	// finally, branch new thread, set it up as a deamon (to terminate
	// upon completion of master program), and start it up
	socketThread = new Thread(this);
	socketThread.setDaemon(true);
	socketThread.start();
      }
    // for now: nothing I can do with exceptions. 
    // Leave things as they are initialized (to null values)
    catch(UnknownHostException e)
      {
	hostName = "0";
      }
    catch(IOException e)
      {
      }

      /** Define a WorkerPool with 60 thread */

      myWorkers = new WorkerPool(60);
  }

   /**
   * service method is called when the servlet is refered by URL. It locates
   * a command on the command line (the default command is get_address), and
   * attempts to serve it. 
   * @param req: a ServletRequest wich contains the parametars of the request
   * @param res: a ServletResponse, an object that contains the response of the
   *             servlet.
   */
  public void 
  service(ServletRequest req, ServletResponse res) 
         throws ServletException,IOException
  {
    String command = req.getParameter("cmd");

    if(command == null || command.equals("get_address"))
      service_address(res);
    else if(command != null && command.equals("start_session"))
      service_address(res);
    else if(command != null && command.equals("end_session"))
      service_sessionTerminate(req,res);
    else if(command != null && command.equals("get_viewer"))
      service_viewer(req, res);
    else if( command.equals("run_modules") || command.equals("destroy_modules") ||
             command.equals("stop_modules") || command.equals("new_module") ||
             command.equals("destroy_module") || command.equals("connect") )
         service_command(req, res);
    else
      service_unknown(res, command);
    
    System.out.println(command+" is being served by ME !! ");

  }// end of service()

  /**
   * Command received on URL was get_address. This method defines 
   * the response of the servlet, i.e. the address of the socket on 
   * which this servlet will communicate with the rest of the world.
   * @param res: a ServletResponse, an object that contains the response of the
   *             servlet.
   */
  void service_address(ServletResponse res)
  {
    if(DEBUG == true)
        System.out.println("I am in service_address ");

    res.setContentType("text/html");

    try
    {
       /** Increment the current number of clients */
       NumOfClients++;

       /** Define a new session for this user */
       WFlowSession Ses = new WFlowSession(moduleUrl);

       /** Put this user-session par info to hashtable */
       UserInfo.put(String.valueOf(NumOfClients),Ses);

       /** Prepare the Servlet output */
       OutputStream os = res.getOutputStream();
       PrintStream out = new PrintStream(os);
       out.println(hostName);
       out.println(portNumber);
       out.println(moduleUrl);
       out.println(String.valueOf(NumOfClients));

       if(DEBUG == true )
       {  System.out.println("hostName :"+hostName);
          System.out.println("portNumber : "+Ses.getPortNumber());
          System.out.println("moduleUrl : "+moduleUrl);
          System.out.println("ClientID : "+String.valueOf(NumOfClients));
       }// end of if

       out.flush();

    } catch(java.io.IOException e)
      {  System.out.println("IOException in service_address() : "+e);
         e.printStackTrace();
      }

  }// end of service_address()

  /**
   * Command received on URL was get_address. This method defines 
   * the response of the servlet, i.e. the address of the socket on 
   * which this servlet will communicate with the rest of the world.
   * @param req: a ServletRequest wich contains the parametars of the request
   * @param res: a ServletResponse, an object that contains the response of the
   *             servlet.
   */
  void service_viewer(ServletRequest req, ServletResponse res)
  {
    // set up the response
    if(DEBUG==true) System.out.println(" I am in service_viewer ");
    res.setContentType("text/html");
    try
    {  OutputStream os = res.getOutputStream();
       PrintStream out = new PrintStream(os);

       /** try to get the key from the request. */ 
       String reqKey = req.getParameter("viewerID");

       /** get the user ID info from the request */ 
       String usrKey = req.getParameter("ClientID");

       /** if it succeeded, get the string from viewerList,  */
       /** and put it to output                              */
       /** otherwise leave the output blank                  */
       if( (reqKey != null) && (usrKey != null) ) 
       {  
          /** look for the user's session object */
          WFlowSession ses = (WFlowSession) UserInfo.get(new String(usrKey));

          if(ses != null) 
          {  if(DEBUG==true) System.out.println("Session is found ");

             /** Get the presentation of the object */
             String viewer = ses.getHTMLofViewer(reqKey);

             if(viewer!=null) out.println(viewer);
             else 
              {  if(DEBUG==true) System.out.println("viewer is NULL");
                 out.println("viewer is not defined/unknown!");
              }
          }
          else 
           {  if(DEBUG==true) 
                System.out.println("Session not found ");
              out.println("Session is not found ");
           }
       }
       else 
        {  if(DEBUG==true) 
             System.out.println("userID or viewerID is not appropriate! ");
           out.println("userID or viewerID is not appropriate! ");
        }

       /** finish up the response */
       out.flush();

    } catch(java.io.IOException e)
      { System.out.println("IOException in service_viewer() of :"+ e); 
        e.printStackTrace();
      }

  }// end of service_viewer()

  /**
  * Command received on URL was get_address. This method defines 
  * the response of the servlet, i.e. the address of the socket on 
  * which this servlet will communicate with the rest of the world.
  * @param req: a ServletRequest wich contains the parametars of the request
  * @param res: a ServletResponse, an object that contains the response of the
  *             servlet.
  */
  void service_command(ServletRequest req, ServletResponse res)
  {
    /** set up the response */
    if(DEBUG==true) 
       System.out.println(" I am in service_command ");

    res.setContentType("text/html");

    try
    { 
       OutputStream os = res.getOutputStream();
       PrintStream out = new PrintStream(os);

       /** try to get the key from the request. */ 
       String command = req.getParameter("cmd");

       /** get the user ID info from the request */ 
       String usrKey = req.getParameter("ClientID");

       /** if it succeeded, get the string from viewerList,  */
       /** and put it to output                              */
       /** otherwise leave the output blank                  */
       if( usrKey != null ) 
       {  
          /** look for the user's session object */
          WFlowSession ses = (WFlowSession) UserInfo.get(new String(usrKey));

          if(ses != null) 
          {  
             if(DEBUG==true) 
                System.out.println("Session is found ");
              
             SessionServices srv = null;
             
             /** Let's parse the service request */
             if(command.equals("run_modules"))
               srv = new ServiceRunAllModules(ses);
             else if(command.equals("destroy_modules"))
               srv = new ServiceDestroyAllModules(ses);
             else if(command.equals("stop_modules"))
               srv = new ServiceStopAllModules(ses);
             else if(command.equals("new_module"))
                  { 
                    String moduleName = req.getParameter("name");
                    if(moduleName!=null) 
                       srv = new ServiceNewModule(moduleName,ses);
                    else
                       out.println("ERROR : Invalid Module Name");
                  }  
             else if(command.equals("destroy_module"))
                  {
                    String str = req.getParameter("string");
                    String val = req.getParameter("value");
                    int    ival = Integer.parseInt(val);
                    if( str != null && val != null )
                      srv = new ServiceDestroyModule(new ID(str,ival), ses);
                    else 
                       out.println("ERROR : Invalid ID "); 
                  } 
             else if(command.equals("connect"))
                  {
                    if(DEBUG==true)
                       System.out.println(" CONNECT is received ");

                    String fromStr = req.getParameter("fromStr");
                    String fromVal = req.getParameter("fromVal");
                    String toStr   = req.getParameter("toStr");
                    String toVal   = req.getParameter("toVal");
                    String mIDfromStr = req.getParameter("mIDfromStr");
                    String mIDfromVal = req.getParameter("mIDfromVal");
                    String mIDtoStr   = req.getParameter("mIDtoStr");
                    String mIDtoVal   = req.getParameter("mIDtoVal");
                    int fromiVal = Integer.parseInt(fromVal);
                    int toiVal   = Integer.parseInt(toVal);
                    int mIDfromiVal = Integer.parseInt(mIDfromVal);
                    int mIDtoiVal   = Integer.parseInt(mIDtoVal);

                    if(DEBUG==true) 
                    {  System.out.println(" fromStr : "+fromStr);
                       System.out.println(" fromVal : "+fromiVal);
                       System.out.println(" toStr   : "+toStr);
                       System.out.println(" toVal   : "+toiVal);
                       System.out.println(" mIDfromStr : "+mIDfromStr);
                       System.out.println(" mIDfromVal : "+ mIDfromiVal);
                       System.out.println(" mIDtoStr   : "+mIDtoStr);
                       System.out.println(" mIDtoVal   : "+mIDtoiVal); 
                    }
                     
                    if( fromStr != null && fromVal != null &&
                        toStr   != null && toVal   != null &&
                        mIDfromStr != null && mIDfromVal != null &&
                        mIDtoStr   != null && mIDtoVal   != null )
                        srv = new ServiceConnect( new ID( fromStr,fromiVal ),
                                                  new ID( toStr,toiVal ),
                                                  new ID( mIDfromStr, mIDfromiVal ),
                                                  new ID( mIDtoStr, mIDtoiVal ), ses );

                   else
                      { out.println("ERROR : Invalid IDs ");
                        if(DEBUG==true)
                            System.out.println(" ERROR : Invalid IDs ");
                      }
                  }
             else 
                out.println("ERROR : Invalid command ");

             /** If we found a service for this rquest */
             if( srv != null )
             {
                Worker mySlave = null;
     
                try 
                {  mySlave =  myWorkers.getWorker();

                
                   /** write OK to the acknowledge */
                   out.println("OK : done");

                   /** write the port number to collect the */
                   /** result of this service               */
                   out.println(srv.getPortNumber());
                   if(DEBUG == true )
                      System.out.println(" PortNumber : "+srv.getPortNumber());
 
             
                   /** Start the service thread */
                   srv.start(); 

                } catch(WorkerPoolFullException wpfe)
                  {  System.out.println(" Error :"+wpfe.getMessage());
                     out.println("ERROR :"+wpfe.getMessage()); 
                  }

             }// end of if

          }
          else 
           {  if(DEBUG==true) 
                System.out.println("Session not found ");
              out.println("ERROR : Session is not found ");
           }
       }
       else 
        {  if(DEBUG==true) 
             System.out.println("userID or viewerID is not appropriate! ");
           out.println("ERROR : userID or viewerID is not appropriate! ");
        }

       /** finish up the response */
       out.flush();

    } catch(java.io.IOException e)
      { System.out.println("IOException in service_viewer() of :"+ e); 
        e.printStackTrace();
      }

  }// end of service_command()

  /**
   * Unknown command received on URL. Return a message with "unknown command" 
   * in it. 
   * @param res: a ServletResponse, an object that contains the response of the
   *             servlet.
   * @param command: String containing the command with which the servlet was
   *                 invoked
   */
  void service_unknown(ServletResponse res, String command)
  {
    res.setContentType("text/html");
    try{
    OutputStream os = res.getOutputStream();
    PrintStream out = new PrintStream(os);
    out.println("Unknown command: " + command);
    out.flush();
    }catch(java.io.IOException e){System.out.println(e);}
  }


  /**
  * Terminates the session related to the given user.
  */
  void service_sessionTerminate(ServletRequest req, ServletResponse res)
  {  
     if(DEBUG==true)
       System.out.println("end_session message is received");

     /** Get ClientID from the request */
     String _usrKey = req.getParameter("ClientID");
     WFlowSession ses = null;

     /** Get Session object from Hashtable */
     if(_usrKey != null)
        ses = (WFlowSession) UserInfo.get(new String(_usrKey));
     
     /** If both of them are OK, then */
     if(_usrKey != null && ses != null )
     {

        /** Remove this user's info from hashtable */
        UserInfo.remove(new String(_usrKey));

        /** Destroy all the running modules */
        /**      in this session.           */
        ses.destroy();

     }// end of if 

     /** Now we need to produce some error message */ 
     /** to take care of this service              */ 
     res.setContentType("text/html");
     try
     {
       OutputStream os = res.getOutputStream();
       PrintStream out = new PrintStream(os);
       if(_usrKey != null && ses != null ) out.println("OK");
       else
         {  if(_usrKey == null) 
               out.println("User identity is not appropriate!!");
            if(ses == null) 
               out.println("There is no session for this user!!");
         }

       out.flush();

     } catch(java.io.IOException e)
       {  System.out.println("IOException in service_sessionTerminate() :"+e);
          e.printStackTrace();
       }

  }// end of service_sessionTerminate()

  /**
   * Provides an information about the servlet.
   * @return String: description of the servlet.
   */
  public String getServletInfo(){
    return "SessionManager servlet";
  }

  /**
   * listen on socket listener ,accept SMrequests, process them,
   * and return SMreply objects.
  */
 public void run()
 {
   boolean finished = false;
   // notice that this loop is in fact an endless loop. It could be made
   // to finish if necessary. 
   while(!finished)
   {
     try 
	 {
	   Socket aSocket = listener.accept();
	   diag.write("received connection");
       System.out.println("received connection");
 
	   SMrequest request = new SMrequest();
	   request.fromStream(aSocket.getInputStream());
	   SMreply reply = serveRequest(request);

	   if(reply != null)
       { diag.write("request served, status = " + reply.getStatus());
	     System.out.println("request served, status = " + reply.getStatus());
       }
       else System.out.println("request served, status = NULL ");

	   if(reply != null)
	     reply.toStream(aSocket.getOutputStream());
	   aSocket.close();
	   diag.write("socket closed");
	   System.out.println("socket closed");

	 } catch(IOException e)
	   { System.out.println("IOException occured in run() ");
         e.printStackTrace();
       }
   }// end of while
 }

 /**
  * serve request to run a module 
  * @return SMreply containing the reply information.
  */
  SMreply runModule()
  {
    SMreply rett = null;
    boolean allSucceeded = true;
    try{
      // for each of the elements in the proxy module list 
      for( Enumeration keys = ProxyML.keys(); keys.hasMoreElements(); ) {
	// get the actual key, and retrieve its module representation
	ID moduleID = (ID) keys.nextElement();
	ModuleRepresentation mr = (ModuleRepresentation)ProxyML.get(moduleID);

	// open a socket towards the corresponding module manager
	Socket s = new Socket(mr.getHostName(),mr.getModuleManagerPort());

	// and send a request to run the module
	MMrequest req = new MMrequest(MMrequest.RUN, moduleID);
	req.toStream(s.getOutputStream());

	// wait for a reply 
	MMreply reply = new MMreply();
	reply.fromStream(s.getInputStream());

	// and if an error occured
	if (reply.getStatus()== MMreply.ERROR) {
	  allSucceeded = false;
	  rett = new SMreply(SMreply.ERROR, reply.getDescription());
	}
	s.close();
      }
    }
    // in case an exception occurs, for now, just ignore the started modules, 
    // and send an 'error occured' message back
    catch(IOException e1) {
      rett = new SMreply(SMreply.ERROR, 
			 "Exception raised while trying to run");
    }

    if(allSucceeded) {
      rett = new SMreply(SMreply.OK, "RAN OK!");
    }

    return rett;
  }

  /**
   * serve request to stop a module
   * @return SMreply containing the reply information
   */
  SMreply stopModule()
  {
    SMreply rett = null;
    boolean allSucceeded = true;
    try{
      // for each of the elements in the proxy module list 
      for( Enumeration keys = ProxyML.keys(); keys.hasMoreElements(); ) {
	// get the actual key, and retrieve its module representation
	ID moduleID = (ID) keys.nextElement();
	ModuleRepresentation mr = (ModuleRepresentation)ProxyML.get(moduleID);

	// open a socket towards the corresponding module manager
	Socket s = new Socket(mr.getHostName(),mr.getModuleManagerPort());

	// and send a request to stop the module
	MMrequest req = new MMrequest(MMrequest.STOP, moduleID);
	req.toStream(s.getOutputStream());

	// wait for a reply 
	MMreply reply = new MMreply();
	reply.fromStream(s.getInputStream());

	// and if an error occured
	if (reply.getStatus()== MMreply.ERROR) {
	  allSucceeded = false;
	  rett = new SMreply(SMreply.ERROR, reply.getDescription());
	}
	s.close();
      }
    }
    // in case an exception occurs, for now, just ignore the started modules, 
    // and send an 'error occured' message back
    catch(IOException e1) {
      rett = new SMreply(SMreply.ERROR, 
			 "Exception raised while trying to run");
    }

    if(allSucceeded)
      rett = new SMreply(SMreply.OK);

    return rett;
  }

  /**
   * serve request to destroy a module
   * @return SMreply containing the reply information.
   */
  SMreply destroyModule()
  {
    SMreply rett = null;
    boolean allSucceeded = true;
    diag.write("destroy module requested");
    try{
      // for each of the elements in the proxy module list 
      for( Enumeration keys = ProxyML.keys(); keys.hasMoreElements(); ) {
	// get the actual key, and retrieve its module representation
	ID moduleID = (ID) keys.nextElement();
	ModuleRepresentation mr = (ModuleRepresentation)ProxyML.get(moduleID);

	diag.write("retrieved a module");

	// open a socket towards the corresponding module manager
	Socket s = new Socket(mr.getHostName(),mr.getModuleManagerPort());

	diag.write("sending a request to destroy the module");
	// and send a request to destroy the module
	MMrequest req = new MMrequest(MMrequest.DESTROY, moduleID);
	req.toStream(s.getOutputStream());

	// wait for a reply 
	MMreply reply = new MMreply();
	reply.fromStream(s.getInputStream());
	diag.write("received reply, status = " + reply.getStatus());

	// and if an error occured
	if (reply.getStatus()== MMreply.ERROR) {
	  allSucceeded = false;
	  rett = new SMreply(SMreply.ERROR, reply.getDescription());
	}
	s.close();
      }
    }
    // in case an exception occurs, for now, just ignore the started modules, 
    // and send an 'error occured' message back
    catch(IOException e1) {
      rett = new SMreply(SMreply.ERROR, 
			 "Exception raised while trying to run");
    }
    catch(Exception e) {
      diag.write("wierd exception raised");
      diag.write("Exception " + e);
    }

    if(allSucceeded)
      rett = new SMreply(SMreply.OK);
      
    // Remove the representations of modules from the SessionManager
    // even if the destruction did not succeed (because of some reason)
    ProxyML.clear();

    return rett;
  }

  /**
   * select one of the servers on the serverList
   * @param serverList - Vector containing a list of servers
   * @return String - one of the Strings in the Vector
   */
  String selectServer(Vector serverList) {
    double random = Math.random();
    int serverListSize = serverList.size();
    int whichServer = (int)Math.round(random * (serverListSize - 1));
    String serverName = (String)serverList.elementAt(whichServer);

    return serverName;
  }

  /**
   * serve request to create a new module
   * @return SMreply containing the reply information.
   */ 
  SMreply newModule(String ModuleName) 
  { 
    SMreply rett = null;

    diag.write("new module request received for module " + ModuleName);
    System.out.println("new module request received for module " + ModuleName);
    // first get a list of servers that have module 'ModuleName' on them
    Vector serverList = moduleList.findServer(ModuleName);
    // and then select one of them
    if(serverList == null)
      return new SMreply(SMreply.ERROR, "No module called: " + ModuleName 
			 + " found in the module list");
    String serverName = selectServer(serverList);
    diag.write("will initialize the module on server " + serverName);
    System.out.println("will initialize the module on server " + serverName);

    // find the socket addresses of the ModuleManager and ConnectionManager
    // on that server
    String hostName = null;
    int MMportNumber = 0;
    int CMportNumber = 0;
    String htmlString= null ;
    String MMurl = "http://" + serverName + "/mm.txt";
    String CMurl = "http://" + serverName + "/cm.txt";
    
    System.out.println("MMurl : "+MMurl);
    System.out.println("CMurl : "+CMurl);

    try {
      ServerAddress MMaddress = new ServerAddress(MMurl);
      ServerAddress CMaddress = new ServerAddress(CMurl);
      hostName = MMaddress.getHostName();
      MMportNumber = MMaddress.getPortNumber();
      CMportNumber = CMaddress.getPortNumber();

      System.out.println("I got the Managers alright ");
      System.out.println("hostName : "+hostName );

      // after the socket addresses are found, make an MMrequest, and send it
      // to the ModuleManager
      Socket toMM = new Socket(hostName, MMportNumber);
      InputStream in = toMM.getInputStream();
      OutputStream out= toMM.getOutputStream();

      System.out.println("I got the sockets  alright ");

      diag.write("sending request to " + MMurl);
      MMrequest request = new MMrequest(MMrequest.INITIALIZE, ModuleName);
      request.toStream(out);

      System.out.println("I send the request  ");

      // receive the MMreply from the ModuleManager
      MMreply reply = new MMreply();
      if(reply.fromStream(in)) {

	diag.write("received a reply with status " + reply.getStatus());
	System.out.println("received a reply with status " + reply.getStatus());

	// if there is an error, just return an error inside the SMreply
	if(reply.getStatus() == MMreply.ERROR)
	  rett = new SMreply(SMreply.ERROR, reply.getDescription());
	else {

	  // if not, extract the MetaModule, ModuleID, the (possibly empty)
	  // HTML string, and all the ports
	  MetaModule mm = reply.getMetaModule();
	  ID moduleID = mm.getModuleID();
	  Vector portStorage = new Vector();
	  	  int htmlKey = 0;
		  if(mm.getHTML()!= null){
 htmlString = new String("<P><APPLET CODEBASE =\""+"http://"+serverName+"/webflow\" ");
 htmlString+=mm.getHTML();
System.out.println("String = "+htmlString);
		  }
	  // extract the ports
	  ID portID = mm.getFirstPortID();
	  while(portID != null) {
	    portStorage.addElement(portID);
	    portID = mm.getNextPortID();
	  }
	  
	  diag.write("making new module representation for the module");
	  System.out.println("making new module representation for the module");

	  // make a module representation from the MetaModule
	  ModuleRepresentation moduleRep = new 
	    ModuleRepresentation(hostName, mm, MMportNumber, CMportNumber);

	  // store the module representation
	  ProxyML.put(moduleID, moduleRep);

	  // if the module needs an HTML called, store that HTML
	  if(htmlString != null) {
	    htmlKey = viewerList.addViewer(htmlString);
	    diag.write("viewer module was returned. Key is " + htmlKey);
	  }

	  // and finally, create a reply 
	  if(htmlString == null)
	    rett = new SMreply(SMreply.OK, moduleID, portStorage);
	  else
	    rett = new SMreply(SMreply.HTML, moduleID, portStorage, htmlKey);

	  diag.write("reply composed");
	  System.out.println("reply composed");
	}
      }
      
    }
    // do nothing, i.e. return a null SMreply
    catch(IOException ioe) {
      System.out.println("IOException in newModule() ");
      ioe.printStackTrace();
    }

    return rett;
  }
  
  /**
   * serve request to connect port ID by from & to
   * @param from - ID of the port from which the connection will start
   * @param to - ID of the port to which the connection will go
   * @param mIDfrom - ID of the module that contains port 'from'
   * @param mIDto - ID of the module that contains port 'to'
   * @return SMreply containing the reply information.
   */
  SMreply Connect(ID from, ID to, ID mIDfrom, ID mIDto)
  {
    SMreply rett = null;

    // get the module representations for the two moduleIDs
    ModuleRepresentation mrFrom = (ModuleRepresentation) ProxyML.get(mIDfrom);
    ModuleRepresentation mrTo =(ModuleRepresentation) ProxyML.get(mIDto);
    diag.write("from host "+mrFrom.getHostName()+" "+"port ="+mrFrom.getConnectionManagerPort()+"to host "+ mrTo.getHostName()+"port ="+mrTo.getConnectionManagerPort());
     try{
      // open a connection and send a request to a connection manager 
      // (i.e. to the connection manager on which the 'from' port resides
      Socket socket = new Socket(mrFrom.getHostName(),
				 mrFrom.getConnectionManagerPort());
      CMrequest req = new CMrequest(CMrequest.ESTABLISH, from, to, 
				    mrTo.getHostName(),"here",
				    mrTo.getConnectionManagerPort(),0);
      req.toStream(socket.getOutputStream());

      // receive the reply
      CMreply reply = new CMreply();
      reply.fromStream(socket.getInputStream());
      if (reply.stat == CMreply.ERROR)
	rett = new SMreply(SMreply.ERROR, reply.getDescription());
      else
	if (reply.stat == CMreply.OK)
	  rett = new SMreply(SMreply.OK);
      socket.close();
    }
    catch (IOException e) {
    }

    return rett;
  }
  
  /**
   * serve the request 
   * @param req :SMrequest being served
   * @return SMreply to the SMrequest.
   */
  SMreply serveRequest(SMrequest req)
  {
    SMreply reply = null;
    int command = req.getCommand();

    diag.write("received command " + command);
    System.out.println("received command " + command);
    // call the appropriate service method, depending on the command in 
    // the request
    switch(command)
      {
      case SMrequest.RUN:
	reply = runModule();
	break;
      case SMrequest.STOP:
	reply = stopModule();
	break;
      case SMrequest.DESTROY:
	reply= destroyModule();
	break;
      case SMrequest.NEW_MODULE:
	reply = newModule(req.getModuleName());
	break;
      case SMrequest.CONNECT:
	reply = Connect(req.getFrom(), req.getTo(), req.getModuleID1(),
			req.getModuleID2());
	break;
      default:
	reply = new SMreply(SMreply.ERROR ," Command not received");
	break;
      }
    diag.write("finished serving command " + command);
    System.out.println("finished serving command " + command);
    return reply;
  }
  
  /* Test stub
     Commented out, since it relies on stuff not in the release
  public static void main(String args[])
  {
    // make a new session manager
    SessionManager sm = new SessionManager();
    Testing6 initParam = new Testing6("http://naos.npac.syr.edu:8765/modules.txt");
    try {
      sm.init((ServletStub)initParam);
    }
    catch(Exception e) {
      System.out.println("Exception " + e);
    }
    
    // now try the making of a new module
    SMreply reply = sm.newModule("Trying");
    if(reply == null)
      System.out.println("Null reply from newModule");
    else {
      
      System.out.println("Not null reply from newModule. " + 
			 "Can't interpret for now");
    }
  }
  */
}
