package webflow.backend;

import java.io.*;
import java.util.*;

/**
 * ViewerList is used to store URLs towards the viewer modules in the system. 
 * It acts like a hashtable, with the additional property that it can find an
 * unused key. 
 */
/* This class is made public only so that it can appear in JavaDoc. It
   should NOT be public. The user will never see it.
*/
public class ViewerList extends Object {
  // list of URLs currently buffered
  Hashtable list;

  // last key selected
  int lastKey;

  /** 
   * constructor for ViewerList. Makes a new hashtable, and initializes the 
   * lastKey to be 0
   */
  public ViewerList() {
    list = new Hashtable();
    lastKey = 0;
  }

  /**
   * find an unused key in the ViewerList
   * @return int - an unused key
   */
  private int unusedKey() {
    lastKey++;
    return(lastKey);
  }

  /**
   * add a new viewer URL to the list. 
   * @param viewerString String to store in the ViewerList
   * @return int - key of the string just stored
   */
  public int addViewer(String viewerString) {
    // first generate the new key
    int key = unusedKey();
    Integer Key = new Integer(key);

    // then store the string under the key
    list.put(Key, viewerString);

    return key;
  }

  /**
   * return a string stored by a given key. Remove that string from the list
   * @param viewerKey: int - key for which to search
   * @return String - string stored for the key 'viewerKey'
   */
  public String getViewer(int viewerKey) {
    Integer key = new Integer(viewerKey);
    String rett = null;
    try {
      rett = (String)list.get(key);
      list.remove(key);
    }
    catch (Exception e) {
      rett = "";
    }
    return rett;
  }

  /* test stub */
  public static void main(String argv[]) {
    ViewerList vl = new ViewerList();

    int key1 = vl.addViewer("blah1");
    int key2 = vl.addViewer("blah2");
    int key3 = key2 + 1000;
    System.out.println("key2=" + key2 + ". vl(key2) = " + vl.getViewer(key2));
    System.out.println("key1=" + key1 + ". vl(key1) = " + vl.getViewer(key1));
    System.out.println("key3=" + key3 + ". vl(key3) = " + vl.getViewer(key3));
    System.out.println("key1=" + key1 + ". vl(key1) = " + vl.getViewer(key1));
  }
}
