package webflow.frontend.WFtree;

import java.lang.*;
import java.util.*;
import java.awt.*;
import uci.graphedit.*;


public class 
TreeNode 
{
  /** Vector of TreeNode */
  private Vector children = null;

  /** encoded node position in tree */
  private String info = null; 

  /** */
  private boolean is_selected = false;

  /** */
  private boolean is_open     = false;

  /** x,y will be set by TreeView */
  private int x,y;	  

  /** width,height should be set in subclass */
  private int width,height;     

  /** */
  private Color selectedColor = Color.red;

  /** */
  private Color unSelectedColor = Color.blue;

  /** */
  private Action myAction = null;
  

  protected void 
  finalize() throws Throwable 
  {
	System.out.println("finalize("+toString()+")");
  }

  /** */
  public 
  TreeNode() 
  {
    info = new String(""); // root 
    children = new Vector(3);

    /** default setting  */
    x = 0 ;
    y = 0 ;
    width = 40;
    height = 20;
    myAction = new ActionOpenSubtree(this);

  }// end of constructor
  

  /** override this to show your own data presented by node */
  public void 
  draw(Graphics g) 
  {
    g.setColor( (is_selected==true?Color.red:Color.blue) );

    String label;

    if ((!is_open) && (!isLeaf())) label = new String("[+]"+info);
    else label = new String(info);

    g.drawString(info,x+5,y+height-5);

    g.drawRect(x,y,width,height);

  }// end of draw
  

  /** 
  * Is the given point is inside the 
  * this components area. 
  */
  public boolean 
  inside(int xc,int yc) 
  {
    return ( (xc>=x) && (xc<=x+width) && (yc>=y) && (yc<=y+height) );
  }// end of inside()
  

  /** */
  public String 
  toString() 
  { return info; }

  /** */
  public void
  setLabel(String _label)
  { info = new String(_label); }

  /** */
  public String
  getLabel()
  { return new String(info); }
  
  /** */
  public void 
  setSelectedColor(Color nClr) 
  { selectedColor = nClr; }

  /** */
  public void
  setUnSelectedColor(Color nClr)
  { unSelectedColor = nClr; }

  /** */
  public Color
  getSelectedColor()
  { return selectedColor; }

  /** */
  public Color
  getUnSelectedColor()
  { return unSelectedColor; }

  /** */
  public void
  setPosition(Point npnt)
  { x=npnt.x; y=npnt.y; }

  /** */
  public void
  setPosition(int nwx, int nwy )
  { x=nwx; y=nwy; }

  /** */
  public Point
  getPosition()
  { return new Point(x,y);}

  /** */
  public void
  setWidth(int _width)
  { width = _width; }

  /** */
  public int
  getWidth() 
  { return width; }

  /** */
  public void
  setHeight(int _height)
  { height = _height; }

  /** */
  public int
  getHeight()
  { return height; }


  /** */
  public int 
  numberChilds() 
  { return children.size(); }
  
  /** */
  public TreeNode 
  getChild(int i) 
  { return (TreeNode) children.elementAt(i); }

  /** */
  public Vector
  getChilds()
  { return children; }
  
  /** */
  public boolean 
  isLeaf() 
  { return (children.isEmpty()); }
  
  /** */
  public boolean 
  isSelected() 
  { return is_selected; }
  
  /** */
  public void 
  select() 
  { is_selected=true; }
  
  /** */
  public void 
  unSelect() 
  { is_selected=false; }
  
  /** 
  * If the node is not a Leaf node,
  * then it can be Opened. Otherwise
  * it is out of the question.
  */
  public void 
  open() 
  { is_open = (isLeaf()==false)&&true; }

  /** */
  public void 
  unOpen() 
  {
    is_open = false;

    Enumeration ele = children.elements();

    if(ele!=null)
    {
       while( ele.hasMoreElements() )
       {  TreeNode a = (TreeNode) ele.nextElement();
          if(a.isOpen()) a.unOpen();
       }// end of while

    }// end of if

  }// end of unOpen()

  /** */
  public boolean 
  isOpen() 
  { return is_open; }

  /** */
  public void 
  close() 
  { unOpen(); }

  /** */
  public void 
  addChild(TreeNode n) 
  {
    int s = children.size();
    n.info = info.concat(String.valueOf(s));
    // System.out.println("TreeNode::addChild() - adding child"+n);
    children.addElement(n);
  }
  
  
  /** */
  public boolean 
  isChild(TreeNode n) 
  { return children.contains(n); }
  
  /**
  * Gets the Action object of this component.
  * @return Action object
  */
  public Action
  getAction()
  {  return myAction; }

  /**
  * Sets the Action of this component.
  * @param  _newAction : A new Action Object 
  */
  public void
  setAction(Action _newAction)
  { myAction = _newAction; }
  
}// end of class TreeNode


