package webflow.frontend;

import java.lang.*;
import java.util.*;
import java.awt.*;
import java.io.*;
import uci.graphedit.*;

/** 
* A DiagramElement that draws arcs between ports. This version
* automatically routes a rectilinear arc. The routing is not very
* good. It avoids the source and sink nodes and no other nodes. It is
* basically case-analysis, and some of the cases are wrong or
* missing. 
*/
public class 
WebArcPerspective extends ArcPerspective 
                  implements UserInterfaceDebug
{

  /** inforation needed for routing */
  protected Vector rects = null ;
  
  /** information needed for routing */
  protected Point lastPoint = null;

  /**
  */
  public
  WebArcPerspective(NetArc _owner)
  { 
     super();
     owner(_owner);
     rects = new Vector();

  }// end of WebArcPerspective()

  /** 
  * Add a routing point by calling addPoint(Point).
  * Called from computeRoute() 
  */
  final void 
  addPoint(int x, int y) 
  { addPoint(new Point(x, y)); }

  /** 
  * Add a routing point. Called from computeRoute() 
  */
  final void 
  addPoint(Point p) 
  {
    if (lastPoint != null) 
    {
      if (lastPoint.x != p.x && lastPoint.y != p.y) 
      {
        addPoint(p.x, lastPoint.y);
        addPoint(p);
        return;

      }// end of if

      Rectangle r = new Rectangle(p);
      r.add(lastPoint);
      rects.addElement(r);

    }// end of if

    lastPoint = p;

  }// end of addPoint()

  /** 
  * forget all old routing info and prepare to recalculate 
  */
  final void 
  resetRectangleBag() 
  {  rects.removeAllElements(); 
     lastPoint = null; 
  }// resetRectangleBag()

  /** 
  * find the route that the arc should follow. Basically case
  * analysis to route around source and destination nodes. <p>
  * needs-more-work: a better algorithm would really be useful. 
  */
  protected void 
  computeRoute() 
  {
    resetRectangleBag();

    Point srcPt = _sourcePortFig.center();
    Point dstPt = _destPortFig.center();
    Rectangle srcRR = _sourcePerspective.routingRect();
    Rectangle dstRR = _destPerspective.routingRect();

    Point mid = new Point( ((srcRR.x + srcRR.width/2) +
			   (dstRR.x + dstRR.width/2)) / 2,
		          ((srcRR.y + srcRR.height/2)+
			   (dstRR.y + dstRR.height/2)) /2);

    NetPort srcPort = (NetPort) _sourcePortFig.owner();
    NetPort dstPort = (NetPort) _destPortFig.owner();
    int srcSector = ((WebPerspective)_sourcePerspective).portSector(srcPort);
    int dstSector = ((WebPerspective)_destPerspective).portSector(dstPort);

    addPoint(srcPt);

    switch( srcSector )
    {
      case 1:
          addPoint( srcPt.x, srcPt.y-10);
      
          if( srcPt.y <  dstPt.y )
          {
             if( srcPt.x < dstRR.x)
             { 
                 addPoint( srcRR.x + srcRR.width +5, srcPt.y-10);
                 addPoint( dstRR.x, srcPt.y - 10);
	         }
             else
              {
                 if( dstPt.x < srcPt.x)
	                 if( dstSector == 4 )
                         addPoint( dstPt.x +10, srcPt.y - 10);
                     else
                         addPoint( dstPt.x, srcPt.y - 10);
                 else
                     addPoint( srcRR.x , srcPt.y - 10);
	
              }	
      
          }
          else
           {
              addPoint(dstRR.x, srcPt.y - 10);
              if( dstSector == 2 )
                  addPoint(dstPt.x, dstPt.y );
              else if( dstSector == 1 || dstSector == 4)
                   {
                       addPoint( dstRR.x, srcPt.y - 10);
                       addPoint( dstRR.x, dstRR.y - 5);
	               }
           }

          break;

      case 2:
          addPoint( srcPt.x - 10, srcPt.y);
          if( srcPt.x < dstPt.x )
          {
            if(srcPt.y < dstPt.y )
            {
               addPoint( srcPt.x -10, srcRR.y + srcRR.height + 5);
               addPoint( dstRR.x,  srcRR.y + srcRR.height + 5);
            }
            else if( srcPt.y < dstPt.y)
                 { addPoint( srcPt.x-10, srcRR.y + srcRR.height + 5); }
	
          }
          else
              if( dstSector == 2 || dstSector == 3)
                   addPoint( dstRR.x, dstRR.y);
          break;

      case 3: 
          addPoint( srcPt.x, srcPt.y +10);
          if( srcPt.y < dstPt.y && srcPt.x <= dstPt.x)
	          addPoint(srcRR.x +srcRR.width +5, srcPt.y +10);
          else if( srcPt.y < dstPt.y && srcPt.x > srcPt.x)
	               addPoint(srcRR.x -5, srcPt.y+10);
          break;

      case 4:
          addPoint( srcPt.x + 10, srcPt.y);
          if( srcPt.x > dstPt.x && srcPt.y <= dstPt.y)
	          addPoint( srcPt.x +10, srcRR.y -5);
          else if( srcPt.x> dstPt.x && srcPt.y > dstPt.y)
	          addPoint( srcPt.x +10, srcRR.y + srcRR.height +5);
          break;

    }// end of switch

    switch( dstSector )
    {
      case 1: addPoint( dstPt.x, dstPt.y - 10);
	          break;
      case 2: addPoint(dstPt.x - 10, dstPt.y);
              break;
      case 3: addPoint(dstRR.x, dstPt.y +10);
              addPoint(dstPt.x, dstPt.y+10);
              break;
      case 4: addPoint(dstPt.x +10, dstPt.y);
              break;

	}// end of switch
  
    addPoint(dstPt);
    _arcFig =  makeArcFig();

  }// end of computeRoute() 

  /** 
  * Overwrite 
  */
  protected Fig 
  makeArcFig()
  {
     FigList ret = new FigList();

     if( rects!=null && rects.isEmpty() == false)
     {  
        /***************************************/
        /** Get all the rectangular components */ 
        /***************************************/
        Enumeration curRect = rects.elements();
     
        /***************************************************/
        /** For each Rectangle definition define a FigRect */
        /** add it to the FigList                          */
        /***************************************************/
        while ( curRect != null && curRect.hasMoreElements()) 
        { 
           Rectangle r = (Rectangle)curRect.nextElement();

           /****************************/
           /** add this to Figure List */
           /****************************/
           ret.addFig( new FigRect(r.x, r.y, r.width+4, r.height+4, Color.blue ) );

        }// end of while

     }// end of if

     return ret;

  }// end of makeArcFig()
         

}// end of class WebArcPerspective 

