
package webflow.frontend;

import java.applet.*;
import java.awt.*;
import java.util.*;
import uci.graphedit.*;
import java.net.*;
import java.io.*;

import webflow.backend.*;


public class 
WebEditor extends Editor 
          implements UserInterfaceDebug 
{
  /** */
  private WebFlowEditor applet;

  /** */
  private Socket server;

  /** */
  private DataInputStream din;

  /** */
  private DataOutputStream dout;

  /** */
  private String hostName;

  /** */
  private int portNumber;

  /** */
  private String moduleListURL;

  /** */
  private String smURL;

  /** */
  private String ClientID = null;

  /**
  *
  */
  public 
  WebEditor(NetList net, Component awt_comp,
		    WebFlowEditor parent, String SM_URL)
  {

    super(net,awt_comp);
   
    applet = parent;

    smURL = SM_URL;
    
    initSocket();

  }//end of constructor
  
  /**
  *  First communication with WebFlowManager to get the session
  *  object, ClientID and ModuleUrl info.
  *  This will enough to carry out the session between client
  *  and the WebFlow. 
  *
  */
  protected void 
  initSocket()
  {
    if(DEBUG==true) 
       System.out.println("init socket");

    try
    {  
       if(DEBUG==true) 
          System.out.println("smURL = " + smURL);

	   URL myDoc = new URL(smURL+"?cmd=start_session");

	   try
	   {  /** get content from remote URL */
	      DataInputStream dis = new DataInputStream( myDoc.openStream() ); 

          hostName  = dis.readLine();
          if(DEBUG==true) 
             System.out.println("Success! hostName = " + hostName);

	      /** now remove host name from the document, and grab port number */
	      String portNumberString = dis.readLine();

	      /** of course, the port Number is interesting, not the string */
	      portNumber = Integer.parseInt(portNumberString);
          if(DEBUG==true) 
	         System.out.println("port number: " + portNumber);

      	  /** open the requested server, get the response and print it */
	      moduleListURL = dis.readLine();
          if(DEBUG==true) 
	         System.out.println("moduleListURL = " + moduleListURL);
          
          /** get ClientID from the incoming  */
          ClientID = dis.readLine();
          if(DEBUG==true) 
	         System.out.println("ClientID = " + ClientID);

	   } catch(ClassCastException e)
	     {  System.out.println("Object received not a string. Raised an exception: " + e); 
            e.printStackTrace();
         }
	     catch(IOException e)
	     {  System.out.println("IO exception occured. " + e); 
            e.printStackTrace();
         }

    } catch(MalformedURLException e)
      {  System.out.println("Malformed URL. Exception " + e + " raised!"); 
         e.printStackTrace();
      }

  }// end of initSocket()

  /**
  * returns the Module List URL address.
  */
  public String 
  getModuleListURL()
  { return moduleListURL; }

  /**
  */
  public void 
  add(DiagramElement de)
  { super.add(de);  }

  /**
  * <ol>
  *  <li> Open an Url connection to the SessionManager Object
  *  <li> Tell her your ClientId, new_module request and the name of the module.
  *  <li> Wait its response.
  *  <li> If the first word of first line is OK, this means that 
  *       she got the message and everything is ok. Then read the
  *       port number from the second line to read the result of the
  *       operation.
  * </ol>
  *
  */
  SMreply 
  requestFromServer(String request)
  {

    SMreply reply = null;
    int current_portNumber = -1;

    if(request==null) return null;

    try
    {
      URL myReq = new URL(request);
      
      try
      {
        DataInputStream dis = new DataInputStream( myReq.openStream() );
        
        String status = dis.readLine();
        StringTokenizer STstatus = new StringTokenizer(status);
        
        System.out.println(" I read status :"+status);

        if( STstatus.countTokens() > 0 &&  
            STstatus.nextElement().equals("OK")  )
        {
          String SportNumber = dis.readLine();
          System.out.println(" I read SportNumber:"+SportNumber+"|");
          
          current_portNumber = Integer.parseInt(SportNumber); 

        } else return reply;

      } catch ( IOException e)
        {
           System.out.println("IOException is raised!");
           e.printStackTrace();
           return reply;
        }

    } catch ( MalformedURLException mue)
      {
        System.out.println(" MalformedURLException is raised!");
        mue.printStackTrace();
        return reply;
      } 
 
    /** If port number is invalid exit */
    if( current_portNumber < 1 ) return reply;

    Socket Csocket = null;
    DataInputStream  Cdin  = null;
    DataOutputStream Cdout = null;

    try
    {
      Csocket = new Socket(hostName, current_portNumber );

      Cdin = new DataInputStream( Csocket.getInputStream() );

      Cdout = new DataOutputStream( Csocket.getOutputStream() );

    } catch( UnknownHostException e)
      {
         System.out.println("UnknownHostException" + e +" raised!");
         e.printStackTrace(); 
         return reply;
      }
      catch( IOException e)
      {
         System.out.println("IOException" + e + " raised!");
         e.printStackTrace(); 
         return reply;
      }
      
   
    reply = new SMreply();

    try
    {
      reply.fromStream(Cdin);

      if(reply.getStatus() == SMreply.ERROR) 
       	System.out.println( reply.getDescription());

    } catch( NullPointerException e)
      {
         System.out.println("NullPointerException vo requestInit" + e);
         e.printStackTrace();
         return null;
      }

    return reply;

  }// end of requestFromServer()

  /**
  * Initialize the module
  */
  public SMreply 
  requestInit(ModuleTreeItem m)
  { 
     String myreq = new String(smURL+"?cmd=new_module&ClientID="+
                               ClientID+"&name="+m.getClassName());

     System.out.println("req(InitModule):"+ myreq);

     return requestFromServer(myreq); 

  }// end of requestInit()


  /**
  */
  public SMreply
  connect(ID from, ID to, ID fromModule, ID toModule )
  {
    String req = new String(smURL+"?cmd=connect&ClientID="+ClientID);
    req += "&fromStr="+from.getString();
    req += "&fromVal="+from.getInt();
    req += "&toStr="+to.getString();
    req += "&toVal="+to.getInt();
    req += "&mIDfromStr="+fromModule.getString();
    req += "&mIDfromVal="+fromModule.getInt();
    req += "&mIDtoStr="+toModule.getString();
    req += "&mIDtoVal="+toModule.getInt();

    return requestFromServer( req );

  }// end of connect()

  /**
  * Runs all the modules in the current session
  */
  public SMreply
  runModules()
  {
     String req = new String(smURL+"?cmd=run_modules&ClientID="+ClientID);

     return requestFromServer(req);

  }// end of run()

  /**
  */
  public SMreply
  refresh()
  {
     String req = new String(smURL+"?cmd=refresh&ClientID="+ClientID);
     SMreply reply = new SMreply();

     try
     {
      URL myReq = new URL(req);
      
      try
      {
        DataInputStream dis = new DataInputStream( myReq.openStream() );
        
        String status = dis.readLine();
        StringTokenizer STstatus = new StringTokenizer(status);
        
        System.out.println(" I read status :"+status);

        if( STstatus.countTokens() > 0 &&  
            STstatus.nextElement().equals("OK")  )
        {
          String SportNumber = dis.readLine();
          System.out.println(" I read SportNumber:"+SportNumber+"|");
          
        } else return reply;

      } catch ( IOException e)
        {
           System.out.println("IOException is raised!");
           e.printStackTrace();
           return reply;
        }

    } catch ( MalformedURLException mue)
      {
        System.out.println(" MalformedURLException is raised!");
        mue.printStackTrace();
        return reply;
      } 
 
     return reply;

  }// end of refresh()


  /**
  * Destroys all the modules in the current session
  */
  public SMreply
  destroy()
  {
     String req = new String(smURL+"?cmd=destroy_modules&ClientID="+ClientID);

     SMreply rett= requestFromServer(req);

     if( rett == null ) 
         rett = new SMreply(SMreply.ERROR,"Unkown");

     return rett ;

  }// end of destroy()

  /**
  * Destroys the given specific module.
  */
  public SMreply 
  destroyModule(ID mdID)
  {
     String req = new String(smURL+"?cmd=destroy_module&ClientID="+ClientID);
     req += "&string="+mdID.getString();
     req += "&value="+ mdID.getInt();
     
     SMreply rett= requestFromServer(req);

     if( rett == null ) 
         rett = new SMreply(SMreply.ERROR,"Unkown");

     Applet receiver = null;
     String name = ""+mdID;
     receiver =  applet.getAppletContext().getApplet(name);

     if( receiver != null ) 
     {  
        System.out.println(" Applet is found ! name:"+name);
        receiver.stop();
     } 
     else 
       {  printApplets(); 
          System.out.println(" Applet is *NOT* found ! name:"+name); 
       }

     return  rett;

  }// end of destroyModule()

  /**
  *
  */
  public void 
  printApplets() 
  {
    /** Enumeration will contain all applets on this page  */
    /** (including this one) that we can send messages to. */
    Enumeration e = applet.getAppletContext().getApplets();

    while (e.hasMoreElements()) 
    {
      Applet applet = (Applet)e.nextElement();
      String info = ((Applet)applet).getAppletInfo();

      if (info != null) 
           System.out.println("- " + info + "\n");  
      else System.out.println("- " + applet.getClass().getName() + "\n");

    }// end of while

  }// end of printApplets()

  /**
  * Opens a window for the visual representation of the module.
  * @param  key A key for this module
  * @param  _WinName Name of the netscape Window
  */
  public void  
  newViewer( int key, String _WinName )
  {
    System.out.println(" WebEditor - newViewer, key = "+ key);
    String url = new String( smURL + "?cmd=get_viewer&viewerID="+key+"&ClientID="+ClientID);

    if(DEBUG==true)
       System.out.println("Viewer URL = " + url);

    applet.showViewer( url , _WinName );

  }// end of newViewer()

  /**
  * Ends the session <br>
  * Does the following processes in order <br>
  * <ol>
  *  <li> Destroy all the running modules
  *  <li> Send the session manager <b>end_session</b> message.
  *  <li> <b>stop()</b> and <b>destroy()</b> the WebFlowEditor applet.
  * </ol>
  */
  public void 
  EndSession()
  {
     /** Destroy all the running modules */
     SMreply reply = destroy();

     if(DEBUG==true)
        System.out.println(" in EndSession() of WebEditor ");

     /** Send "end_session" message to SessionManager */

     try
     {  URL myDoc = new URL(smURL+"?cmd=end_session&ClientID="+ClientID);
        String answer = null;
        try
        {  DataInputStream dis = new DataInputStream( myDoc.openStream());
           answer = dis.readLine();

        } catch(java.io.IOException ioe)
          {  System.out.println(" IOException raised when trying to read ");
             System.out.println(" the file from given URL "+ioe);
             ioe.printStackTrace();
          } 

     } catch(MalformedURLException e)
       {  System.out.println(" MalformedURLException is raised! "+e);
          e.printStackTrace();
       }
	   
     /** stop() and destroy() the WebFlowEditor Applet */
     applet.stop();
     applet.destroy();

  }// end of EndSession();

  /**
  * We better overwrite this method to handle WINDOW_DESTROY event.
  */
  public boolean 
  handleEvent(Event e)
  {
     boolean ret = false;
     Menu  fileM = menu.menuBar.getMenu(0);
     MenuItem quitItem = fileM.getItem(6);
     MenuItem closeItem = fileM.getItem(5);
     Menu  EditM = menu.menuBar.getMenu(1);
     MenuItem disposeItem = EditM.getItem(0);

     if(e.target == disposeItem )
     {  if(DEBUG==true)
        {
          System.out.println("Item is being disposed ");
        }
        
     }
     else
     if(e.target == quitItem || e.target == closeItem)
     { if(DEBUG==true)
          System.out.println("Event.WINDOW_DESTROY is received in WebEditor from quit/close !");

       executeAction(new ActionWebFlowQuit(this),e);
       ret = true;
     }
     else
     if( e.id == Event.WINDOW_DESTROY )
     {  if(DEBUG==true) 
           System.out.println("Event.WINDOW_DESTROY is received in WebEditor !");
        executeAction(new ActionWebFlowQuit(this),e);
        ret = true;
     }// end of if

     return ret || super.handleEvent(e);

  }// end of handleEvent()

  /**
  * Overwrites the <b>toString()</b> message of <b>Object</b>
  */
  public String
  toString()
  { return new String("WebEditor"); }

  /**
  *
  */
  public void
  remove(DiagramElement de)
  {  
    if(DEBUG==true)
       System.out.println("in remove(DiagramElement de) of WebEditor");
    if(de instanceof Perspective)
    {
       System.out.println("in remove is Perspective ");
       WebNode arg = (WebNode) ((Perspective)de).owner();
       System.out.println(" Name : "+arg.getName());
       System.out.println(" Serve: "+arg.getServer());
       System.out.println("  ID  : "+arg.getID());
       destroyModule(arg.getID());
    } 
    super.remove(de);
  }// end of remove()

  public boolean mouseDrag(Event e, int x, int y) {
    System.out.println("Mouse Drag");
    return super.mouseDrag(e,x,y);
  }
}// end of class WebEditor
