
package webflow.frontend;

import java.applet.*;
import java.awt.*;
import java.util.*;
import uci.util.*;
import java.net.*;
import java.io.*;
import uci.graphedit.*;

import webflow.frontend.WFtree.*;


/**
* This object handles to user interface of WebFlow.
*
*
*/
public class 
WebFlowEditor extends Applet 
              implements UserInterfaceDebug 
{

  ////////////////////////////////////////////////////////////////
  // Instance Variables
  
  protected static boolean _spawnFrame = true;
  protected static boolean _spawnPalette = true;
  protected static int _drawAreaWidth = 400;
  protected static int _drawAreaHeight = 300;
  protected static String _loadDocument;

  /** The Editor window */
  private WebEditor wed;

  /** The net-level model to edit */
  private NetList net;

  /** example palette */
  Palette examplePalette;

  /** The module palette, shows nodes listed in ModuleList */
  PaletteTop modulePalette;

  /** The palette of shapes and selection tool */
  Palette shapePalette;

  /** The palette of attributes for shapes. Needed because applets
  /** cannot have menus when embedded in the browser page. */
  Palette attrPalette;

  /** The overall palette window */
  Palette masterPalette;

  /** The overall palette window */
  PaletteTop topPalette;

  /** SessionManager's URL address */
  String SM_URL;

  /** Module List */
  ModuleTree ml;
  ControlPanel control;  
  gefFrame     cntFrame;
  int winNum = 0;


  ////////////////////////////////////////////////////////////////
  // Constructors
  
  /** Construct a new Example */
  public WebFlowEditor() 
  {
    System.out.println("Creating WebFlowEditor");
    net = new NetList();
    net.name("WebFlow Editor Network");
  }


  ////////////////////////////////////////////////////////////////
  // Main

  /** If this is run as an application, basically do the same thing as
   * the applet */
  public static void main(String[] args) {
    WebFlowEditor wfe = new WebFlowEditor();
    wfe.init();
  }

  ////////////////////////////////////////////////////////////////
  // Argument and Parameter Methods

  public void parseParams(Applet applet) {
    String vendor = System.getProperty("java.vendor");
    System.out.println("vendor='" + vendor + "'");
    System.out.println("parsing params");
    if ("true".equals(applet.getParameter("ShouldPreLoad")) &&
	!("Netscape Communications Corporation". equals(vendor) ||
	   "Sun Microsystems Inc.". equals(vendor))) {
      Vector packs = new Vector();
      packs.addElement("uci.graphedit.PackageGraphEdit");
      System.out.println("preloading!!!!");
      Preloader.loadPackages(packs);
    }
    _drawAreaWidth = 400;
    _drawAreaHeight = 300;
    _spawnFrame = "true".equals(applet.getParameter("SpawnFrame"));
    _spawnPalette = "true".equals(applet.getParameter("SpawnPalette"));
    String widthStr = applet.getParameter("DrawAreaWidth");
    if (widthStr != null) _drawAreaWidth = Integer.parseInt(widthStr);
    String heightStr = applet.getParameter("DrawAreaHeight");
    if (heightStr != null) _drawAreaHeight = Integer.parseInt(heightStr);
    _loadDocument = applet.getParameter("LoadDocument");
    System.out.println("SpawnFrame= " + _spawnFrame);
    System.out.println("SpawnPalette= " + _spawnPalette);
    System.out.println("DrawArea= " + _drawAreaWidth + " by " +
		       _drawAreaHeight);
    System.out.println("LoadDocument= " + _loadDocument);

    String smURL = applet.getParameter("SM_URL");
    if(smURL == null) {
	System.out.println("SessionManager URL not defined.");
	stop();
    } else {
	SM_URL = smURL;
	System.out.println("SM_URL= " + smURL); 
    }

  }

  /**
  * Here you prepare the three windows on the screen 
  */
  public void 
  setupWindows() 
  {
    Component awt_comp;
    Dimension drawAreaSize = new Dimension(_drawAreaWidth, _drawAreaHeight);
    awt_comp = new ForwardingPanel(drawAreaSize);
    wed = new WebEditor(net, awt_comp, this, SM_URL);
    //((ForwardingComponent)awt_comp).editor(wed);
    ((ForwardingComponent)awt_comp).setEventHandler(wed);

    if (_spawnFrame) 
    {
      if(DEBUG==true)  System.out.println("spawning frame");
      Frame drawingFrame = new ForwardingFrame(wed, drawAreaSize);
      wed.frame(drawingFrame);
      drawingFrame.add("Center", awt_comp);
      drawingFrame.pack();
      drawingFrame.move(10, 10);
      drawingFrame.setTitle("WebFlow Graph Editor");
      drawingFrame.show();
    }
    else add("Center", awt_comp);

    ml = new ModuleTree("Global Module List", wed.getModuleListURL());
    if(!ml.load()) {
	System.out.println("Loading ModuleList failed");
    }

    PaletteScrollTreeView myTreeView=  new PaletteScrollTreeView(
                        //new ModuleTreeNode((ModuleTreeItem)ml.getItems().firstElement()),350,250, null);
                        wed,ml,350,250, null);

    modulePalette = new PaletteTop(myTreeView); 

    //shapePalette = new PaletteFig();
    shapePalette = new PaletteFigExample();
    attrPalette = new PaletteAttr();

    Vector pals = new Vector();

    //pals.addElement(new PaletteRefreshButton(ml,myTreeView));
    pals.addElement(shapePalette);

    if (!_spawnPalette) pals.addElement(attrPalette);
    
    masterPalette = new PaletteSticky(new PaletteCompound(pals));
    
    Vector pal1 = new Vector();
    pal1.addElement(modulePalette);
    pal1.addElement(masterPalette);

    topPalette = new PaletteTop(new PaletteCompound(pal1));
    topPalette.definePanel();

    if (_spawnPalette) 
    {
      if(DEBUG==true) System.out.println("spawning palette");
      //Frame paletteFrame = new Frame();
      gefFrame paletteFrame = new gefFrame(wed);
      paletteFrame.setTitle("WebFlow Modules ");
      topPalette.frame(paletteFrame);
      modulePalette.frame(paletteFrame);
      myTreeView.setFrame(paletteFrame);

      paletteFrame.add("Center", topPalette);
      paletteFrame.pack();
      paletteFrame.move(60 + drawAreaSize.width, 95);
      paletteFrame.show();
    }
    else add("East", topPalette);

    control = new ControlPanel(wed);
    
    if (_spawnPalette)
    {
      if(DEBUG==true) System.out.println("spawning palette");
      cntFrame = new gefFrame(wed);
      cntFrame.setTitle("WebFlow Control ");
      cntFrame.setLayout( new BorderLayout() );
      cntFrame.add("Center", control);
      cntFrame.pack();
      cntFrame.move(60 + drawAreaSize.width, 10);
      cntFrame.show();

    } else add("South",control);
 
  }

  ////////////////////////////////////////////////////////////////
  // Applet related methods
  

  /** Part of the required Applet API. Does nothing. */
  public void start() {  }

  /** This is called when the Applet/Application starts up. It does
   * preloading if the 'ShouldPreLoad' property is set to true. It
   * also registers some well known Action's as an example. */
  public void init() {
    Globals.setApplet(this);
    parseParams(this);
    setupWindows();
    Action.register(new ActionSave());
    Action.register(new ActionLoad());
    Action.register(new ActionDispose());
    Action.register(new ActionDelete());
    Action.register(new ActionSpawn());
    Action.register(new ActionAdjustGrid());
    Action.register(new ActionEditNode());
    Action.register(new ActionCreateNode("uci.graphedit.demo.SampleNode",
					 null, false));
    Action.register(new ActionQuit());
  }

  /** Part of the required Applet API. Does nothing. */
  public void stop() 
  {  }// end of stop()

  /** When the user closes this window try to free up as many objects
   * as possible. */
  public void destroy() 
  {
    if (null != topPalette) topPalette.close();
    if (null != wed) wed.close();
    if (null != cntFrame) cntFrame.dispose();
    masterPalette = null;
    topPalette = null;
    wed = null;
    net = null;
    modulePalette = null;
    shapePalette = null;
    attrPalette = null;
    control = null;
    ml = null;
  }

  /** reply a breif string that describes this applet in the "About"
   * box. */
  public String getAppletInfo() {
    return "WebFlowEditor is a visual graph editor applet for the WebFlow " +
  	"system at NPAC, based on the GEF (Graph Editing Framework) by " +
	"Jason Robbins at UCI.\n";
  }

  /** Reply an array of strings that describe the HTML PARAM's that
   * can be passed to this applet. */
  public String[][] getParameterInfo() {
    String[][] info = new String[5][3];

    info[0][0] = "LoadDocument";
    info[0][1] = "java.lang.String\n";
    info[0][2] = "If supplied, automatically load the given URL.\n\n";

    info[1][0] = "SpawnFrame";
    info[1][1] = "boolean\n";
    info[1][2] = "Open a new Frame for the drawing area.\n\n";

    info[2][0] = "SpawnPalette";
    info[2][1] = "boolean\n";
    info[2][2] = "Open a new Frame for the palette.\n\n";

    info[3][0] = "DrawAreaWidth";
    info[3][1] = "int\n";
    info[3][2] = "Width of the drawing area in pixels.\n\n";

    info[4][0] = "DrawAreaHeight";
    info[4][1] = "int\n";
    info[4][2] = "Height of the drawing area in pixels.\n\n";

    return info;
  }

  /**
  * Shows the given Url in the given netscape window
  */
  public void 
  showViewer( String viewerURL , String WinName)
  {
    URL url = null;
    try
    {  
       url = new URL( viewerURL );

	   if(DEBUG==true)
          System.out.println(" URL OK!");

       /** Show the url in the given netscape window */
       getAppletContext().showDocument( url, (WinName!=null?WinName : "Viewer") );

    } catch( MalformedURLException e )
      {  System.out.println(" Can not create URL from string "+viewerURL);
         e.printStackTrace();
      }
   
  }// end of showViewer()

} /* end class WebFlowEditor */
