
/*
 * @(#)ImageToolServer.java	1.11 97/02/27
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */
package webflow.modules.demo.collab;

import webflow.modules.demo.image.*;
import SharedData.*;

/**
 * This is just a simple server/client based proof-of-concept
 * implementation to make sure that the Shared Data classes have
 * all the required API in them.
 *
 * It's based on the chat demonstration which is part of the
 * west-coast rmi distribution.
 *
 * @version     0.90 11 February 1997
 * @author      Rich Burridge
 */

public class
ImageToolServer implements ImageToolDebugFlags {

    private static String sessionName = "ImageToolSession";


    public static void
    main(String args[]) {
        ImageToolClient client      = null;
	SharedData.Session    ImageToolSession = null;
	String     url         = null;
	String     sessionType = null;
	String     hostname    = null;
	int        hostport    = 0;

        if (ImageToolServer_Debug) {
            System.out.println("ImageToolServer: main.");
        }

        hostname    = getHost(args);
	hostport    = getPort(args);
	sessionType = getType(args);
	url = "coll://" + hostname + ":" + hostport + "/" + sessionType +
	      "/Session/" + sessionName;

	try {

/* Create a session, [re]bind it and create a channel. */
 
	    ImageToolSession = SessionFactory.createSession(url);
	    ImageToolSession.createChannel("ImageToolChannel", true);
            ImageToolSession.createChannel("ChChannel", true);
            System.out.println("Setup and bound ImageTool server.");
        } catch (SharedDataException e) {
            System.out.println("ImageToolServer: main: shared data exception: " + e);
            if (ImageToolServer_Debug) {
		e.printStackTrace();
	    }
        }
    }


    private static String
    getHost(String args[]) {
        String defHost = "stard";   /* Default host name for connections. */
        int length = args.length;

        if (ImageToolServer_Debug) {
            System.out.println("ImageToolServer: getHost.");
        }

        for (int i = 0; i < length; i++) {
            if (args[i].equals("-server")) {
                if (++i < length) {
                    return(args[i]);
                }
            }
        }
        return(defHost);
    }


    private static int
    getPort(String args[]) {
        int defPort = 4461;   /* Default port number for connections. */
        int length = args.length;

        if (ImageToolServer_Debug) {
            System.out.println("ImageToolServer: getPort.");
        }

        for (int i = 0; i < length; i++) {
            if (args[i].equals("-port")) {
                if (++i < length) {
                    return(Integer.parseInt(args[i]));
                }
            }    
        }
        return(defPort);
    }


    private static String
    getType(String args[]) {
        String defType = "socket";   /* Default Session type. */
        int length = args.length;

        if (ImageToolServer_Debug) {
            System.out.println("ImageToolServer: getType.");
        }

        for (int i = 0; i < length; i++) {
            if (args[i].equals("-type")) {
                if (++i < length) {
                    return(args[i]);
                }
            }    
        }
        return(defType);
    }
}
